@@ -36,7 +36,7 @@ from gajim.common import modules
 from gajim.common import passwords
 from gajim.common.events import AccountConnected
 from gajim.common.events import MessageSent
-from gajim.common.events import AccountDisonnected
+from gajim.common.events import AccountDisconnected
 from gajim.common.events import PasswordRequired
 from gajim.common.events import PlainConnection
 from gajim.common.events import SignedIn
@@ -150,6 +150,9 @@ class Client(Observable):
             return address.to_string().split(':')[0]
         return None
 
+    def is_destroyed(self) -> bool:
+        return self._destroyed
+
     def set_remove_account(self, value: bool) -> None:
         # Used by the RemoveAccount Assistant to make the Client
         # not react to any stream errors that happen while the
@@ -157,16 +160,6 @@ class Client(Observable):
         self._remove_account = value
 
     def _create_client(self) -> None:
-        if self._destroyed:
-            # If we disable an account cleanup() is called and all
-            # modules are unregistered. Because disable_account() does not wait
-            # for the client to properly disconnect, handlers of the
-            # nbxmpp.Client() are emitted after we called cleanup().
-            # After nbxmpp.Client() disconnects and is destroyed we create a
-            # new instance with this method but modules.get_handlers() fails
-            # because modules are already unregistered.
-            # TODO: Make this nicer
-            return
         log.info('Create new nbxmpp client')
         self._client = NBXMPPClient(log_context=self._account)
         self.connection = self._client
@@ -181,8 +174,6 @@ class Client(Observable):
             self.password = passwords.get_password(self._account)
 
         self._client.set_password(self.password)
-        self._client.set_accepted_certificates(
-            app.cert_store.get_certificates())
 
         self._client.subscribe('resume-failed', self._on_resume_failed)
         self._client.subscribe('resume-successful', self._on_resume_successful)
@@ -316,7 +307,7 @@ class Client(Observable):
             self._destroy_client = False
             self._create_client()
 
-        app.ged.raise_event(AccountDisonnected(account=self._account))
+        app.ged.raise_event(AccountDisconnected(account=self._account))
 
     def _on_connection_failed(self,
                               _client: NBXMPPClient,
@@ -349,10 +340,10 @@ class Client(Observable):
                                            stanza=stanza))
 
     def get_own_jid(self) -> JID:
-        """
+        '''
         Return the last full JID we received on a bind event.
         In case we were never connected it returns the bare JID from config.
-        """
+        '''
         if self._client is not None:
             jid = self._client.get_bound_jid()
             if jid is not None:
@@ -381,7 +372,7 @@ class Client(Observable):
                 helpers.to_one_line(message))
 
         if self._state.is_disconnecting:
-            log.warning('Can\'t change status while '
+            log.warning("Can\'t change status while "
                         'disconnect is in progress')
             return
 
@@ -472,9 +463,9 @@ class Client(Observable):
         modules.send_stored_publish(self._account)
 
     def send_stanza(self, stanza: Any) -> None:
-        """
+        '''
         Send a stanza untouched
-        """
+        '''
         return self._client.send_stanza(stanza)
 
     def send_message(self, message: OutgoingMessage) -> None:
@@ -503,6 +494,10 @@ class Client(Observable):
         message.set_sent_timestamp()
         message.message_id = self.send_stanza(message.stanza)
 
+        log_line_id = None
+        if not message.is_groupchat:
+            log_line_id = self.get_module('Message').log_message(message)
+
         app.ged.raise_event(
             MessageSent(jid=message.jid,
                         account=message.account,
@@ -513,13 +508,9 @@ class Client(Observable):
                         label=message.label,
                         correct_id=message.correct_id,
                         message_id=message.message_id,
+                        msg_log_id=log_line_id,
                         play_sound=message.play_sound))
 
-        if message.is_groupchat:
-            return
-
-        self.get_module('Message').log_message(message)
-
     def _prepare_for_connect(self) -> None:
         custom_host = get_custom_host(self._account)
         if custom_host is not None:
@@ -553,6 +544,8 @@ class Client(Observable):
 
         log.info('Connect')
 
+        self._client.set_accepted_certificates(
+            app.cert_store.get_certificates())
         self._client.set_ignored_tls_errors(ignored_tls_errors)
         self._reconnect = True
         self._disable_reconnect_timer()
@@ -570,14 +563,13 @@ class Client(Observable):
 
     def _schedule_reconnect(self) -> None:
         self._set_state(ClientState.RECONNECT_SCHEDULED)
-        log.info("Reconnect to %s in 3s", self._account)
+        log.info('Reconnect to %s in 3s', self._account)
         self._reconnect_timer_source = GLib.timeout_add_seconds(
             3, self._prepare_for_connect)
 
     def _abort_reconnect(self) -> None:
         self._set_state(ClientState.DISCONNECTED)
         self._disable_reconnect_timer()
-        self.notify('state-changed', SimpleClientState.DISCONNECTED)
 
         if self._destroy_client:
             self._client.destroy()
@@ -585,6 +577,8 @@ class Client(Observable):
             self._destroy_client = False
             self._create_client()
 
+        self.notify('state-changed', SimpleClientState.DISCONNECTED)
+
     def _disable_reconnect_timer(self) -> None:
         if self._reconnect_timer_source is not None:
             GLib.source_remove(self._reconnect_timer_source)
@@ -647,12 +641,7 @@ class Client(Observable):
             Monitor.disconnect(self._idle_handler_id)
             app.app.disconnect(self._screensaver_handler_id)
         if self._client is not None:
-            # cleanup() is called before nbmxpp.Client has disconnected,
-            # when we disable the account. So we need to unregister
-            # handlers here.
-            # TODO: cleanup() should not be called before disconnect is finished
-            for handler in modules.get_handlers(self):
-                self._client.unregister_handler(handler)
+            self._client.destroy()
         modules.unregister_modules(self)
 
     def quit(self, kill_core: bool) -> None:
