@@ -27,10 +27,7 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-from typing import Any  # pylint: disable=unused-import
-from typing import Dict  # pylint: disable=unused-import
-from typing import List  # pylint: disable=unused-import
-from typing import Tuple  # pylint: disable=unused-import
+from typing import Any
 
 import re
 import copy
@@ -63,7 +60,7 @@ class Config:
     DEFAULT_MOOD_ICONSET = 'default'
     DEFAULT_ACTIVITY_ICONSET = 'default'
 
-    __options = ({
+    __options: tuple[dict[str, list[Any]], dict[Any, Any]] = ({
         # name: [ type, default_value, help_string, restart ]
         'autopopup': [opt_bool, False],
         'autopopupaway': [opt_bool, False],
@@ -208,9 +205,10 @@ class Config:
         'check_for_update': [opt_bool, True, 'Check for Gajim updates periodically'],
         'last_update_check': [opt_str, '', 'Date of the last update check'],
         'always_ask_for_status_message': [opt_bool, False],
-    }, {})  # type: Tuple[Dict[str, List[Any]], Dict[Any, Any]]
+    }, {})
 
-    __options_per_key = {
+    __options_per_key: dict[str,
+                            tuple[dict[str, list[Any]], dict[Any, Any]]] = {
         'accounts': ({
             'name': [opt_str, '', '', True],
             'account_label': [opt_str, '', '', False],
@@ -314,7 +312,7 @@ class Config:
         'plugins': ({
             'active': [opt_bool, False, 'If enabled, plugins will be activated on startup (this is saved when exiting Gajim). This option SHOULD NOT be used to (de)activate plugins. Use the plugin window instead.'],
         }, {}),
-    }  # type: Dict[str, Tuple[Dict[str, List[Any]], Dict[Any, Any]]]
+    }
 
     statusmsg_default = {
         'Sleeping': ['ZZZZzzzzzZZZZZ', 'inactive', 'sleeping', '', 'sleepy', ''],
@@ -417,7 +415,6 @@ class Config:
             return
 
         self.__options[1][optname] = value
-        self._timeout_save()
 
     def get(self, optname=None):
         if not optname:
@@ -459,7 +456,6 @@ class Config:
         opt[1][name] = {}
         for o in opt[0]:
             opt[1][name][o] = opt[0][o][Option.VAL]
-        self._timeout_save()
 
     def del_per(self, typename, name, subname=None): # per_group_of_option
         if typename not in self.__options_per_key:
@@ -471,7 +467,6 @@ class Config:
         # if subname is specified, delete the item in the group.
         elif subname in opt[1][name]:
             del opt[1][name][subname]
-        self._timeout_save()
 
     def del_all_per(self, typename, subname):
         # Deletes all settings per typename
@@ -485,7 +480,6 @@ class Config:
                 del opt[1][name][subname]
             except KeyError:
                 pass
-        self._timeout_save()
 
     def set_per(self, optname, key, subname, value): # per_group_of_option
         if optname not in self.__options_per_key:
@@ -503,7 +497,6 @@ class Config:
         if value is None:
             return
         obj[subname] = value
-        self._timeout_save()
 
     def get_per(self, optname, key=None, subname=None, default=None): # per_group_of_option
         if optname not in self.__options_per_key:
@@ -593,11 +586,6 @@ class Config:
         self.save_timeout_id = None
         return False
 
-    def _timeout_save(self):
-        if self.save_timeout_id:
-            return
-        self.save_timeout_id = GLib.timeout_add(1000, self._really_save)
-
     def __init__(self):
         #init default values
         self._init_options()
