@@ -125,7 +125,6 @@ class ConfigPaths:
         source_paths = [
             ('DATA', basedir / 'data'),
             ('STYLE', basedir / 'data' / 'style'),
-            ('EMOTICONS', basedir / 'data' / 'emoticons'),
             ('GUI', basedir / 'data' / 'gui'),
             ('ICONS', basedir / 'data' / 'icons'),
             ('HOME', Path.home()),
@@ -189,7 +188,6 @@ class ConfigPaths:
         unique_profile_paths = [
             # Data paths
             ('SECRETS_FILE', 'secrets', PathLocation.DATA, PathType.FILE),
-            ('MY_PEER_CERTS', 'certs', PathLocation.DATA, PathType.FOLDER),
             ('CERT_STORE', 'cert_store', PathLocation.DATA, PathType.FOLDER),
             ('DEBUG', 'debug', PathLocation.DATA, PathType.FOLDER),
             ('PLUGINS_DATA', 'plugins_data',
@@ -200,7 +198,6 @@ class ConfigPaths:
             ('CONFIG_FILE', 'config', PathLocation.CONFIG, PathType.FILE),
             ('PLUGINS_CONFIG_DIR',
              'pluginsconfig', PathLocation.CONFIG, PathType.FOLDER),
-            ('MY_CERT', 'localcerts', PathLocation.CONFIG, PathType.FOLDER),
             ('MY_SHORTCUTS', 'shortcuts.json',
              PathLocation.CONFIG, PathType.FILE),
         ]
@@ -218,8 +215,6 @@ class ConfigPaths:
             ('PLUGINS_IMAGES', 'plugins_images',
              PathLocation.CACHE, PathType.FOLDER),
             ('PLUGINS_USER', 'plugins', PathLocation.DATA, PathType.FOLDER),
-            ('MY_EMOTS',
-             'emoticons', PathLocation.DATA, PathType.FOLDER_OPTIONAL),
             ('MY_ICONSETS',
              'iconsets', PathLocation.DATA, PathType.FOLDER_OPTIONAL),
 
