@@ -15,6 +15,7 @@
 from __future__ import annotations
 
 from typing import Any
+from typing import Optional
 from typing import Union
 from typing import NamedTuple
 
@@ -36,6 +37,8 @@ from gajim.common.i18n import Q_
 STOP_EVENT = True
 PROPAGATE_EVENT = False
 
+MAX_MESSAGE_CORRECTION_DELAY = 300
+
 
 class EncryptionData(NamedTuple):
     additional_data: Any = None
@@ -49,9 +52,8 @@ class Entity(NamedTuple):
 
 
 class RowHeaderType(IntEnum):
-    ACTIVE = 0
-    CONVERSATIONS = 1
-    PINNED = 2
+    CONVERSATIONS = 0
+    PINNED = 1
 
 
 class AvatarSize(IntEnum):
@@ -111,16 +113,6 @@ class KindConstant(IntEnum):
         return str(self.value)
 
 
-@unique
-class ShowConstant(IntEnum):
-    ONLINE = 0
-    CHAT = 1
-    AWAY = 2
-    XA = 3
-    DND = 4
-    OFFLINE = 5
-
-
 @unique
 class TypeConstant(IntEnum):
     AIM = 0
@@ -163,6 +155,7 @@ class StyleAttr(Enum):
 
 @unique
 class CSSPriority(IntEnum):
+    PRE_APPLICATION = 599
     APPLICATION = 600
     APPLICATION_DARK = 601
     DEFAULT_THEME = 610
@@ -223,7 +216,7 @@ class Display(Enum):
 
 
 class URIType(Enum):
-    UNKNOWN = 'unknown'
+    INVALID = 'invalid'
     XMPP = 'xmpp'
     MAIL = 'mail'
     GEO = 'geo'
@@ -233,10 +226,18 @@ class URIType(Enum):
     TEL = 'tel'
 
 
-class URIAction(Enum):
+# https://xmpp.org/registrar/querytypes.html
+class XmppUriQuery(Enum):
+    NONE = ''
     MESSAGE = 'message'
     JOIN = 'join'
-    SUBSCRIBE = 'subscribe'
+
+    @staticmethod
+    def from_str(s: str) -> Optional[XmppUriQuery]:
+        try:
+            return XmppUriQuery(s)
+        except ValueError:
+            return None
 
 
 class MUCJoinedState(Enum):
@@ -246,6 +247,7 @@ class MUCJoinedState(Enum):
     CREATING = 'creating'
     CAPTCHA_REQUEST = 'captcha in progress'
     CAPTCHA_FAILED = 'captcha failed'
+    PASSWORD_REQUEST = 'password request'
 
     def __str__(self):
         return self.name
@@ -274,6 +276,10 @@ class MUCJoinedState(Enum):
     def is_captcha_failed(self):
         return self == MUCJoinedState.CAPTCHA_FAILED
 
+    @property
+    def is_password_request(self):
+        return self == MUCJoinedState.PASSWORD_REQUEST
+
 
 class ClientState(IntEnum):
     DISCONNECTING = 0
@@ -424,38 +430,38 @@ LOCATION_DATA = {
 
 
 SSLError = {
-    2: _("Unable to get issuer certificate"),
-    3: _("Unable to get certificate CRL"),
+    2: _('Unable to get issuer certificate'),
+    3: _('Unable to get certificate CRL'),
     4: _("Unable to decrypt certificate's signature"),
     5: _("Unable to decrypt CRL's signature"),
-    6: _("Unable to decode issuer public key"),
-    7: _("Certificate signature failure"),
-    8: _("CRL signature failure"),
-    9: _("Certificate is not yet valid"),
-    10: _("Certificate has expired"),
-    11: _("CRL is not yet valid"),
-    12: _("CRL has expired"),
+    6: _('Unable to decode issuer public key'),
+    7: _('Certificate signature failure'),
+    8: _('CRL signature failure'),
+    9: _('Certificate is not yet valid'),
+    10: _('Certificate has expired'),
+    11: _('CRL is not yet valid'),
+    12: _('CRL has expired'),
     13: _("Format error in certificate's notBefore field"),
     14: _("Format error in certificate's notAfter field"),
     15: _("Format error in CRL's lastUpdate field"),
     16: _("Format error in CRL's nextUpdate field"),
-    17: _("Out of memory"),
-    18: _("Self signed certificate"),
-    19: _("Self signed certificate in certificate chain"),
-    20: _("Unable to get local issuer certificate"),
-    21: _("Unable to verify the first certificate"),
-    22: _("Certificate chain too long"),
-    23: _("Certificate revoked"),
-    24: _("Invalid CA certificate"),
-    25: _("Path length constraint exceeded"),
-    26: _("Unsupported certificate purpose"),
-    27: _("Certificate not trusted"),
-    28: _("Certificate rejected"),
-    29: _("Subject issuer mismatch"),
-    30: _("Authority and subject key identifier mismatch"),
-    31: _("Authority and issuer serial number mismatch"),
-    32: _("Key usage does not include certificate signing"),
-    50: _("Application verification failure"),
+    17: _('Out of memory'),
+    18: _('Self signed certificate'),
+    19: _('Self signed certificate in certificate chain'),
+    20: _('Unable to get local issuer certificate'),
+    21: _('Unable to verify the first certificate'),
+    22: _('Certificate chain too long'),
+    23: _('Certificate revoked'),
+    24: _('Invalid CA certificate'),
+    25: _('Path length constraint exceeded'),
+    26: _('Unsupported certificate purpose'),
+    27: _('Certificate not trusted'),
+    28: _('Certificate rejected'),
+    29: _('Subject issuer mismatch'),
+    30: _('Authority and subject key identifier mismatch'),
+    31: _('Authority and issuer serial number mismatch'),
+    32: _('Key usage does not include certificate signing'),
+    50: _('Application verification failure'),
 }
 
 
@@ -465,85 +471,88 @@ VOWELS = 'aeiou'
 CONSONANTS = 'bcdfghjklmnpqrstvwxyz'
 
 
-THANKS = u"""\
-Alexander Futász
-Alexander V. Butenko
-Alexey Nezhdanov
-Alfredo Junix
-Anaël Verrier
-Anders Ström
-Andrew Sayman
-Anton Shmigirilov
-Christian Bjälevik
-Christophe Got
-Christoph Neuroth
-David Campey
-Dennis Craven
-Fabian Neumann
-Filippos Papadopoulos
-Francisco Alburquerque Parra (Membris Khan)
-Frederic Lory
-Fridtjof Bussefor
-Geobert Quach
-Guillaume Morin
-Gustavo J. A. M. Carneiro
-Ivo Anjo
-Josef Vybíral
-Juraj Michalek
-Kjell Braden
-Luis Peralta
-Michael Scherer
-Michele Campeotto
-Mike Albon
-Miguel Fonseca
-Norman Rasmussen
-Oscar Hellström
-Peter Saint-Andre
-Petr Menšík
-Sergey Kuleshov
-Stavros Giannouris
-Stian B. Barmen
-Thilo Molitor
-Thomas Klein-Hitpaß
-Urtzi Alfaro
-Witold Kieraś
-Yakov Bezrukov
-Yavor Doganov
-""".strip().split("\n")
+THANKS = [
+    'Alexander Futász',
+    'Alexander V. Butenko',
+    'Alexey Nezhdanov',
+    'Alfredo Junix',
+    'Anaël Verrier',
+    'Anders Ström',
+    'Andrew Sayman',
+    'Anton Shmigirilov',
+    'Christian Bjälevik',
+    'Christophe Got',
+    'Christoph Neuroth',
+    'David Campey',
+    'Dennis Craven',
+    'Fabian Neumann',
+    'Filippos Papadopoulos',
+    'Francisco Alburquerque Parra (Membris Khan)',
+    'Frederic Lory',
+    'Fridtjof Bussefor',
+    'Geobert Quach',
+    'Guillaume Morin',
+    'Gustavo J. A. M. Carneiro',
+    'Ivo Anjo',
+    'Josef Vybíral',
+    'Juraj Michalek',
+    'Kjell Braden',
+    'Luis Peralta',
+    'Michael Scherer',
+    'Michele Campeotto',
+    'Mike Albon',
+    'Miguel Fonseca',
+    'Norman Rasmussen',
+    'Oscar Hellström',
+    'Peter Saint-Andre',
+    'Petr Menšík',
+    'Sergey Kuleshov',
+    'Stavros Giannouris',
+    'Stian B. Barmen',
+    'Thilo Molitor',
+    'Thomas Klein-Hitpaß',
+    'Urtzi Alfaro',
+    'Witold Kieraś',
+    'Yakov Bezrukov',
+    'Yavor Doganov',
+]
 
-ARTISTS = u"""\
-Anders Ström
-Christophe Got
-Dennis Craven
-Dmitry Korzhevin
-Guillaume Morin
-Gvorcek Spajreh
-Josef Vybíral
-Membris Khan
-Rederick Asher
-Jakub Szypulka
-""".strip().split("\n")
 
-DEVS_CURRENT = u"""\
-Yann Leboulanger (asterix AT lagaule.org)
-Philipp Hörist (philipp AT hoerist.com)
-Daniel Brötzmann (wurstsalat AT posteo.de)
-André Apitzsch
-""".strip().split("\n")
+ARTISTS = [
+    'Anders Ström',
+    'Christophe Got',
+    'Dennis Craven',
+    'Dmitry Korzhevin',
+    'Guillaume Morin',
+    'Gvorcek Spajreh',
+    'Josef Vybíral',
+    'Membris Khan',
+    'Rederick Asher',
+    'Jakub Szypulka',
+]
 
-DEVS_PAST = u"""\
-Stefan Bethge (stefan AT lanpartei.de)
-Alexander Cherniuk (ts33kr AT gmail.com)
-Stephan Erb (steve-e AT h3c.de)
-Vincent Hanquez (tab AT snarc.org)
-Dimitur Kirov (dkirov AT gmail.com)
-Nikos Kouremenos (kourem AT gmail.com)
-Julien Pivotto (roidelapluie AT gmail.com)
-Jonathan Schleifer (js-gajim AT webkeks.org)
-Travis Shirk (travis AT pobox.com)
-Brendan Taylor (whateley AT gmail.com)
-Jean-Marie Traissard (jim AT lapin.org)
-""".strip().split("\n")
+
+DEVS_CURRENT = [
+    'Yann Leboulanger (asterix AT lagaule.org)',
+    'Philipp Hörist (philipp AT hoerist.com)',
+    'Daniel Brötzmann (wurstsalat AT posteo.de)',
+    'André Apitzsch',
+]
+
+
+DEVS_PAST = [
+    'Stefan Bethge (stefan AT lanpartei.de)',
+    'Alexander Cherniuk (ts33kr AT gmail.com)',
+    'Stephan Erb (steve-e AT h3c.de)',
+    'Vincent Hanquez (tab AT snarc.org)',
+    'Dimitur Kirov (dkirov AT gmail.com)',
+    'Nikos Kouremenos (kourem AT gmail.com)',
+    'Julien Pivotto (roidelapluie AT gmail.com)',
+    'Jonathan Schleifer (js-gajim AT webkeks.org)',
+    'Travis Shirk (travis AT pobox.com)',
+    'Brendan Taylor (whateley AT gmail.com)',
+    'Jean-Marie Traissard (jim AT lapin.org)',
+]
 
 
 RFC5646_LANGUAGE_TAGS = {
@@ -768,7 +777,7 @@ RFC5646_LANGUAGE_TAGS = {
     'uz-UZ': 'Uzbek (Latin) (Uzbekistan)',
     'uz-Cyrl-UZ': 'Uzbek (Cyrillic) (Uzbekistan)',
     'vi': 'Vietnamese',
-    'vi-VN': 'Vietnamese (Viet Nam)',
+    'vi-VN': 'Vietnamese (Vietnam)',
     'xh': 'Xhosa',
     'xh-ZA': 'Xhosa (South Africa)',
     'zh': 'Chinese',
@@ -894,7 +903,6 @@ COMMON_FEATURES = [
     Namespace.HASHES_BLAKE2B_512,
     Namespace.JINGLE,
     Namespace.JINGLE_FILE_TRANSFER_5,
-    Namespace.JINGLE_XTLS,
     Namespace.JINGLE_BYTESTREAM,
     Namespace.JINGLE_IBB,
     Namespace.AVATAR_METADATA + '+notify',
@@ -936,97 +944,6 @@ GAJIM_FAQ_URI = 'https://dev.gajim.org/gajim/gajim/wikis/help/gajimfaq'
 GAJIM_WIKI_URI = 'https://dev.gajim.org/gajim/gajim/wikis'
 GAJIM_SUPPORT_JID = 'gajim@conference.gajim.org'
 
-URI_SCHEMES = {
-    'aaa://',
-    'aaas://',
-    'acap://',
-    'cap://',
-    'cid:',
-    'crid://',
-    'data:',
-    'dav:',
-    'dict://',
-    'dns:',
-    'fax:',
-    'file:/',
-    'ftp://',
-    'geo:',
-    'go:',
-    'gopher://',
-    'h323:',
-    'http://',
-    'https://',
-    'iax:',
-    'icap://',
-    'im:',
-    'imap://',
-    'info:',
-    'ipp://',
-    'iris:',
-    'iris.beep:',
-    'iris.xpc:',
-    'iris.xpcs:',
-    'iris.lwz:',
-    'ldap://',
-    'mailto:',
-    'mid:',
-    'modem:',
-    'msrp://',
-    'msrps://',
-    'mtqp://',
-    'mupdate://',
-    'news:',
-    'nfs://',
-    'nntp://',
-    'opaquelocktoken:',
-    'pop://',
-    'pres:',
-    'prospero://',
-    'rtsp://',
-    'service:',
-    'sip:',
-    'sips:',
-    'sms:',
-    'snmp://',
-    'soap.beep://',
-    'soap.beeps://',
-    'tag:',
-    'tel:',
-    'telnet://',
-    'tftp://',
-    'thismessage:/',
-    'tip://',
-    'tv:',
-    'urn://',
-    'vemmi://',
-    'xmlrpc.beep://',
-    'xmlrpc.beeps://',
-    'xmpp:',
-    'z39.50r://',
-    'z39.50s://',
-    'about:',
-    'apt:',
-    'cvs://',
-    'daap://',
-    'ed2k://',
-    'feed:',
-    'fish://',
-    'git://',
-    'iax2:',
-    'irc://',
-    'ircs://',
-    'ldaps://',
-    'magnet:',
-    'mms://',
-    'rsync://',
-    'ssh://',
-    'svn://',
-    'sftp://',
-    'smb://',
-    'webcal://',
-    'aesgcm://',
-}
-
 
 # This is an excerpt of Media Types from
 # https://www.iana.org/assignments/media-types/media-types.xhtml
@@ -1180,6 +1097,15 @@ class PresenceShowExt(Enum):
     def __lt__(self, other: Union[PresenceShowExt, PresenceShow]) -> bool:
         if isinstance(other, PresenceShowExt):
             return False
-        if not isinstance(other, PresenceShow):
+        if not isinstance(other, PresenceShow):  # pyright: ignore
             return NotImplemented
         return True
+
+
+# Replace keys with Gio.TlsProtocolVersion enum once
+# we require GLib >= 2.69
+TLS_VERSION_STRINGS = {
+    3: '1.1',
+    4: '1.2',
+    5: '1.3',
+}
