@@ -25,12 +25,17 @@ from dataclasses import field
 
 from nbxmpp.protocol import JID
 from nbxmpp.modules.security_labels import Catalog
+from nbxmpp.modules.security_labels import Displaymarking
+from nbxmpp.modules.security_labels import SecurityLabel
 from nbxmpp.structs import HTTPAuthData
 from nbxmpp.structs import ModerationData
 from nbxmpp.structs import LocationData
 from nbxmpp.structs import RosterItem
 from nbxmpp.structs import TuneData
+from nbxmpp.const import Affiliation
 from nbxmpp.const import InviteType
+from nbxmpp.const import Role
+from nbxmpp.const import StatusCode
 
 from gajim.common.file_props import FileProp
 from gajim.common.const import JingleState
@@ -39,22 +44,20 @@ from gajim.common.helpers import AdditionalDataDict
 
 if typing.TYPE_CHECKING:
     from gajim.common.client import Client
+    from gajim.common.modules.httpupload import HTTPFileTransfer
     from gajim.plugins.pluginmanager import PluginManifest
 
-
-MainEventT = Union['PresenceReceived',
-                   'MessageSent',
-                   'MamMessageReceived',
-                   'GcMessageReceived',
-                   'MessageUpdated',
-                   'ReceiptReceived',
-                   'DisplayedReceived',
-                   'MessageError',
-                   'MucDiscoUpdate',
-                   'JingleConnectedReceived',
-                   'JingleDisconnectedReceived',
-                   'JingleErrorReceived',
-                   'FileRequestSent']
+ChatListEventT = Union[
+    'MessageReceived',
+    'MamMessageReceived',
+    'GcMessageReceived',
+    'MessageUpdated',
+    'MessageModerated',
+    'PresenceReceived',
+    'MessageSent',
+    'JingleRequestReceived',
+    'FileRequestReceivedEvent'
+]
 
 
 @dataclass
@@ -86,7 +89,7 @@ class AccountConnected(ApplicationEvent):
 
 
 @dataclass
-class AccountDisonnected(ApplicationEvent):
+class AccountDisconnected(ApplicationEvent):
     name: str = field(init=False, default='account-disconnected')
     account: str
 
@@ -159,10 +162,11 @@ class MessageSent(ApplicationEvent):
     jid: JID
     message: str
     message_id: str
+    msg_log_id: Optional[int]
     chatstate: Optional[str]
     timestamp: float
     additional_data: AdditionalDataDict
-    label: Optional[str]
+    label: Optional[SecurityLabel]
     correct_id: Optional[str]
     play_sound: bool
 
@@ -233,6 +237,22 @@ class FileSendError(ApplicationEvent):
     error_msg: str = ''
 
 
+@dataclass
+class HTTPUploadStarted(ApplicationEvent):
+    name: str = field(init=False, default='http-upload-started')
+    account: str
+    jid: JID
+    transfer: HTTPFileTransfer
+
+
+@dataclass
+class HTTPUploadError(ApplicationEvent):
+    name: str = field(init=False, default='http-upload-error')
+    account: str
+    jid: JID
+    error_msg: str
+
+
 @dataclass
 class AccountEnabled(ApplicationEvent):
     name: str = field(init=False, default='account-enabled')
@@ -259,8 +279,16 @@ class BookmarksReceived(ApplicationEvent):
 
 
 @dataclass
-class BaseChatMarkerEvent(ApplicationEvent):
-    name: str
+class ReadStateSync(ApplicationEvent):
+    name: str = field(init=False, default='read-state-sync')
+    account: str
+    jid: JID
+    marker_id: str
+
+
+@dataclass
+class DisplayedReceived(ApplicationEvent):
+    name: str = field(init=False, default='displayed-received')
     account: str
     jid: JID
     properties: Any
@@ -269,16 +297,6 @@ class BaseChatMarkerEvent(ApplicationEvent):
     marker_id: str
 
 
-@dataclass
-class ReadStateSync(BaseChatMarkerEvent):
-    name: str = field(init=False, default='read-state-sync')
-
-
-@dataclass
-class DisplayedReceived(BaseChatMarkerEvent):
-    name: str = field(init=False, default='displayed-received')
-
-
 @dataclass
 class HttpAuth(ApplicationEvent):
     name: str = field(init=False, default='http-auth')
@@ -390,10 +408,9 @@ class MessageReceived(ApplicationEvent):
     stanza_id: str
     fjid: str
     resource: str
-    session: Any
     delayed: Optional[float]
-    msg_log_id: int
-    displaymarking: str
+    msg_log_id: Optional[int]
+    displaymarking: Optional[Displaymarking]
 
 
 @dataclass
@@ -591,6 +608,9 @@ class PingReply(ApplicationEvent):
     seconds: float
 
 
+PingEventT = Union[PingSent, PingReply, PingError]
+
+
 @dataclass
 class SecCatalogReceived(ApplicationEvent):
     name: str = field(init=False, default='sec-catalog-received')
@@ -673,3 +693,101 @@ class AllowGajimUpdateCheck(ApplicationEvent):
 class GajimUpdateAvailable(ApplicationEvent):
     name: str = field(init=False, default='gajim-update-available')
     version: str
+
+
+@dataclass
+class MUCNicknameChanged(ApplicationEvent):
+    name: str = field(init=False, default='muc-nickname-changed')
+    is_self: bool
+    new_name: str
+    old_name: str
+    timestamp: float
+
+
+@dataclass
+class MUCRoomConfigChanged(ApplicationEvent):
+    name: str = field(init=False, default='muc-room-config-changed')
+    timestamp: float
+    status_codes: set[StatusCode]
+
+
+@dataclass
+class MUCRoomConfigFinished(ApplicationEvent):
+    name: str = field(init=False, default='muc-room-config-finished')
+    timestamp: float
+
+
+@dataclass
+class MUCRoomPresenceError(ApplicationEvent):
+    name: str = field(init=False, default='muc-room-presence-error')
+    timestamp: float
+    error: Any
+
+
+@dataclass
+class MUCRoomKicked(ApplicationEvent):
+    name: str = field(init=False, default='muc-room-kicked')
+    timestamp: float
+    status_codes: Optional[set[StatusCode]]
+    reason: Optional[str]
+    actor: Optional[str]
+
+
+@dataclass
+class MUCRoomDestroyed(ApplicationEvent):
+    name: str = field(init=False, default='muc-room-destroyed')
+    timestamp: float
+    reason: Optional[str]
+    alternate: Optional[JID]
+
+
+@dataclass
+class MUCUserJoined(ApplicationEvent):
+    name: str = field(init=False, default='muc-user-joined')
+    timestamp: float
+    is_self: bool
+    nick: str
+    status_codes: Optional[set[StatusCode]]
+
+
+@dataclass
+class MUCUserLeft(ApplicationEvent):
+    name: str = field(init=False, default='muc-user-left')
+    timestamp: float
+    is_self: bool
+    nick: str
+    status_codes: Optional[set[StatusCode]]
+    reason: Optional[str]
+    actor: Optional[str]
+
+
+@dataclass
+class MUCUserRoleChanged(ApplicationEvent):
+    name: str = field(init=False, default='muc-user-role-changed')
+    timestamp: float
+    is_self: bool
+    nick: str
+    role: Role
+    reason: Optional[str]
+    actor: Optional[str]
+
+
+@dataclass
+class MUCUserAffiliationChanged(ApplicationEvent):
+    name: str = field(init=False, default='muc-user-affiliation-changed')
+    timestamp: float
+    is_self: bool
+    nick: str
+    affiliation: Affiliation
+    reason: Optional[str]
+    actor: Optional[str]
+
+
+@dataclass
+class MUCUserStatusShowChanged(ApplicationEvent):
+    name: str = field(init=False, default='muc-user-status-show-changed')
+    timestamp: float
+    is_self: bool
+    nick: str
+    status: str
+    show_value: str
