@@ -12,9 +12,9 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-"""
-Handles  Jingle File Transfer (XEP 0234)
-"""
+
+# Handles  Jingle File Transfer (XEP 0234)
+
 
 from __future__ import annotations
 
@@ -33,8 +33,6 @@ from nbxmpp import JID
 from nbxmpp.namespaces import Namespace
 
 from gajim.common import app
-from gajim.common import configpaths
-from gajim.common import jingle_xtls
 from gajim.common import helpers
 from gajim.common.const import KindConstant
 from gajim.common.helpers import AdditionalDataDict
@@ -80,12 +78,11 @@ class JingleFileTransfer(JingleContent):
                  session: JingleSession,
                  file_props: FileProp,
                  transport: Optional[JingleTransport] = None,
-                 use_security: bool = False,
                  senders: Optional[str] = None
                  ) -> None:
 
         JingleContent.__init__(self, session, transport, senders)
-        log.info("transport value: %s", transport)
+        log.info('transport value: %s', transport)
         # events we might be interested in
         self.callbacks['session-initiate'] += [self.__on_session_initiate]
         self.callbacks['session-initiate-sent'] += [
@@ -102,8 +99,6 @@ class JingleFileTransfer(JingleContent):
         self.callbacks['transport-info'] += [self.__on_transport_info]
         self.callbacks['iq-result'] += [self.__on_iq_result]
 
-        self.use_security = use_security
-        self.x509_fingerprint: Optional[str] = None
         self.file_props = file_props
         self.weinitiate = self.session.weinitiate
         self.werequest = self.session.werequest
@@ -123,7 +118,7 @@ class JingleFileTransfer(JingleContent):
             self.file_props.receiver = self.session.peerjid
         self.file_props.session_type = 'jingle'
         self.file_props.sid = session.sid
-        self.file_props.transfered_size = []
+        self.file_props.transferred_size = []
         self.file_props.transport_sid = self.transport.sid
 
         self.session = session
@@ -139,12 +134,6 @@ class JingleFileTransfer(JingleContent):
             State.CAND_SENT_AND_RECEIVED: StateCandSentAndRecv(self)
         }
 
-        cert_name = (configpaths.get('MY_CERT') /
-                     jingle_xtls.SELF_SIGNED_CERTIFICATE)
-        if not (cert_name.with_suffix('.cert').exists() and
-                cert_name.with_suffix('.pkey').exists()):
-            jingle_xtls.make_certs(cert_name, 'gajim')
-
     def __state_changed(self,
                         nextstate: State,
                         args: Optional[dict[str, Any]] = None
@@ -163,7 +152,7 @@ class JingleFileTransfer(JingleContent):
                               error: Optional[nbxmpp.Node],
                               action: str
                               ) -> None:
-        log.debug("Jingle FT request received")
+        log.debug('Jingle FT request received')
         self._raise_event(stanza, content)
 
         account = self.session.connection.name
@@ -200,13 +189,6 @@ class JingleFileTransfer(JingleContent):
         if not content:
             return
 
-        secu = content.getTag('security')
-        self.use_security = bool(secu)
-        if secu:
-            fingerprint = secu.getTag('fingerprint')
-            if fingerprint:
-                self.x509_fingerprint = fingerprint.getData()
-
         if not self.transport:
             self.transport = JingleTransportSocks5()
             self.transport.set_our_jid(self.session.ourjid)
@@ -262,7 +244,7 @@ class JingleFileTransfer(JingleContent):
         file_desc_tag = file_tag.getTag('desc')
         if file_desc_tag is not None:
             file_props.desc = file_desc_tag.getData()
-        file_props.transfered_size = []
+        file_props.transferred_size = []
 
         app.ged.raise_event(FileRequestReceivedEvent(
             conn=con,
@@ -332,27 +314,7 @@ class JingleFileTransfer(JingleContent):
                             error: Optional[nbxmpp.Node],
                             action: str
                             ) -> None:
-        log.info("__on_session_accept")
-        con = self.session.connection
-        security = content.getTag('security')
-        if not security:  # responder can not verify our fingerprint
-            self.use_security = False
-        else:
-            fingerprint = security.getTag('fingerprint')
-            if fingerprint:
-                fingerprint = fingerprint.getData()
-                self.x509_fingerprint = fingerprint
-                if not jingle_xtls.check_cert(app.get_jid_without_resource(
-                        self.session.responder), fingerprint):
-                    id_ = jingle_xtls.send_cert_request(con,
-                                                        self.session.responder)
-                    jingle_xtls.key_exchange_pend(id_,
-                                                  self.continue_session_accept,
-                                                  [stanza])
-                    raise nbxmpp.NodeProcessed
-        self.continue_session_accept(stanza)
-
-    def continue_session_accept(self, stanza: nbxmpp.Node) -> None:
+        log.info('__on_session_accept')
         if self.state == State.TRANSPORT_REPLACE:
             # If we are requesting we don't have the file
             if self.session.werequest:
@@ -371,16 +333,12 @@ class JingleFileTransfer(JingleContent):
             host['initiator'] = self.session.initiator
             host['target'] = self.session.responder
             host['sid'] = self.file_props.sid
-        fingerprint = None
-        if self.use_security:
-            fingerprint = 'client'
         if self.transport.type_ == TransportType.SOCKS5:
             sid = self.file_props.transport_sid
             app.socks5queue.connect_to_hosts(self.session.connection.name,
                                              sid,
                                              self.on_connect,
                                              self._on_connect_error,
-                                             fingerprint=fingerprint,
                                              receiving=False)
             raise nbxmpp.NodeProcessed
         self.__state_changed(State.TRANSFERRING)
@@ -392,7 +350,7 @@ class JingleFileTransfer(JingleContent):
                                error: Optional[nbxmpp.Node],
                                action: str
                                ) -> None:
-        log.info("__on_session_terminate")
+        log.info('__on_session_terminate')
 
     def __on_session_info(self,
                           stanza: nbxmpp.Node,
@@ -408,7 +366,7 @@ class JingleFileTransfer(JingleContent):
                               error: Optional[nbxmpp.Node],
                               action: str
                               ) -> None:
-        log.info("__on_transport_accept")
+        log.info('__on_transport_accept')
 
     def __on_transport_replace(self,
                                stanza: nbxmpp.Node,
@@ -416,7 +374,7 @@ class JingleFileTransfer(JingleContent):
                                error: Optional[nbxmpp.Node],
                                action: str
                                ) -> None:
-        log.info("__on_transport_replace")
+        log.info('__on_transport_replace')
 
     def __on_transport_reject(self,
                               stanza: nbxmpp.Node,
@@ -424,7 +382,7 @@ class JingleFileTransfer(JingleContent):
                               error: Optional[nbxmpp.Node],
                               action: str
                               ) -> None:
-        log.info("__on_transport_reject")
+        log.info('__on_transport_reject')
 
     def __on_transport_info(self,
                             stanza: nbxmpp.Node,
@@ -432,7 +390,7 @@ class JingleFileTransfer(JingleContent):
                             error: Optional[nbxmpp.Node],
                             action: str
                             ) -> None:
-        log.info("__on_transport_info")
+        log.info('__on_transport_info')
         cand_error = content.getTag('transport').getTag('candidate-error')
         cand_used = content.getTag('transport').getTag('candidate-used')
         if (cand_error or cand_used) and \
@@ -488,7 +446,7 @@ class JingleFileTransfer(JingleContent):
                        error: Optional[nbxmpp.Node],
                        action: str
                        ) -> None:
-        log.info("__on_iq_result")
+        log.info('__on_iq_result')
 
         if self.state in (State.NOT_STARTED, State.CAND_RECEIVED):
             self.__state_changed(State.INITIALIZED)
@@ -516,9 +474,9 @@ class JingleFileTransfer(JingleContent):
             self._listen_host()
 
     def on_connect(self, streamhost):
-        """
+        '''
         send candidate-used stanza
-        """
+        '''
         log.info('send_candidate_used')
         if streamhost is None:
             return
@@ -550,15 +508,11 @@ class JingleFileTransfer(JingleContent):
                                        receiver)
         self.file_props.sha_str = sha_str
         port = app.settings.get('file_transfers_port')
-        fingerprint = None
-        if self.use_security:
-            fingerprint = 'server'
         listener = app.socks5queue.start_listener(
             port,
             sha_str,
             self._store_socks5_sid,
             self.file_props,
-            fingerprint=fingerprint,
             typ='sender' if self.weinitiate else 'receiver')
         if not listener:
             # send error message, notify the user
