@@ -14,50 +14,60 @@
 
 # XEP-0085: Chat State Notifications
 
+from __future__ import annotations
+
 from typing import Any
-from typing import Dict  # pylint: disable=unused-import
-from typing import List  # pylint: disable=unused-import
 from typing import Optional
-from typing import Tuple
 
 import time
+from itertools import chain
 from functools import wraps
 
-import nbxmpp
 from nbxmpp.namespaces import Namespace
+from nbxmpp.protocol import JID
+from nbxmpp.protocol import Presence
+from nbxmpp.structs import MessageProperties
+from nbxmpp.structs import PresenceProperties
 from nbxmpp.structs import StanzaHandler
+from nbxmpp.const import Chatstate as State
+
 from gi.repository import GLib
 
-from gajim.common import app
-from gajim.common.nec import NetworkEvent
-from gajim.common.const import Chatstate as State
+from gajim.common import types
+from gajim.common.const import ClientState
 from gajim.common.structs import OutgoingMessage
 from gajim.common.modules.base import BaseModule
-
-from gajim.common.types import ContactT
-from gajim.common.types import ConnectionT
-
+from gajim.common.modules.contacts import GroupchatParticipant
+from gajim.common.modules.contacts import BareContact
 
 INACTIVE_AFTER = 60
 PAUSED_AFTER = 10
+REMOTE_PAUSED_AFTER = 30
 
 
-def ensure_enabled(func):
+def ensure_enabled(func: Any) -> Any:
     @wraps(func)
-    def func_wrapper(self, *args, **kwargs):
-        if not self.enabled:
+    def func_wrapper(self: Any, *args: Any, **kwargs: Any):
+        # pylint: disable=protected-access
+        if not self._enabled:
             return None
         return func(self, *args, **kwargs)
     return func_wrapper
 
 
 class Chatstate(BaseModule):
-    def __init__(self, con: ConnectionT) -> None:
+    def __init__(self, con: types.Client) -> None:
         BaseModule.__init__(self, con)
 
         self.handlers = [
             StanzaHandler(name='presence',
-                          callback=self._presence_received),
+                          callback=self._presence_received,
+                          typ='error',
+                          priority=50),
+            StanzaHandler(name='presence',
+                          callback=self._presence_received,
+                          typ='unavailable',
+                          priority=50),
             StanzaHandler(name='message',
                           callback=self._process_chatstate,
                           ns=Namespace.CHATSTATES,
@@ -65,23 +75,43 @@ class Chatstate(BaseModule):
         ]
 
         # Our current chatstate with a specific contact
-        self._chatstates = {}  # type: Dict[str, State]
+        self._chatstates: dict[JID, State] = {}
 
-        self._last_keyboard_activity = {}  # type: Dict[str, float]
-        self._last_mouse_activity = {}  # type: Dict[str, float]
+        # The current chatstate we received from a contact
+        self._remote_chatstate: dict[JID, State] = {}
+
+        self._remote_chatstate_composing_timeouts: dict[JID, int] = {}
+
+        self._last_keyboard_activity: dict[JID, float] = {}
+        self._last_mouse_activity: dict[JID, float] = {}
         self._timeout_id = None
-        self._delay_timeout_ids = {}  # type: Dict[str, str]
-        self._blocked = []  # type: List[str]
+        self._delay_timeout_ids: dict[JID, int] = {}
+        self._blocked: list[JID] = []
         self._enabled = False
 
-    @property
-    def enabled(self):
-        return self._enabled
+        self._con.connect_signal('state-changed', self._on_client_state_changed)
+        self._con.connect_signal('resume-failed', self._on_client_resume_failed)
 
-    @enabled.setter
-    def enabled(self, value):
+    def _on_client_resume_failed(self,
+                                 _client: types.Client,
+                                 _signal_name: str
+                                 ) -> None:
+        self._set_enabled(False)
+
+    def _on_client_state_changed(self,
+                                 _client: types.Client,
+                                 _signal_name: str,
+                                 state: ClientState
+                                 ) -> None:
+        if state.is_disconnected:
+            self._set_enabled(False)
+        elif state.is_connected:
+            self._set_enabled(True)
+
+    def _set_enabled(self, value: bool) -> None:
         if self._enabled == value:
             return
+
         self._log.info('Chatstate module %s',
                        'enabled' if value else 'disabled')
         self._enabled = value
@@ -91,82 +121,89 @@ class Chatstate(BaseModule):
                 2, self._check_last_interaction)
         else:
             self.cleanup()
-            self._chatstates = {}
-            self._last_keyboard_activity = {}
-            self._last_mouse_activity = {}
-            self._blocked = []
+            self._con.get_module('Contacts').force_chatstate_update()
 
     @ensure_enabled
     def _presence_received(self,
-                           _con: ConnectionT,
-                           stanza: nbxmpp.Presence,
-                           _properties: Any) -> None:
-        if stanza.getType() not in ('unavailable', 'error'):
+                           _con: types.xmppClient,
+                           _stanza: Presence,
+                           properties: PresenceProperties
+                           ) -> None:
+
+        if properties.is_self_bare:
             return
 
-        full_jid = stanza.getFrom()
-        if full_jid is None or self._con.get_own_jid().bare_match(full_jid):
-            # Presence from ourself
-            return
+        jid = properties.jid
 
-        contact = app.contacts.get_gc_contact(
-            self._account, full_jid.bare, full_jid.resource)
-        if contact is None:
-            contact = app.contacts.get_contact_from_full_jid(
-                self._account, str(full_jid))
-        if contact is None:
-            return
-
-        if contact.chatstate is None:
-            return
-
-        if contact.is_gc_contact:
-            jid = contact.get_full_jid()
-        else:
-            jid = contact.jid
-
-        contact.chatstate = None
+        assert jid is not None
+        self._remote_chatstate.pop(jid, None)
         self._chatstates.pop(jid, None)
         self._last_mouse_activity.pop(jid, None)
         self._last_keyboard_activity.pop(jid, None)
 
         self._log.info('Reset chatstate for %s', jid)
 
-        app.nec.push_outgoing_event(
-            NetworkEvent('chatstate-received',
-                         account=self._account,
-                         contact=contact))
+        contact = self._get_contact(jid)
+        if contact.is_groupchat:
+            return
+
+        contact.notify('chatstate-update')
+
+    def _process_chatstate(self,
+                           _con: types.xmppClient,
+                           _stanza: Any,
+                           properties: MessageProperties
+                           ) -> None:
+        if properties.type.is_error:
+            return
 
-    def _process_chatstate(self, _con, _stanza, properties):
         if not properties.has_chatstate:
             return
 
         if (properties.is_self_message or
-                properties.type.is_groupchat or
+                not properties.type.is_chat or
                 properties.is_mam_message or
                 properties.is_carbon_message and properties.carbon.is_sent):
             return
 
-        if properties.is_muc_pm:
-            contact = app.contacts.get_gc_contact(
-                self._account,
-                properties.jid.bare,
-                properties.jid.resource)
-        else:
-            contact = app.contacts.get_contact_from_full_jid(
-                self._account, str(properties.jid))
+        jid = properties.jid
+        assert jid is not None
+
+        state = properties.chatstate
+        self._remote_chatstate[jid] = state
+
+        self._log.info('Recv: %-10s - %s', state, jid)
+
+        contact = self._get_contact(jid)
         if contact is None:
             return
 
-        contact.chatstate = properties.chatstate
-        self._log.info('Recv: %-10s - %s', properties.chatstate, properties.jid)
-        app.nec.push_outgoing_event(
-            NetworkEvent('chatstate-received',
-                         account=self._account,
-                         contact=contact))
+        self._remove_remote_composing_timeout(contact)
+        if state == State.COMPOSING:
+            # the spec does not cover any timeout for the composing action,
+            # but if a contact's client does not send another chat state,
+            # we don't want the GUI to show that they are "composing" forever
+            self._remote_chatstate_composing_timeouts[jid] = \
+                GLib.timeout_add_seconds(REMOTE_PAUSED_AFTER,
+                                         self._on_remote_composing_timeout,
+                                         contact)
+
+        contact.notify('chatstate-update')
+
+    def _on_remote_composing_timeout(self, contact: types.ContactT):
+        self._log.info(
+            'Automatically switching the chat state of %s to ACTIVE', contact)
+        self._remote_chatstate[contact.jid] = State.ACTIVE
+        contact.notify('chatstate-update')
+
+    def _remove_remote_composing_timeout(self, contact: types.ContactT):
+        source_id = self._delay_timeout_ids.pop(contact.jid, None)
+        if source_id is not None:
+            self._log.debug('Removing remote composing timeout of %s', contact)
+            GLib.source_remove(source_id)
 
     @ensure_enabled
-    def _check_last_interaction(self) -> GLib.SOURCE_CONTINUE:
+    def _check_last_interaction(self) -> bool:
         now = time.time()
         for jid in list(self._last_mouse_activity.keys()):
             time_ = self._last_mouse_activity[jid]
@@ -190,34 +227,24 @@ class Chatstate(BaseModule):
 
             if new_chatstate is not None:
                 if self._chatstates.get(jid) != new_chatstate:
-                    contact = app.contacts.get_contact(self._account, jid)
-                    if contact is None:
-                        room, nick = app.get_room_and_nick_from_fjid(jid)
-                        contact = app.contacts.get_gc_contact(
-                            self._account, room, nick)
-                        if contact is not None:
-                            contact = contact.as_contact()
-                        else:
-                            # Contact not found, maybe we left the group chat
-                            # or the contact was removed from the roster
-                            self._log.info(
-                                'Contact %s not found, reset chatstate', jid)
-                            self._chatstates.pop(jid, None)
-                            self._last_mouse_activity.pop(jid, None)
-                            self._last_keyboard_activity.pop(jid, None)
-                            continue
-                self.set_chatstate(contact, new_chatstate)
+                    contact = self._get_contact(jid)
+                    self.set_chatstate(contact, new_chatstate)
 
         return GLib.SOURCE_CONTINUE
 
+    def get_remote_chatstate(self, jid: JID) -> Chatstate:
+        return self._remote_chatstate.get(jid)
+
     @ensure_enabled
-    def set_active(self, contact: ContactT) -> None:
+    def set_active(self, contact: types.ChatContactT) -> None:
         if contact.settings.get('send_chatstate') == 'disabled':
             return
         self._last_mouse_activity[contact.jid] = time.time()
         self._chatstates[contact.jid] = State.ACTIVE
 
-    def get_active_chatstate(self, contact: ContactT) -> Optional[str]:
+    def get_active_chatstate(self,
+                             contact: types.ChatContactT
+                             ) -> Optional[str]:
         # determines if we add 'active' on outgoing messages
         if contact.settings.get('send_chatstate') == 'disabled':
             return None
@@ -234,7 +261,10 @@ class Chatstate(BaseModule):
         return 'active'
 
     @ensure_enabled
-    def block_chatstates(self, contact: ContactT, block: bool) -> None:
+    def block_chatstates(self,
+                         contact: types.ChatContactT,
+                         block: bool
+                         ) -> None:
         # Block sending chatstates to a contact
         # Used for example if we cycle through the MUC nick list, which
         # produces a lot of buffer 'changed' signals from the input textview.
@@ -245,16 +275,24 @@ class Chatstate(BaseModule):
             self._blocked.remove(contact.jid)
 
     @ensure_enabled
-    def set_chatstate_delayed(self, contact: ContactT, state: State) -> None:
+    def set_chatstate_delayed(self,
+                              contact: types.ChatContactT,
+                              state: State
+                              ) -> None:
         # Used when we go from Composing -> Active after deleting all text
         # from the Textview. We delay the Active state because maybe the
         # User starts writing again.
+
+        # Don’t send chatstates to ourself
+        if self._con.get_own_jid().bare_match(contact.jid):
+            return
+
         self.remove_delay_timeout(contact)
         self._delay_timeout_ids[contact.jid] = GLib.timeout_add_seconds(
             2, self.set_chatstate, contact, state)
 
     @ensure_enabled
-    def set_chatstate(self, contact: ContactT, state: State) -> None:
+    def set_chatstate(self, contact: types.ChatContactT, state: State) -> None:
         # Don’t send chatstates to ourself
         if self._con.get_own_jid().bare_match(contact.jid):
             return
@@ -268,31 +306,34 @@ class Chatstate(BaseModule):
         if setting == 'disabled':
             # Send a last 'active' state after user disabled chatstates
             if current_state is not None:
-                self._log.info('Disabled for %s', contact.jid)
+                self._log.info('Disabled for %s', contact)
                 self._log.info('Send last state: %-10s - %s',
-                               State.ACTIVE, contact.jid)
+                               State.ACTIVE, contact)
 
-                self._send_chatstate(contact, str(State.ACTIVE))
+                self._send_chatstate(contact, State.ACTIVE)
 
             self._chatstates.pop(contact.jid, None)
             self._last_mouse_activity.pop(contact.jid, None)
             self._last_keyboard_activity.pop(contact.jid, None)
             return
 
-        if not contact.is_groupchat:
-            # Don’t leak presence to contacts
-            # which are not allowed to see our status
-            if not contact.is_pm_contact:
-                if contact and contact.sub in ('to', 'none'):
-                    self._log.info('Contact not subscribed: %s', contact.jid)
-                    return
-
-            if contact.show == 'offline':
-                self._log.info('Contact offline: %s', contact.jid)
+        if isinstance(contact, BareContact):
+            if not contact.is_subscribed:
+                self._log.debug('Contact not subscribed: %s', contact)
                 return
 
-            if not contact.supports(Namespace.CHATSTATES):
-                self._log.info('Chatstates not supported: %s', contact.jid)
+            if not contact.is_available:
+                self._log.debug('Contact offline: %s', contact)
+                return
+
+        elif isinstance(contact, GroupchatParticipant):
+            if not contact.is_available:
+                self._log.debug('Contact offline: %s', contact)
+                return
+
+        else:
+            if not contact.is_joined:
+                self._log.debug('Groupchat not joined: %s', contact)
                 return
 
         if state in (State.ACTIVE, State.COMPOSING):
@@ -305,25 +346,31 @@ class Chatstate(BaseModule):
         if current_state == state:
             return
 
-        self._log.info('Send: %-10s - %s', state, contact.jid)
+        self._log.info('Send: %-10s - %s', state, contact)
 
-        self._send_chatstate(contact, str(state))
+        self._send_chatstate(contact, state)
 
         self._chatstates[contact.jid] = state
 
-    def _send_chatstate(self, contact, chatstate):
+    def _send_chatstate(self,
+                        contact: types.ChatContactT,
+                        chatstate: State
+                        ) -> None:
         type_ = 'groupchat' if contact.is_groupchat else 'chat'
         message = OutgoingMessage(account=self._account,
                                   contact=contact,
                                   message=None,
                                   type_=type_,
-                                  chatstate=chatstate,
+                                  chatstate=chatstate.value,
                                   play_sound=False)
 
         self._con.send_message(message)
 
     @ensure_enabled
-    def set_mouse_activity(self, contact: ContactT, was_paused: bool) -> None:
+    def set_mouse_activity(self,
+                           contact: types.ChatContactT,
+                           was_paused: bool
+                           ) -> None:
         if contact.settings.get('send_chatstate') == 'disabled':
             return
         self._last_mouse_activity[contact.jid] = time.time()
@@ -334,25 +381,31 @@ class Chatstate(BaseModule):
                 self.set_chatstate(contact, State.ACTIVE)
 
     @ensure_enabled
-    def set_keyboard_activity(self, contact: ContactT) -> None:
+    def set_keyboard_activity(self, contact: types.ChatContactT) -> None:
         self._last_keyboard_activity[contact.jid] = time.time()
 
-    def remove_delay_timeout(self, contact):
+    def remove_delay_timeout(self, contact: types.ChatContactT) -> None:
         timeout = self._delay_timeout_ids.get(contact.jid)
         if timeout is not None:
             GLib.source_remove(timeout)
             del self._delay_timeout_ids[contact.jid]
 
-    def remove_all_delay_timeouts(self):
-        for timeout in self._delay_timeout_ids.values():
+    def remove_all_timeouts(self) -> None:
+        for timeout in chain(
+                self._delay_timeout_ids.values(),
+                self._remote_chatstate_composing_timeouts.values()):
             GLib.source_remove(timeout)
-        self._delay_timeout_ids = {}
+        self._delay_timeout_ids.clear()
+        self._remote_chatstate_composing_timeouts.clear()
 
-    def cleanup(self):
-        self.remove_all_delay_timeouts()
+    def cleanup(self) -> None:
+        self.remove_all_timeouts()
         if self._timeout_id is not None:
             GLib.source_remove(self._timeout_id)
+            self._timeout_id = None
 
-
-def get_instance(*args: Any, **kwargs: Any) -> Tuple[Chatstate, str]:
-    return Chatstate(*args, **kwargs), 'Chatstate'
+        self._chatstates.clear()
+        self._remote_chatstate.clear()
+        self._last_keyboard_activity.clear()
+        self._last_mouse_activity.clear()
+        self._blocked = []
