@@ -20,8 +20,6 @@ from typing import Optional
 from typing import Union
 from typing import overload
 
-from functools import partial
-
 import cairo
 from nbxmpp.const import Affiliation
 from nbxmpp.const import Chatstate
@@ -37,6 +35,8 @@ from gajim.common import app
 from gajim.common import types
 from gajim.common.const import PresenceShowExt
 from gajim.common.const import SimpleClientState
+from gajim.common.setting_values import BoolContactSettings
+from gajim.common.setting_values import StringContactSettings
 from gajim.common.setting_values import BoolGroupChatSettings
 from gajim.common.setting_values import IntGroupChatSettings
 from gajim.common.setting_values import StringGroupChatSettings
@@ -45,6 +45,7 @@ from gajim.common.structs import PresenceData
 from gajim.common.structs import UNKNOWN_MUC_PRESENCE
 from gajim.common.structs import MUCPresenceData
 from gajim.common.helpers import Observable
+from gajim.common.helpers import chatstate_to_string
 from gajim.common.modules.base import BaseModule
 from gajim.common.modules.util import LogAdapter
 from gajim.common.helpers import get_groupchat_name
@@ -52,8 +53,26 @@ from gajim.common.helpers import get_groupchat_name
 
 class ContactSettings:
     def __init__(self, account: str, jid: JID) -> None:
-        self.get = partial(app.settings.get_contact_setting, account, jid)
-        self.set = partial(app.settings.set_contact_setting, account, jid)
+        self._account = account
+        self._jid = jid
+
+    @overload
+    def get(self, setting: StringContactSettings) -> str: ...  # noqa: E704
+    @overload
+    def get(self, setting: BoolContactSettings) -> bool: ...  # noqa: E704
+
+    def get(self, setting: Any) -> Any:
+        return app.settings.get_contact_setting(
+            self._account, self._jid, setting)
+
+    @overload
+    def set(self, setting: StringContactSettings, value: str) -> None: ...  # noqa: E501, E704
+    @overload
+    def set(self, setting: BoolContactSettings, value: bool) -> None: ...  # noqa: E501, E704
+
+    def set(self, setting: Any, value: Any) -> None:
+        app.settings.set_contact_setting(
+            self._account, self._jid, setting, value)
 
 
 class GroupChatSettings:
@@ -104,6 +123,57 @@ class Contacts(BaseModule):
         if state.is_disconnected:
             self._reset_presence()
 
+    def add_chat_contact(self, jid: Union[str, JID]) -> BareContact:
+        if isinstance(jid, str):
+            jid = JID.from_string(jid)
+
+        contact = self._contacts.get(jid)
+        if contact is not None:
+            if not isinstance(contact, BareContact):
+                raise ValueError(f'Trying to add BareContact {jid}, '
+                                 f'but contact exists already as {contact}')
+            return contact
+
+        contact = BareContact(self._log, jid, self._account)
+
+        self._contacts[jid] = contact
+        return contact
+
+    def add_group_chat_contact(self, jid: Union[str, JID]) -> GroupchatContact:
+        if isinstance(jid, str):
+            jid = JID.from_string(jid)
+
+        contact = self._contacts.get(jid)
+        if contact is not None:
+            if not isinstance(contact, GroupchatContact):
+                raise ValueError(f'Trying to add GroupchatContact {jid}, '
+                                 f'but contact already exists as {contact} '
+                                 f'(in roster: {contact.is_in_roster})')
+            return contact
+
+        contact = GroupchatContact(self._log, jid, self._account)
+
+        self._contacts[jid] = contact
+        return contact
+
+    def add_private_contact(self, jid: Union[str, JID]) -> GroupchatParticipant:
+        if isinstance(jid, str):
+            jid = JID.from_string(jid)
+
+        if jid.resource is None:
+            raise ValueError(f'Trying to add a bare JID as private {jid}')
+
+        contact = self._contacts.get(jid.bare)
+        if contact is None:
+            group_chat_contact = self.add_group_chat_contact(jid.bare)
+            return group_chat_contact.add_resource(jid.resource)
+
+        if not isinstance(contact, GroupchatContact):
+            raise ValueError(f'Trying to add GroupchatParticipant {jid}, '
+                             f'to BareContact {contact}')
+
+        return contact.add_resource(jid.resource)
+
     def add_contact(self,
                     jid: Union[str, JID],
                     groupchat: bool = False) -> Union[BareContact,
@@ -144,14 +214,15 @@ class Contacts(BaseModule):
         contact = contact.get_resource(resource)
         return contact
 
-    def get_group_chat_contact(self,
-                               jid: Union[str, JID]
-                               ) -> Union[BareContact,
-                                          ResourceContact,
-                                          GroupchatContact,
-                                          GroupchatParticipant]:
-
-        return self.get_contact(jid, groupchat=True)
+    def get_bare_contact(self, jid: Union[str, JID]) -> Union[BareContact,
+                                                              GroupchatContact]:
+        '''This method gives direct access to the contacts dict.
+           This is helpful when performance is essential. In difference to
+           get_contact() this method does not create contacts nor can it handle
+           JIDs which are not bare. Use this only if you know the contact
+           exists.
+        '''
+        return self._contacts[jid]
 
     def get_contacts_with_domain(self,
                                  domain: str
@@ -187,6 +258,16 @@ class CommonContact(Observable):
         self._jid = jid
         self._account = account
 
+    def __hash__(self) -> int:
+        return hash(f'{self._account}-{self._jid}')
+
+    def __eq__(self, obj: object) -> bool:
+        if not isinstance(obj, CommonContact):
+            return NotImplemented
+
+        return (self._account == obj.account and
+                obj._jid == self._jid)
+
     def _module(self, name: str) -> BaseModule:
         return app.get_client(self._account).get_module(name)
 
@@ -220,6 +301,10 @@ class CommonContact(Observable):
 
         return disco_info.supports(requested_feature)
 
+    @property
+    def is_chat(self) -> bool:
+        return False
+
     @property
     def is_groupchat(self) -> bool:
         return False
@@ -228,6 +313,10 @@ class CommonContact(Observable):
     def is_pm_contact(self) -> bool:
         return False
 
+    @property
+    def type_string(self) -> str:
+        raise NotImplementedError
+
     @property
     def is_jingle_available(self) -> bool:
         return False
@@ -235,6 +324,14 @@ class CommonContact(Observable):
     def get_address(self, _prefer_real: bool = True) -> JID:
         return self._jid
 
+    @property
+    def chatstate(self) -> Optional[Chatstate]:
+        return None
+
+    @property
+    def chatstate_string(self) -> str:
+        return chatstate_to_string(self.chatstate)
+
     def __repr__(self) -> str:
         return f'{self.jid} ({self._account})'
 
@@ -272,6 +369,11 @@ class BareContact(CommonContact):
         return False
 
     def add_resource(self, resource: str) -> ResourceContact:
+        assert resource is not None
+        # Check if resource is not None because not the whole
+        # codebase is type checked and it creates hard to track
+        # problems if we create a ResourceContact without resource
+
         jid = self._jid.new_with(resource=resource)
         assert isinstance(self._log, LogAdapter)
         contact = ResourceContact(self._log, jid, self._account)
@@ -355,7 +457,6 @@ class BareContact(CommonContact):
                    size: int,
                    scale: int,
                    add_show: bool = True,
-                   pixbuf: bool = False,
                    default: bool = False,
                    style: str = 'circle'):
 
@@ -378,15 +479,6 @@ class BareContact(CommonContact):
         if self.avatar_sha is not None:
             transport_icon = None
 
-        if pixbuf:
-            return app.app.avatar_storage.get_pixbuf(
-                self,
-                size,
-                scale,
-                show,
-                default=default,
-                transport_icon=transport_icon,
-                style=style)
         return app.app.avatar_storage.get_surface(
             self,
             size,
@@ -465,6 +557,14 @@ class BareContact(CommonContact):
             return False
         return self.is_available
 
+    @property
+    def is_chat(self) -> bool:
+        return True
+
+    @property
+    def type_string(self) -> str:
+        return 'chat'
+
 
 class ResourceContact(CommonContact):
     def __init__(self, logger: LogAdapter, jid: JID, account: str) -> None:
@@ -539,6 +639,10 @@ class ResourceContact(CommonContact):
         if notify:
             self.notify('presence-update')
 
+    @property
+    def type_string(self) -> str:
+        raise NotImplementedError
+
 
 class GroupchatContact(CommonContact):
     def __init__(self, logger: LogAdapter, jid: JID, account: str) -> None:
@@ -551,7 +655,47 @@ class GroupchatContact(CommonContact):
     def is_groupchat(self) -> bool:
         return True
 
+    @property
+    def is_irc(self) -> bool:
+        disco_info = self.get_disco()
+        if disco_info is None:
+            return False
+        return disco_info.is_irc
+
+    @property
+    def muc_context(self) -> Optional[str]:
+        disco_info = self.get_disco()
+        if disco_info is None:
+            return None
+
+        if disco_info.muc_is_members_only and disco_info.muc_is_nonanonymous:
+            return 'private'
+        return 'public'
+
+    @property
+    def encryption_available(self) -> bool:
+        disco_info = self.get_disco()
+        if disco_info is None:
+            return True
+
+        return (disco_info.muc_is_members_only and
+                disco_info.muc_is_nonanonymous)
+
+    def get_config_value(self, field_name: str) -> Any:
+        disco_info = self.get_disco()
+        assert disco_info is not None
+        return disco_info.get_field_value(Namespace.MUC_INFO, field_name)
+
     def add_resource(self, resource: str) -> GroupchatParticipant:
+        assert resource is not None
+        # Check if resource is not None because not the whole
+        # codebase is type checked and it creates hard to track
+        # problems if we create a GroupchatParticipant without resource
+
+        contact = self._resources.get(resource)
+        if contact is not None:
+            return contact
+
         jid = self._jid.new_with(resource=resource)
         assert isinstance(self._log, LogAdapter)
         contact = GroupchatParticipant(self._log, jid, self._account)
@@ -570,6 +714,11 @@ class GroupchatContact(CommonContact):
             contact = self.add_resource(resource)
         return contact
 
+    def get_participants(self) -> Iterator[GroupchatParticipant]:
+        for contact in self._resources.values():
+            if contact.is_available:
+                yield contact
+
     @property
     def name(self) -> str:
         client = app.get_client(self._account)
@@ -658,7 +807,7 @@ class GroupchatContact(CommonContact):
 
     def set_not_joined(self) -> None:
         for contact in self._resources.values():
-            contact.update_presence(UNKNOWN_MUC_PRESENCE, notify=False)
+            contact.update_presence(UNKNOWN_MUC_PRESENCE)
 
     def get_user_nicknames(self) -> list[str]:
         client = app.get_client(self._account)
@@ -672,6 +821,10 @@ class GroupchatContact(CommonContact):
         room = self.settings.get('notify_on_all_messages')
         return all_ or room
 
+    @property
+    def type_string(self) -> str:
+        return 'groupchat'
+
 
 class GroupchatParticipant(CommonContact):
     def __init__(self, logger: LogAdapter, jid: JID, account: str) -> None:
@@ -707,6 +860,14 @@ class GroupchatParticipant(CommonContact):
     def is_in_roster(self) -> bool:
         return False
 
+    @property
+    def room(self) -> GroupchatContact:
+        return self._module('Contacts').get_bare_contact(self.jid.bare)
+
+    @property
+    def muc_context(self) -> Optional[str]:
+        return self.room.muc_context
+
     @property
     def presence(self) -> MUCPresenceData:
         return self._presence
@@ -775,44 +936,17 @@ class GroupchatParticipant(CommonContact):
         return app.app.avatar_storage.get_surface(
             self, size, scale, show, style=style)
 
-    def update_presence(self,
-                        presence: MUCPresenceData,
-                        *args: Any,
-                        notify: bool = True
-                        ) -> None:
-        if not notify:
-            self._presence = presence
-            return
-
-        if not self._presence.available and presence.available:
-            self._presence = presence
-            self.notify('user-joined', *args)
-            return
-
-        if not presence.available:
-            self._presence = presence
-            self.notify('user-left', *args)
-            return
-
-        signals: list[str] = []
-        if self._presence.affiliation != presence.affiliation:
-            signals.append('user-affiliation-changed')
-
-        if self._presence.role != presence.role:
-            signals.append('user-role-changed')
-
-        if (self._presence.status != presence.status or
-                self._presence.show != presence.show):
-            signals.append('user-status-show-changed')
-
+    def update_presence(self, presence: MUCPresenceData) -> None:
         self._presence = presence
-        for signal in signals:
-            self.notify(signal, *args)
 
     def update_avatar(self, *args: Any) -> None:
         app.app.avatar_storage.invalidate_cache(self._jid)
         self.notify('user-avatar-update')
 
+    @property
+    def type_string(self) -> str:
+        return 'pm'
+
 
 def can_add_to_roster(contact: Union[BareContact,
                                      GroupchatContact,
