@@ -32,13 +32,12 @@ from gajim.common import types
 from gajim.common.events import GcMessageReceived
 from gajim.common.events import MessageError
 from gajim.common.events import MessageReceived
-from gajim.common.events import MessageUpdated
 from gajim.common.events import RawMessageReceived
 from gajim.common.helpers import AdditionalDataDict
 from gajim.common.const import KindConstant
 from gajim.common.modules.base import BaseModule
+from gajim.common.modules.util import check_if_message_correction
 from gajim.common.modules.util import get_eme_message
-from gajim.common.modules.misc import parse_correction
 from gajim.common.modules.misc import parse_oob
 from gajim.common.modules.misc import parse_xhtml
 from gajim.common.structs import OutgoingMessage
@@ -127,18 +126,6 @@ class Message(BaseModule):
 
         stanza_id, message_id = self._get_unique_id(properties)
 
-        if properties.type.is_groupchat and properties.has_server_delay:
-            # Only for XEP-0045 MUC History
-            # Don’t check for message text because the message could be
-            # encrypted.
-            if app.storage.archive.deduplicate_muc_message(
-                    self._account,
-                    properties.jid.bare,
-                    properties.jid.resource,
-                    properties.timestamp,
-                    properties.id):
-                raise nbxmpp.NodeProcessed
-
         if (properties.is_self_message or properties.is_muc_pm):
             archive_jid = self._con.get_own_jid().bare
             if app.storage.archive.find_stanza_id(
@@ -151,8 +138,6 @@ class Message(BaseModule):
 
         msgtxt = properties.body
 
-        session = None
-
         additional_data = AdditionalDataDict()
 
         if properties.has_user_delay:
@@ -183,30 +168,26 @@ class Message(BaseModule):
             'stanza_id': stanza_id,
             'unique_id': stanza_id or message_id,
             'msgtxt': msgtxt,
-            'session': session,
             'delayed': properties.user_timestamp is not None,
             'msg_log_id': None,
             'displaymarking': displaymarking,
             'properties': properties,
         }
 
-        correct_id = parse_correction(properties)
-        if correct_id is not None:
-            nickname = properties.muc_nickname or properties.nickname
-            event = MessageUpdated(account=self._account,
-                                   jid=event_attr['jid'],
-                                   msgtxt=msgtxt,
-                                   nickname=nickname,
-                                   properties=properties,
-                                   correct_id=correct_id)
+        if type_.is_groupchat:
+            kind = KindConstant.GC_MSG
+        elif properties.is_sent_carbon:
+            kind = KindConstant.CHAT_MSG_SENT
+        else:
+            kind = KindConstant.CHAT_MSG_RECV
 
-            app.storage.archive.store_message_correction(
-                self._account,
-                jid,
-                correct_id,
-                msgtxt,
-                properties.type.is_groupchat)
-            app.ged.raise_event(event)
+        if check_if_message_correction(properties,
+                                       self._account,
+                                       from_,
+                                       msgtxt,
+                                       kind,
+                                       properties.timestamp,
+                                       self._log):
             return
 
         if type_.is_groupchat:
@@ -216,32 +197,34 @@ class Message(BaseModule):
             event_attr.update({
                 'room_jid': jid,
             })
+
             event = GcMessageReceived(**event_attr)
-            app.ged.raise_event(event)
+
             # TODO: Some plugins modify msgtxt in the GUI event
-            self._log_muc_message(event)
+            msg_log_id = self._log_muc_message(event)
+            event.msg_log_id = msg_log_id
+            app.ged.raise_event(event)
             return
 
-        app.ged.raise_event(MessageReceived(**event_attr))
-
-        log_type = KindConstant.CHAT_MSG_RECV
-        if properties.is_sent_carbon:
-            log_type = KindConstant.CHAT_MSG_SENT
-
+        event = MessageReceived(**event_attr)
         if not msgtxt:
+            app.ged.raise_event(event)
             return
 
-        app.storage.archive.insert_into_logs(
+        msg_log_id = app.storage.archive.insert_into_logs(
             self._account,
             fjid if properties.is_muc_pm else jid,
             properties.timestamp,
-            log_type,
+            kind,
             message=msgtxt,
             subject=properties.subject,
             additional_data=additional_data,
             stanza_id=stanza_id or message_id,
             message_id=properties.id)
 
+        event.msg_log_id = msg_log_id
+        app.ged.raise_event(event)
+
     def _message_error_received(self,
                                 _con: types.xmppClient,
                                 _stanza: nbxmpp.Message,
@@ -266,11 +249,11 @@ class Message(BaseModule):
                          message_id=properties.id,
                          error=properties.error))
 
-    def _log_muc_message(self, event: GcMessageReceived) -> None:
+    def _log_muc_message(self, event: GcMessageReceived) -> Optional[int]:
         self._check_for_mam_compliance(event.room_jid, event.stanza_id)
 
         if event.msgtxt and event.properties.muc_nickname:
-            app.storage.archive.insert_into_logs(
+            msg_log_id = app.storage.archive.insert_into_logs(
                 self._account,
                 event.jid,
                 event.properties.timestamp,
@@ -280,6 +263,9 @@ class Message(BaseModule):
                 additional_data=event.additional_data,
                 stanza_id=event.stanza_id,
                 message_id=event.properties.id)
+            return msg_log_id
+
+        return None
 
     def _check_for_mam_compliance(self, room_jid: str, stanza_id: str) -> None:
         disco_info = app.storage.cache.get_last_disco_info(room_jid)
@@ -391,23 +377,25 @@ class Message(BaseModule):
 
         return stanza
 
-    def log_message(self, message: OutgoingMessage) -> None:
+    def log_message(self, message: OutgoingMessage) -> Optional[int]:
         if not message.is_loggable:
-            return
+            return None
 
         if message.message is None:
-            return
+            return None
 
         if message.correct_id is not None:
-            app.storage.archive.store_message_correction(
+            app.storage.archive.try_message_correction(
                 self._account,
                 message.jid,
-                message.correct_id,
+                None,
                 message.message,
-                message.is_groupchat)
-            return
+                message.correct_id,
+                KindConstant.CHAT_MSG_SENT,
+                message.timestamp)
+            return None
 
-        app.storage.archive.insert_into_logs(
+        msg_log_id = app.storage.archive.insert_into_logs(
             self._account,
             message.jid,
             message.timestamp,
@@ -417,3 +405,4 @@ class Message(BaseModule):
             additional_data=message.additional_data,
             message_id=message.message_id,
             stanza_id=message.message_id)
+        return msg_log_id
