@@ -26,7 +26,6 @@ import time
 
 import nbxmpp
 from nbxmpp.const import InviteType
-from nbxmpp.const import PresenceType
 from nbxmpp.const import StatusCode
 from nbxmpp.errors import StanzaError
 from nbxmpp.modules.dataforms import SimpleDataForm
@@ -34,30 +33,34 @@ from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import JID
 from nbxmpp.protocol import Message
 from nbxmpp.protocol import Presence
+from nbxmpp.structs import CommonError
 from nbxmpp.structs import DiscoInfo
 from nbxmpp.structs import MessageProperties
 from nbxmpp.structs import PresenceProperties
 from nbxmpp.structs import StanzaHandler
+from nbxmpp.structs import VoiceRequest
 from nbxmpp.task import Task
 
 from gi.repository import GLib
 
 from gajim.common import app
+from gajim.common import events
 from gajim.common import helpers
 from gajim.common import types
-from gajim.common.const import ClientState, KindConstant
+from gajim.common.const import ClientState
 from gajim.common.const import MUCJoinedState
 from gajim.common.events import MessageModerated
 from gajim.common.events import MucAdded
 from gajim.common.events import MucDecline
 from gajim.common.events import MucInvitation
-from gajim.common.helpers import AdditionalDataDict
 from gajim.common.helpers import get_default_muc_config
 from gajim.common.helpers import get_group_chat_nick
 from gajim.common.structs import MUCData
 from gajim.common.structs import MUCPresenceData
 from gajim.common.modules.bits_of_binary import store_bob_data
 from gajim.common.modules.base import BaseModule
+from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import GroupchatParticipant
 
 log = logging.getLogger('gajim.c.m.muc')
 
@@ -90,6 +93,12 @@ class MUC(BaseModule):
         self.handlers = [
             StanzaHandler(name='presence',
                           callback=self._on_muc_user_presence,
+                          typ='available',
+                          ns=Namespace.MUC_USER,
+                          priority=49),
+            StanzaHandler(name='presence',
+                          callback=self._on_muc_user_presence,
+                          typ='unavailable',
                           ns=Namespace.MUC_USER,
                           priority=49),
             StanzaHandler(name='presence',
@@ -134,13 +143,14 @@ class MUC(BaseModule):
                                  self._on_client_resume_failed)
 
         self._rejoin_muc: set[str] = set()
-        self._join_timeouts: dict[str, int] = {}
         self._rejoin_timeouts: dict[str, int] = {}
         self._muc_service_jid = None
         self._joined_users: defaultdict[
             str, dict[str, MUCPresenceData]] = defaultdict(dict)
         self._mucs: dict[str, MUCData] = {}
         self._muc_nicknames = {}
+        self._voice_requests: dict[
+            GroupchatContact, list[VoiceRequest]] = defaultdict(list)
 
     def _on_resume_failed(self,
                           _client: types.Client,
@@ -205,12 +215,7 @@ class MUC(BaseModule):
         contact.notify('state-changed')
 
     def _reset_state(self) -> None:
-        for room_jid in list(self._rejoin_timeouts.keys()):
-            self._remove_rejoin_timeout(room_jid)
-
-        for room_jid in list(self._join_timeouts.keys()):
-            self._remove_join_timeout(room_jid)
-
+        self._remove_all_timeouts()
         for muc in self._mucs.values():
             self._joined_users.pop(muc.jid, None)
             self._set_muc_state(muc.jid, MUCJoinedState.NOT_JOINED)
@@ -260,7 +265,8 @@ class MUC(BaseModule):
             # join a different nickname, so update MUCData here.
             muc_data.nick = nick
 
-        if not muc_data.state.is_not_joined:
+        if muc_data.state not in (MUCJoinedState.NOT_JOINED,
+                                  MUCJoinedState.PASSWORD_REQUEST):
             self._log.warning('Can’t join MUC %s, state: %s',
                               jid, muc_data.state)
             return
@@ -355,7 +361,6 @@ class MUC(BaseModule):
         if muc_data.state.is_not_joined:
             return
 
-        self._remove_join_timeout(room_jid)
         self._remove_rejoin_timeout(room_jid)
 
         self._con.get_module('Presence').send_presence(
@@ -369,6 +374,17 @@ class MUC(BaseModule):
         room.set_not_joined()
         room.notify('room-left')
 
+    def abort_join(self, room_jid: str) -> None:
+        self._remove_rejoin_timeout(room_jid)
+
+        muc = self._mucs[room_jid]
+        self._con.get_module('Presence').send_presence(
+            muc.occupant_jid,
+            typ='unavailable',
+            caps=False)
+
+        self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
+
     def configure_room(self, room_jid: str) -> None:
         self._nbxmpp('MUC').request_config(room_jid,
                                            callback=self._on_room_config)
@@ -419,20 +435,6 @@ class MUC(BaseModule):
         self._con.get_module('Discovery').disco_muc(
             result.jid, callback=self._on_disco_result_after_config)
 
-        # If this is an automatic room creation
-        try:
-            invites = app.automatic_rooms[self._account][result.jid]['invities']
-        except KeyError:
-            return
-
-        user_list = {}
-        for jid in invites:
-            user_list[jid] = {'affiliation': 'member'}
-        self.set_affiliation(result.jid, user_list)
-
-        for jid in invites:
-            self.invite(result.jid, jid)
-
     def _on_disco_result_after_config(self, task: Task) -> None:
         try:
             result = task.finish()
@@ -447,7 +449,11 @@ class MUC(BaseModule):
         self._log.info('Configuration finished: %s', jid)
 
         room = self._get_contact(jid.bare)
-        room.notify('room-config-finished')
+        event = events.MUCRoomConfigFinished(timestamp=time.time())
+
+        assert isinstance(room, GroupchatContact)
+        app.storage.events.store(room, event)
+        room.notify('room-config-finished', event)
 
     def update_presence(self) -> None:
         mucs = self._get_mucs_with_state([MUCJoinedState.JOINED,
@@ -473,6 +479,7 @@ class MUC(BaseModule):
                            stanza: Presence,
                            properties: PresenceProperties
                            ) -> None:
+
         room_jid = properties.jid.bare
         muc_data = self._mucs.get(room_jid)
         if muc_data is None:
@@ -495,27 +502,42 @@ class MUC(BaseModule):
 
             elif properties.error.condition == 'not-authorized':
                 self._remove_rejoin_timeout(room_jid)
-                self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
+                self._set_muc_state(room_jid, MUCJoinedState.PASSWORD_REQUEST)
                 room.notify('room-password-required', properties)
 
             else:
                 self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
                 if room_jid not in self._rejoin_muc:
+                    muc_data.error = 'join-failed'
+                    assert isinstance(properties.error, CommonError)
+                    muc_data.error_text = helpers.to_user_string(
+                        properties.error)
                     room.notify('room-join-failed', properties.error)
 
         elif muc_data.state == MUCJoinedState.CREATING:
             self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
+            muc_data.error = 'creation-failed'
+            assert isinstance(properties.error, CommonError)
+            muc_data.error_text = helpers.to_user_string(properties.error)
             room.notify('room-creation-failed', properties)
 
         elif muc_data.state == MUCJoinedState.CAPTCHA_REQUEST:
             self._set_muc_state(room_jid, MUCJoinedState.CAPTCHA_FAILED)
             self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
+            muc_data.error = 'captcha-failed'
+            assert isinstance(properties.error, CommonError)
+            muc_data.error_text = helpers.to_user_string(properties.error)
             room.notify('room-captcha-error', properties.error)
 
         elif muc_data.state == MUCJoinedState.CAPTCHA_FAILED:
             self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
 
         else:
+            event = events.MUCRoomPresenceError(
+                timestamp=time.time(),
+                error=properties.error)
+            assert isinstance(room, GroupchatContact)
+            app.storage.events.store(room, event)
             room.notify('room-presence-error', properties)
 
     def _on_muc_user_presence(self,
@@ -523,8 +545,6 @@ class MUC(BaseModule):
                               stanza: Presence,
                               properties: PresenceProperties
                               ) -> None:
-        if properties.type == PresenceType.ERROR:
-            return
 
         room_jid = str(properties.muc_jid)
         if room_jid not in self._mucs:
@@ -538,11 +558,18 @@ class MUC(BaseModule):
 
         if properties.is_muc_destroyed:
             self._log.info('MUC destroyed: %s', room_jid)
-            self._remove_join_timeout(room_jid)
             self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
             self._con.get_module('Bookmarks').remove(room_jid)
             room.set_not_joined()
-            room.notify('room-destroyed', properties)
+
+            event = events.MUCRoomDestroyed(
+                timestamp=time.time(),
+                reason=properties.muc_destroyed.reason,
+                alternate=properties.muc_destroyed.alternate)
+            assert isinstance(room, GroupchatContact)
+            app.storage.events.store(room, event)
+
+            room.notify('room-destroyed', event)
             return
 
         if properties.is_nickname_changed:
@@ -557,21 +584,29 @@ class MUC(BaseModule):
                            properties.jid,
                            properties.muc_user.nick)
 
-            # Create a copy of the contact object
-            new_occupant = room.add_resource(properties.muc_user.nick)
+            # We receive only the unavailable presence here, so we take
+            # the current presence and create a new contact with it, before we
+            # update the presence.
+            nickname = properties.muc_user.nick
+            new_occupant = room.add_resource(nickname)
             new_occupant.set_presence(occupant.presence)
+            self._joined_users[room.jid][nickname] = occupant.presence
 
             presence = self._process_user_presence(properties)
-            occupant.update_presence(presence, properties)
+            self._process_occupant_presence_change(properties,
+                                                   presence,
+                                                   occupant)
 
-            # new_nick = properties.muc_user.nick
-            # nick = properties.muc_nickname
-            # presence_dict = self._joined_users[properties.jid.bare]
-            # presence_dict[new_nick] = presence_dict.pop(nick)
-            # self._manager.change_nickname
-            # TODO remove presence from manager
+            event = events.MUCNicknameChanged(
+                timestamp=time.time(),
+                is_self=properties.is_muc_self_presence,
+                new_name=new_occupant.name,
+                old_name=occupant.name)
 
-            room.notify('user-nickname-changed', occupant, properties)
+            assert isinstance(room, GroupchatContact)
+            app.storage.events.store(room, event)
+
+            room.notify('user-nickname-changed', event, occupant, new_occupant)
             return
 
         is_joined = self._is_user_joined(properties.jid)
@@ -589,16 +624,25 @@ class MUC(BaseModule):
                     if properties.is_new_room:
                         self.configure_room(room_jid)
 
-                self._start_join_timeout(room_jid)
-
             presence = self._process_user_presence(properties)
-            occupant.update_presence(presence, properties)
+            self._process_occupant_presence_change(properties,
+                                                   presence,
+                                                   occupant)
             return
 
         if properties.is_muc_self_presence and properties.is_kicked:
             self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
             room.set_not_joined()
-            room.notify('room-kicked', properties)
+
+            event = events.MUCRoomKicked(
+                timestamp=time.time(),
+                status_codes=properties.muc_status_codes,
+                reason=properties.muc_user.reason,
+                actor=properties.muc_user.actor)
+            assert isinstance(room, GroupchatContact)
+            app.storage.events.store(room, event)
+            room.notify('room-kicked', event)
+
             status_codes = properties.muc_status_codes or []
             if StatusCode.REMOVED_SERVICE_SHUTDOWN in status_codes:
                 self._start_rejoin_timeout(room_jid)
@@ -609,8 +653,101 @@ class MUC(BaseModule):
             # unavailable presence, because we left the MUC
             return
 
-        presence = self._process_user_presence(properties)
-        occupant.update_presence(presence, properties)
+        try:
+            presence = self._process_user_presence(properties)
+        except KeyError:
+            # Sometimes it seems to happen that we get unavailable presence
+            # from occupants we don’t know
+            log.warning('Unexpected presence received')
+            log.warning(stanza)
+            return
+
+        self._process_occupant_presence_change(properties, presence, occupant)
+
+    def _process_occupant_presence_change(
+            self,
+            properties: PresenceProperties,
+            presence: MUCPresenceData,
+            occupant: GroupchatParticipant) -> None:
+
+        timestamp = time.time()
+
+        if not occupant.is_available and presence.available:
+
+            event = events.MUCUserJoined(
+                timestamp=timestamp,
+                is_self=properties.is_muc_self_presence,
+                nick=occupant.name,
+                status_codes=properties.muc_status_codes)
+
+            if occupant.room.is_joined or properties.is_muc_self_presence:
+                # Don’t store initial presences on join
+                app.storage.events.store(occupant.room, event)
+
+            occupant.update_presence(presence)
+            occupant.notify('user-joined', event)
+            return
+
+        if not presence.available:
+
+            event = events.MUCUserLeft(
+                timestamp=timestamp,
+                is_self=properties.is_muc_self_presence,
+                nick=occupant.name,
+                status_codes=properties.muc_status_codes,
+                reason=properties.muc_user.reason,
+                actor=properties.muc_user.actor)
+
+            app.storage.events.store(occupant.room, event)
+            occupant.update_presence(presence)
+            occupant.notify('user-left', event)
+            return
+
+        signals_and_events: list[tuple[str, Any]] = []
+
+        if occupant.affiliation != presence.affiliation:
+
+            event = events.MUCUserAffiliationChanged(
+                timestamp=timestamp,
+                is_self=properties.is_muc_self_presence,
+                nick=occupant.name,
+                affiliation=presence.affiliation,
+                reason=properties.muc_user.reason,
+                actor=properties.muc_user.actor)
+
+            app.storage.events.store(occupant.room, event)
+            signals_and_events.append(('user-affiliation-changed', event))
+
+        if occupant.role != presence.role:
+
+            event = events.MUCUserRoleChanged(
+                timestamp=timestamp,
+                is_self=properties.is_muc_self_presence,
+                nick=occupant.name,
+                role=presence.role,
+                reason=properties.muc_user.reason,
+                actor=properties.muc_user.actor)
+
+            app.storage.events.store(occupant.room, event)
+            signals_and_events.append(('user-role-changed', event))
+
+        if (occupant.status != presence.status or
+                occupant.show != presence.show):
+
+            event = events.MUCUserStatusShowChanged(
+                timestamp=timestamp,
+                is_self=properties.is_muc_self_presence,
+                nick=occupant.name,
+                status=properties.status,
+                show_value=properties.show.value)
+
+            app.storage.events.store(occupant, event)
+            app.storage.events.store(occupant.room, event)
+            signals_and_events.append(('user-status-show-changed', event))
+
+        occupant.update_presence(presence)
+        for signal, event in signals_and_events:
+            occupant.notify(signal, event)
 
     def _process_user_presence(self,
                                properties: PresenceProperties
@@ -648,61 +785,12 @@ class MUC(BaseModule):
             GLib.source_remove(id_)
             del self._rejoin_timeouts[room_jid]
 
-    def _start_join_timeout(self, room_jid: str) -> None:
-        self._remove_join_timeout(room_jid)
-        self._log.info('Start join timeout for: %s', room_jid)
-        id_ = GLib.timeout_add_seconds(
-            10, self._fake_subject_change, room_jid)
-        self._join_timeouts[room_jid] = id_
-
-    def _remove_join_timeout(self, room_jid: str) -> None:
-        id_ = self._join_timeouts.get(room_jid)
-        if id_ is not None:
-            self._log.info('Remove join timeout for: %s', room_jid)
-            GLib.source_remove(id_)
-            del self._join_timeouts[room_jid]
-
-    def _log_muc_event(self,
-                       event_name: str,
-                       properties: PresenceProperties
-                       ) -> None:
-        # TODO CURRENTLY NOT USED
-        if event_name not in ['muc-user-joined',
-                              'muc-user-left',
-                              'muc-user-status-show-changed']:
-            return
-
-        if not app.settings.get('log_contact_status_changes'):
-            return
-
-        additional_data = AdditionalDataDict()
-        if properties.muc_user is not None:
-            if properties.muc_user.jid is not None:
-                additional_data.set_value(
-                    'gajim', 'real_jid', str(properties.muc_user.jid))
-
-        # TODO: Refactor
-        if properties.type == PresenceType.UNAVAILABLE:
-            show = 'offline'
-        else:
-            show = properties.show.value
-        show = app.storage.archive.convert_show_values_to_db_api_values(show)
-
-        app.storage.archive.insert_into_logs(
-            self._account,
-            properties.jid.bare,
-            properties.timestamp,
-            KindConstant.GCSTATUS,
-            contact_name=properties.muc_nickname,
-            message=properties.status or None,
-            show=show,
-            additional_data=additional_data)
-
     def _on_subject_change(self,
                            _con: types.xmppClient,
                            _stanza: Message,
                            properties: MessageProperties
                            ) -> None:
+
         if not properties.is_muc_subject:
             return
 
@@ -713,13 +801,26 @@ class MUC(BaseModule):
             self._log.warning('No MUCData found for %s', room_jid)
             return
 
+        room = self._get_contact(JID.from_string(room_jid))
+        assert isinstance(room, GroupchatContact)
+
+        old_subject = muc_data.subject
+
         muc_subject = properties.muc_subject
-        if muc_subject is not None and muc_subject.timestamp is None:
+        assert muc_subject is not None
+        muc_data.subject = muc_subject
+
+        if muc_subject.timestamp is None:
             muc_subject = muc_subject._replace(timestamp=time.time())
 
-        muc_data.subject = muc_subject
-        room = self._get_contact(room_jid)
-        room.notify('room-subject', muc_subject)
+        if old_subject is None:
+            muc_data.last_subject_timestamp = time.time()
+            room.notify('room-subject', muc_subject)
+
+        elif old_subject.text != muc_subject.text:
+            # Check if we already showed that subject (rejoin)
+            muc_data.last_subject_timestamp = time.time()
+            room.notify('room-subject', muc_subject)
 
         if muc_data.state == MUCJoinedState.JOINING:
             self._room_join_complete(muc_data)
@@ -745,16 +846,10 @@ class MUC(BaseModule):
 
         raise nbxmpp.NodeProcessed
 
-    def _fake_subject_change(self, room_jid: str) -> None:
-        # This is for servers which don’t send empty subjects as part of the
-        # event order on joining a MUC. For example jabber.ru
-        self._log.warning('Fake subject received for %s', room_jid)
-        del self._join_timeouts[room_jid]
-        room = self._get_contact(room_jid)
-        room.notify('room-joined')
+    def cancel_password_request(self, room_jid: str) -> None:
+        self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
 
     def _room_join_complete(self, muc_data: MUCData):
-        self._remove_join_timeout(muc_data.jid)
         self._set_muc_state(muc_data.jid, MUCJoinedState.JOINED)
         self._remove_rejoin_timeout(muc_data.jid)
 
@@ -779,10 +874,35 @@ class MUC(BaseModule):
             return
 
         room = self._get_contact(properties.jid.bare)
+        assert isinstance(room, GroupchatContact)
+        assert properties.voice_request is not None
+
+        self._voice_requests[room].append(properties.voice_request)
         room.notify('room-voice-request', properties)
 
         raise nbxmpp.NodeProcessed
 
+    def get_voice_requests(self,
+                           contact: GroupchatContact
+                           ) -> list[VoiceRequest]:
+
+        return self._voice_requests.get(contact, [])
+
+    def approve_voice_request(self,
+                              contact: GroupchatContact,
+                              voice_request: VoiceRequest
+                              ) -> None:
+
+        self._voice_requests[contact].remove(voice_request)
+        self._nbxmpp('MUC').approve_voice_request(contact.jid, voice_request)
+
+    def decline_voice_request(self,
+                              contact: GroupchatContact,
+                              voice_request: VoiceRequest
+                              ) -> None:
+
+        self._voice_requests[contact].remove(voice_request)
+
     def _on_captcha_challenge(self,
                               _con: types.xmppClient,
                               _stanza: Message,
@@ -806,8 +926,11 @@ class MUC(BaseModule):
             return
 
         self._log.info('Captcha challenge received from %s', properties.jid)
+
+        assert properties.captcha is not None
         store_bob_data(properties.captcha.bob_data)
         muc_data.captcha_id = properties.id
+        muc_data.captcha_form = properties.captcha.form
 
         self._set_muc_state(properties.jid, MUCJoinedState.CAPTCHA_REQUEST)
         self._remove_rejoin_timeout(properties.jid)
@@ -857,12 +980,21 @@ class MUC(BaseModule):
         if not properties.is_muc_config_change:
             return
 
-        room_jid = str(properties.muc_jid)
         self._log.info('Received config change: %s %s',
-                       room_jid, properties.muc_status_codes)
+                       properties.muc_jid, properties.muc_status_codes)
 
-        room = self._get_contact(room_jid)
-        room.notify('room-config-changed', properties)
+        assert properties.muc_status_codes is not None
+        event = events.MUCRoomConfigChanged(
+            timestamp=time.time(),
+            status_codes=properties.muc_status_codes)
+
+        assert properties.muc_jid is not None
+        room = self._get_contact(properties.muc_jid)
+
+        assert isinstance(room, GroupchatContact)
+        app.storage.events.store(room, event)
+
+        room.notify('room-config-changed', event)
 
         raise nbxmpp.NodeProcessed
 
@@ -871,10 +1003,10 @@ class MUC(BaseModule):
                               _stanza: Message,
                               properties: MessageProperties
                               ) -> None:
+
         if properties.muc_decline is not None:
             data = properties.muc_decline
-            if helpers.ignore_contact(self._account, data.from_):
-                raise nbxmpp.NodeProcessed
+            contact = self._get_contact(data.muc, groupchat=True)
 
             self._log.info('Invite declined from: %s, reason: %s',
                            data.from_, data.reason)
@@ -886,12 +1018,10 @@ class MUC(BaseModule):
 
         if properties.muc_invite is not None:
             data = properties.muc_invite
-            if helpers.ignore_contact(self._account, data.from_):
-                raise nbxmpp.NodeProcessed
+            contact = self._get_contact(data.muc, groupchat=True)
 
             self._log.info('Invite from: %s, to: %s', data.from_, data.muc)
 
-            contact = self._get_contact(data.muc, groupchat=True)
             if contact.is_joined:
                 # We are already in groupchat. Ignore invitation
                 self._log.info('We are already in this room')
@@ -919,20 +1049,35 @@ class MUC(BaseModule):
                           **invite_data._asdict()))
 
     def invite(self,
-               room: str,
-               jid: str,
+               room: JID,
+               jid: JID,
                reason: Optional[str] = None,
                continue_: bool = False
                ) -> str:
-        type_ = InviteType.MEDIATED
-        contact = self._get_contact(jid)
-        if contact and contact.supports(Namespace.CONFERENCE):
+
+        room_contact = self._get_contact(room, groupchat=True)
+        assert isinstance(room_contact, GroupchatContact)
+        disco = room_contact.get_disco()
+        assert disco is not None
+
+        if disco.muc_is_members_only:
+            self_contact = room_contact.get_self()
+            assert self_contact is not None
+            affiliation = self_contact.affiliation
+            admin = affiliation.is_owner or affiliation.is_admin
+            if admin:
+                self.set_affiliation(
+                    room, {jid: {'affiliation': 'member'}})
+                type_ = InviteType.DIRECT
+            else:
+                type_ = InviteType.MEDIATED
+        else:
             type_ = InviteType.DIRECT
 
-        password = self._mucs[room].password
+        password = self._mucs[str(room)].password
         self._log.info('Invite %s to %s', jid, room)
         return self._nbxmpp('MUC').invite(
-            room, jid, reason, password, continue_, type_)
+            room, jid, password, reason, continue_, type_)
 
     def _on_client_state_changed(self,
                                  _client: types.Client,
@@ -947,3 +1092,11 @@ class MUC(BaseModule):
                                  _signal_name: str
                                  ) -> None:
         self._reset_state()
+
+    def _remove_all_timeouts(self) -> None:
+        for room_jid in list(self._rejoin_timeouts.keys()):
+            self._remove_rejoin_timeout(room_jid)
+
+    def cleanup(self) -> None:
+        super().cleanup()
+        self._remove_all_timeouts()
