@@ -22,6 +22,7 @@
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 from typing import Any
+from typing import Literal
 from typing import Iterator
 from typing import KeysView
 from typing import NamedTuple
@@ -42,14 +43,15 @@ from nbxmpp.structs import MessageProperties
 from gajim.common import app
 from gajim.common import configpaths
 from gajim.common.helpers import AdditionalDataDict
-from gajim.common.const import ShowConstant
+from gajim.common.const import MAX_MESSAGE_CORRECTION_DELAY
 from gajim.common.const import KindConstant
 from gajim.common.const import JIDConstant
+from gajim.common.modules.contacts import GroupchatContact
 
 from gajim.common.storage.base import SqliteStorage
 from gajim.common.storage.base import timeit
 
-CURRENT_USER_VERSION = 6
+CURRENT_USER_VERSION = 7
 
 ARCHIVE_SQL_STATEMENT = '''
     CREATE TABLE jids(
@@ -62,6 +64,8 @@ ARCHIVE_SQL_STATEMENT = '''
             account_id INTEGER,
             jid_id INTEGER,
             contact_name TEXT,
+            occupant_id TEXT,
+            real_jid TEXT,
             time INTEGER,
             kind INTEGER,
             show INTEGER,
@@ -177,8 +181,7 @@ class MessageArchiveStorage(SqliteStorage):
 
         self._con.row_factory = self._namedtuple_factory
 
-        self._con.create_function("like", 1, self._like)
-        self._con.create_function("get_timeout", 0, self._get_timeout)
+        self._con.create_function('like', 1, self._like)
 
         self._get_jid_ids_from_db()
 
@@ -188,7 +191,7 @@ class MessageArchiveStorage(SqliteStorage):
 
         assert cursor.description is not None
         fields = [col[0] for col in cursor.description]
-        Row = namedtuple("Row", fields)  # type: ignore
+        Row = namedtuple('Row', fields)  # type: ignore
         named_row = Row(*row)
         if 'additional_data' in fields:
             _dict = json.loads(named_row.additional_data or '{}')
@@ -222,7 +225,7 @@ class MessageArchiveStorage(SqliteStorage):
                     )''',
 
                 '''CREATE INDEX IF NOT EXISTS idx_logs_stanza_id
-                    ON logs(stanza_id)''',
+                   ON logs(stanza_id)''',
                 'PRAGMA user_version=1'
             ]
 
@@ -230,7 +233,8 @@ class MessageArchiveStorage(SqliteStorage):
 
         if user_version < 2:
             statements = [
-                'ALTER TABLE last_archive_message ADD COLUMN "sync_threshold" INTEGER',
+                ('ALTER TABLE last_archive_message '
+                 'ADD COLUMN "sync_threshold" INTEGER'),
                 'PRAGMA user_version=2'
             ]
             self._execute_multiple(statements)
@@ -251,22 +255,19 @@ class MessageArchiveStorage(SqliteStorage):
 
         if user_version < 5:
             statements = [
-                'CREATE INDEX idx_logs_message_id ON logs (message_id)',
+                '''CREATE INDEX IF NOT EXISTS idx_logs_message_id
+                   ON logs (message_id)''',
                 'PRAGMA user_version=5'
             ]
             self._execute_multiple(statements)
 
-    @staticmethod
-    def _get_timeout() -> int:
-        """
-        returns the timeout in epoch
-        """
-        timeout = app.settings.get('restore_timeout')
-
-        now = int(time.time())
-        if timeout > 0:
-            timeout = now - (timeout * 60)
-        return timeout
+        if user_version < 7:
+            statements = [
+                'ALTER TABLE logs ADD COLUMN "real_jid" TEXT',
+                'ALTER TABLE logs ADD COLUMN "occupant_id" TEXT',
+                'PRAGMA user_version=7'
+            ]
+            self._execute_multiple(statements)
 
     @staticmethod
     def _like(search_str: str) -> str:
@@ -274,9 +275,9 @@ class MessageArchiveStorage(SqliteStorage):
 
     @timeit
     def _get_jid_ids_from_db(self) -> None:
-        """
+        '''
         Load all jid/jid_id tuples into a dict for faster access
-        """
+        '''
         rows = self._con.execute(
             'SELECT jid_id, jid, type FROM jids').fetchall()
         for row in rows:
@@ -309,7 +310,7 @@ class MessageArchiveStorage(SqliteStorage):
                    kind: Optional[KindConstant] = None,
                    type_: Optional[JIDConstant] = None
                    ) -> int:
-        """
+        '''
         Get the jid id from a jid.
         In case the jid id is not found create a new one.
 
@@ -320,7 +321,7 @@ class MessageArchiveStorage(SqliteStorage):
         :param type_:   The JIDConstant
 
         return the jid id
-        """
+        '''
 
         if kind in (KindConstant.GC_MSG, KindConstant.GCSTATUS):
             type_ = JIDConstant.ROOM_TYPE
@@ -350,31 +351,6 @@ class MessageArchiveStorage(SqliteStorage):
         self._delayed_commit()
         return lastrowid
 
-    @staticmethod
-    def convert_show_values_to_db_api_values(show: Optional[str]
-                                             ) -> Optional[ShowConstant]:
-        """
-        Convert from string style to constant ints for db
-        """
-
-        if show == 'online':
-            return ShowConstant.ONLINE
-        if show == 'chat':
-            return ShowConstant.CHAT
-        if show == 'away':
-            return ShowConstant.AWAY
-        if show == 'xa':
-            return ShowConstant.XA
-        if show == 'dnd':
-            return ShowConstant.DND
-        if show == 'offline':
-            return ShowConstant.OFFLINE
-        if show is None:
-            return ShowConstant.ONLINE
-        # invisible in GC when someone goes invisible
-        # it's a RFC violation .... but we should not crash
-        return None
-
     @timeit
     def get_conversation_before_after(self,
                                       account: str,
@@ -383,7 +359,7 @@ class MessageArchiveStorage(SqliteStorage):
                                       timestamp: float,
                                       n_lines: int
                                       ) -> list[ConversationRow]:
-        """
+        '''
         Load n_lines lines of conversation with jid before or after timestamp
 
         :param account:         The account
@@ -395,7 +371,7 @@ class MessageArchiveStorage(SqliteStorage):
         :param timestamp:       timestamp
 
         returns a list of namedtuples
-        """
+        '''
         jids = [jid]
         account_id = self.get_account_id(account)
         kinds = map(str, [KindConstant.ERROR,
@@ -430,7 +406,7 @@ class MessageArchiveStorage(SqliteStorage):
                                    account: str,
                                    jid: JID
                                    ) -> LastConversationRow:
-        """
+        '''
         Load the last line of a conversation with jid for account.
         Loads messages, but no status messages or error messages.
 
@@ -439,7 +415,7 @@ class MessageArchiveStorage(SqliteStorage):
         :param jid:             The jid for which we request the conversation
 
         returns a list of namedtuples
-        """
+        '''
         jids = [jid]
         account_id = self.get_account_id(account)
 
@@ -467,7 +443,7 @@ class MessageArchiveStorage(SqliteStorage):
                                 timestamp: float
                                 ) -> tuple[list[ConversationRow],
                                            list[ConversationRow]]:
-        """
+        '''
         Load all lines of conversation with jid around a specific timestamp
 
         :param account:         The account
@@ -477,7 +453,7 @@ class MessageArchiveStorage(SqliteStorage):
         :param timestamp:       Timestamp around which to fetch messages
 
         returns a list of namedtuples
-        """
+        '''
         jids = [jid]
         account_id = self.get_account_id(account)
         kinds = map(str, [KindConstant.ERROR])
@@ -525,7 +501,7 @@ class MessageArchiveStorage(SqliteStorage):
                                  jid: str,
                                  before: float,
                                  after: float) -> list[ConversationRow]:
-        """
+        '''
         Load all lines of conversation with jid between two timestamps
 
         :param account:         The account
@@ -537,7 +513,7 @@ class MessageArchiveStorage(SqliteStorage):
         :param after:           earliest timestamp
 
         returns a list of namedtuples
-        """
+        '''
         jids = [jid]
         account_id = self.get_account_id(account)
         kinds = map(str, [KindConstant.ERROR])
@@ -562,14 +538,14 @@ class MessageArchiveStorage(SqliteStorage):
 
     @timeit
     def search_log(self,
-                   _account: str,
+                   account: str,
                    jid: JID,
                    query: str,
                    from_users: Optional[list[str]] = None,
                    before: Optional[datetime.datetime] = None,
                    after: Optional[datetime.datetime] = None
                    ) -> Iterator[SearchLogRow]:
-        """
+        '''
         Search the conversation log for messages containing the `query` string.
 
         The search can either span the complete log for the given
@@ -589,7 +565,8 @@ class MessageArchiveStorage(SqliteStorage):
         :param after: A datetime.datetime instance or None
 
         returns a list of namedtuples
-        """
+        '''
+        account_id = self.get_account_id(account)
         jids = [jid]
 
         kinds = map(str, [KindConstant.STATUS,
@@ -600,9 +577,8 @@ class MessageArchiveStorage(SqliteStorage):
         else:
             before_ts = before.timestamp()
 
-        if after is None:
-            after_ts = datetime.datetime(1971, 1, 1).timestamp()
-        else:
+        after_ts = 0
+        if after is not None:
             after_ts = after.timestamp()
 
         if from_users is None:
@@ -614,12 +590,14 @@ class MessageArchiveStorage(SqliteStorage):
             SELECT account_id, jid_id, contact_name, time, kind, show, message,
                    subject, additional_data, log_line_id
             FROM logs NATURAL JOIN jids WHERE jid IN ({jids})
+            AND account_id = {account_id}
             AND message LIKE like(?)
             AND kind NOT IN ({kinds})
             {users_query}
             AND time BETWEEN ? AND ?
             ORDER BY time DESC, log_line_id
             '''.format(jids=', '.join('?' * len(jids)),
+                       account_id=account_id,
                        kinds=', '.join(kinds),
                        users_query=users_query_string)
 
@@ -646,7 +624,6 @@ class MessageArchiveStorage(SqliteStorage):
             for result in results:
                 yield result
 
-
     @timeit
     def search_all_logs(self,
                         query: str,
@@ -654,7 +631,7 @@ class MessageArchiveStorage(SqliteStorage):
                         before: Optional[datetime.datetime] = None,
                         after: Optional[datetime.datetime] = None
                         ) -> Iterator[SearchLogRow]:
-        """
+        '''
         Search all conversation logs for messages containing the `query`
         string.
 
@@ -667,7 +644,7 @@ class MessageArchiveStorage(SqliteStorage):
         :param after: A datetime.datetime instance or None
 
         returns a list of namedtuples
-        """
+        '''
         account_ids = self.get_active_account_ids()
         kinds = map(str, [KindConstant.STATUS,
                           KindConstant.GCSTATUS])
@@ -677,9 +654,8 @@ class MessageArchiveStorage(SqliteStorage):
         else:
             before_ts = before.timestamp()
 
-        if after is None:
-            after_ts = datetime.datetime(1971, 1, 1).timestamp()
-        else:
+        after_ts = 0
+        if after is not None:
             after_ts = after.timestamp()
 
         if from_users is None:
@@ -726,9 +702,9 @@ class MessageArchiveStorage(SqliteStorage):
                               year: int,
                               month: int
                               ) -> list[HistoryDayRow]:
-        """
+        '''
         Get days in month where messages for 'jid' exist
-        """
+        '''
         jids = [jid]
         account_id = self.get_account_id(account)
         kinds = map(str, [KindConstant.STATUS,
@@ -740,7 +716,7 @@ class MessageArchiveStorage(SqliteStorage):
         delta = datetime.timedelta(
             days=days, hours=23, minutes=59, seconds=59, microseconds=999999)
 
-        sql = """
+        sql = '''
             SELECT DISTINCT
             CAST(strftime('%d', time, 'unixepoch', 'localtime') AS INTEGER)
             AS day FROM logs NATURAL JOIN jids WHERE jid IN ({jids})
@@ -748,22 +724,23 @@ class MessageArchiveStorage(SqliteStorage):
             AND time BETWEEN ? AND ?
             AND kind NOT IN ({kinds})
             ORDER BY time
-            """.format(jids=', '.join('?' * len(jids)),
+            '''.format(jids=', '.join('?' * len(jids)),
                        account_id=account_id,
                        kinds=', '.join(kinds))
 
-        return self._con.execute(sql, tuple(jids) +
-                                      (date.timestamp(),
-                                      (date + delta).timestamp())).fetchall()
+        return self._con.execute(
+            sql,
+            tuple(jids) + (date.timestamp(),
+                           (date + delta).timestamp())).fetchall()
 
     @timeit
     def get_last_history_timestamp(self,
                                    account: str,
                                    jid: str
                                    ) -> Optional[float]:
-        """
+        '''
         Get the timestamp of the last message we received for the jid
-        """
+        '''
         jids = [jid]
         account_id = self.get_account_id(account)
         kinds = map(str, [KindConstant.STATUS,
@@ -787,9 +764,9 @@ class MessageArchiveStorage(SqliteStorage):
                                     account: str,
                                     jid: str
                                     ) -> Optional[float]:
-        """
+        '''
         Get the timestamp of the first message we received for the jid
-        """
+        '''
         jids = [jid]
         account_id = self.get_account_id(account)
         kinds = map(str, [KindConstant.STATUS,
@@ -814,10 +791,10 @@ class MessageArchiveStorage(SqliteStorage):
                          jid: str,
                          date: datetime.datetime
                          ) -> Optional[float]:
-        """
+        '''
         Get a single timestamp of a message for 'jid'
         in time range of one day
-        """
+        '''
         jids = [jid]
         account_id = self.get_account_id(account)
         delta = datetime.timedelta(
@@ -843,9 +820,9 @@ class MessageArchiveStorage(SqliteStorage):
                                         jid: str,
                                         date: datetime.datetime
                                         ) -> Optional[MessageMetaRow]:
-        """
+        '''
         Load meta data for the first message of a specific date
-        """
+        '''
         jids = [jid]
         account_id = self.get_account_id(account)
 
@@ -868,57 +845,36 @@ class MessageArchiveStorage(SqliteStorage):
             tuple(jids) + (date_ts, delta_ts)).fetchone()
 
     @timeit
-    def deduplicate_muc_message(self,
-                                account: str,
-                                jid: str,
-                                resource: str,
-                                timestamp: float,
-                                message_id: str
-                                ) -> bool:
-        """
-        Check if a message is already in the `logs` table
-
-        :param account:     The account
-
-        :param jid:         The muc jid as string
-
-        :param resource:    The resource
-
-        :param timestamp:   The timestamp in UTC epoch
-
-        :param message_id:  The message-id
-        """
-
-        # Add 60 seconds around the timestamp
-        start_time = timestamp - 60
-        end_time = timestamp + 60
-
-        account_id = self.get_account_id(account)
-        log.debug('Search for MUC duplicate')
-        log.debug('start: %s, end: %s, jid: %s, resource: %s, message-id: %s',
-                  start_time, end_time, jid, resource, message_id)
+    def get_recent_muc_nicks(self, contact: GroupchatContact) -> list[str]:
+        '''
+        Queries the last 50 message rows and gathers nicknames in a list
+        '''
+        jids = [contact.jid]
+        account_id = self.get_account_id(contact.account)
+        kinds = map(str, [KindConstant.STATUS,
+                          KindConstant.GCSTATUS,
+                          KindConstant.ERROR])
 
         sql = '''
-            SELECT * FROM logs
-            NATURAL JOIN jids WHERE
-            jid = ? AND
-            contact_name = ? AND
-            message_id = ? AND
-            account_id = ? AND
-            time BETWEEN ? AND ?
-            '''
+            SELECT contact_name
+            FROM logs NATURAL JOIN jids WHERE jid IN ({jids})
+            AND account_id = {account_id}
+            AND kind NOT IN ({kinds})
+            ORDER BY time DESC
+            '''.format(jids=', '.join('?' * len(jids)),
+                       account_id=account_id,
+                       kinds=', '.join(kinds))
 
-        result = self._con.execute(sql, (jid,
-                                         resource,
-                                         message_id,
-                                         account_id,
-                                         start_time,
-                                         end_time)).fetchone()
+        result = self._con.execute(
+            sql,
+            tuple(jids)).fetchmany(50)
 
-        if result is not None:
-            log.debug('Found duplicate')
-            return True
-        return False
+        nicknames: list[str] = []
+        for row in result:
+            if row.contact_name not in nicknames:
+                nicknames.append(row.contact_name)
+
+        return nicknames
 
     @timeit
     def find_stanza_id(self,
@@ -928,7 +884,7 @@ class MessageArchiveStorage(SqliteStorage):
                        origin_id: Optional[str] = None,
                        groupchat: bool = False
                        ) -> bool:
-        """
+        '''
         Checks if a stanza-id is already in the `logs` table
 
         :param account:     The account
@@ -943,7 +899,7 @@ class MessageArchiveStorage(SqliteStorage):
         :param groupchat:   stanza-id is from a groupchat
 
         return True if the stanza-id was found
-        """
+        '''
         ids: list[str] = []
         if stanza_id is not None:
             ids.append(stanza_id)
@@ -978,7 +934,8 @@ class MessageArchiveStorage(SqliteStorage):
         else:
             sql = '''
                 SELECT stanza_id FROM logs
-                WHERE stanza_id IN ({values}) AND account_id = ? AND kind != ? LIMIT 1
+                WHERE stanza_id IN ({values}) AND account_id = ? AND
+                kind != ? LIMIT 1
                 '''.format(values=', '.join('?' * len(ids)))
             result = self._con.execute(
                 sql, tuple(ids) + (account_id, KindConstant.GC_MSG)).fetchone()
@@ -991,29 +948,86 @@ class MessageArchiveStorage(SqliteStorage):
         return False
 
     @timeit
-    def store_message_correction(self,
-                                 account: str,
-                                 jid: JID,
-                                 correct_id: str,
-                                 corrected_text: str,
-                                 is_groupchat: bool) -> None:
-        type_ = JIDConstant.NORMAL_TYPE
-        if is_groupchat:
-            type_ = JIDConstant.ROOM_TYPE
-
-        jid_id = self.get_jid_id(str(jid), type_=type_)
+    def get_last_correctable_message(self,
+                                     account: str,
+                                     jid: JID,
+                                     message_id: str
+                                     ) -> Optional[LastConversationRow]:
+        '''
+        Load the last correctable message of a conversation by message_id.
+        Conditions: max 5 min old
+        '''
+        jids = [jid]
         account_id = self.get_account_id(account)
+        min_time = time.time() - MAX_MESSAGE_CORRECTION_DELAY
+
         sql = '''
-            SELECT log_line_id, message, additional_data
-            FROM logs
-            WHERE +jid_id = ?
-            AND account_id = ?
+            SELECT contact_name, time, kind, message, stanza_id, message_id,
+            additional_data
+            FROM logs NATURAL JOIN jids WHERE jid IN ({jids})
+            AND account_id = {account_id}
             AND message_id = ?
-            '''
-        row = self._con.execute(
-            sql, (jid_id, account_id, correct_id)).fetchone()
-        if row is None:
-            return
+            AND time > ?
+            '''.format(jids=', '.join('?' * len(jids)),
+                       account_id=account_id)
+
+        return self._con.execute(
+            sql,
+            tuple(jids) + (message_id, min_time)).fetchone()
+
+    @timeit
+    def try_message_correction(self,
+                               account: str,
+                               jid: JID,
+                               nickname: Optional[str],
+                               corrected_text: str,
+                               correct_id: str,
+                               kind: KindConstant,
+                               timestamp: float) -> bool:
+
+        '''Try to correct a message
+
+        :param jid:  This can be a full jid or bare jid. A full jid only if the
+                     message is a MUC PM, otherwise a bare jid needs to be
+                     passed. `nickname` should only be passed if the message
+                     is a group chat message.
+        '''
+
+        account_id = self.get_account_id(account)
+        max_timestamp = timestamp - MAX_MESSAGE_CORRECTION_DELAY
+
+        self._log.debug(
+            'Check if message is correctable, parameters: %s %s %s %s %s',
+            jid, account_id, nickname, correct_id, max_timestamp)
+
+        sql = '''SELECT log_line_id, message, additional_data
+                 FROM logs
+                 NATURAL JOIN jids jid_id
+                 WHERE +jid = ?
+                 AND account_id = ?
+                 AND contact_name IS ?
+                 AND message_id = ?
+                 AND kind = ?
+                 AND time > ?
+                 '''
+
+        rows = self._con.execute(sql, (jid,
+                                       account_id,
+                                       nickname,
+                                       correct_id,
+                                       kind,
+                                       max_timestamp
+                                       )).fetchall()
+
+        if not rows:
+            self._log.debug('No correctable messages found')
+            return False
+
+        if len(rows) != 1:
+            self._log.warning('More than one correctable message found')
+            return False
+
+        row = rows[0]
 
         if row.additional_data is None:
             additional_data = AdditionalDataDict()
@@ -1035,6 +1049,8 @@ class MessageArchiveStorage(SqliteStorage):
         self._con.execute(
             sql, (corrected_text, serialized_dict, row.log_line_id))
 
+        return True
+
     @timeit
     def update_additional_data(self,
                                account: str,
@@ -1120,7 +1136,7 @@ class MessageArchiveStorage(SqliteStorage):
                    kind: Optional[KindConstant] = None,
                    type_: JIDConstant = JIDConstant.NORMAL_TYPE
                    ) -> int:
-        """
+        '''
         Insert a new jid into the `jids` table.
         This is an alias of get_jid_id() for better readablility.
 
@@ -1129,7 +1145,7 @@ class MessageArchiveStorage(SqliteStorage):
         :param kind:    A KindConstant
 
         :param type_:   A JIDConstant
-        """
+        '''
         return self.get_jid_id(jid, kind, type_)
 
     @timeit
@@ -1140,7 +1156,7 @@ class MessageArchiveStorage(SqliteStorage):
                          kind: KindConstant,
                          **kwargs: Any
                          ) -> int:
-        """
+        '''
         Insert a new message into the `logs` table
 
         :param jid:     The jid as string
@@ -1154,7 +1170,7 @@ class MessageArchiveStorage(SqliteStorage):
 
         :param kwargs:  Every additional named argument must correspond to
                         a field in the `logs` table
-        """
+        '''
         jid_id = self.get_jid_id(jid, kind=kind)
         account_id = self.get_account_id(account)
 
@@ -1162,7 +1178,7 @@ class MessageArchiveStorage(SqliteStorage):
             if not kwargs['additional_data']:
                 del kwargs['additional_data']
             else:
-                serialized_dict = json.dumps(kwargs["additional_data"].data)
+                serialized_dict = json.dumps(kwargs['additional_data'].data)
                 kwargs['additional_data'] = serialized_dict
 
         sql = '''
@@ -1172,7 +1188,9 @@ class MessageArchiveStorage(SqliteStorage):
                          values=', '.join('?' * len(kwargs)))
 
         lastrowid = self._con.execute(
-            sql, (account_id, jid_id, time_, kind) + tuple(kwargs.values())).lastrowid
+            sql, (account_id,
+                  jid_id, time_,
+                  kind) + tuple(kwargs.values())).lastrowid
         assert lastrowid is not None
 
         log.info('Insert into DB: jid: %s, time: %s, kind: %s, stanza_id: %s',
@@ -1189,7 +1207,7 @@ class MessageArchiveStorage(SqliteStorage):
                           message_id: str,
                           error: str
                           ) -> None:
-        """
+        '''
         Update the corresponding message with the error
 
         :param account_jid: The jid of the account
@@ -1200,7 +1218,7 @@ class MessageArchiveStorage(SqliteStorage):
 
         :param error:       The error stanza as string
 
-        """
+        '''
 
         account_id = self.get_jid_id(account_jid)
         try:
@@ -1221,20 +1239,12 @@ class MessageArchiveStorage(SqliteStorage):
                    account_jid: str,
                    jid: str,
                    message_id: str,
-                   state: str
+                   state: Literal['received', 'displayed']
                    ) -> None:
-        """
+        '''
         Update the marker state of the corresponding message
+        '''
 
-        :param account_jid: The jid of the account
-
-        :param jid:         The jid that belongs to the avatar
-
-        :param message_id:  The id of the message
-
-        :param state:       The state, 'received' or 'displayed'
-
-        """
         if state not in ('received', 'displayed'):
             raise ValueError('Invalid marker state')
 
@@ -1256,19 +1266,19 @@ class MessageArchiveStorage(SqliteStorage):
 
     @timeit
     def get_archive_infos(self, jid: str) -> Optional[LastArchiveMessageRow]:
-        """
+        '''
         Get the archive infos
 
         :param jid:     The jid that belongs to the avatar
 
-        """
+        '''
         jid_id = self.get_jid_id(jid, type_=JIDConstant.ROOM_TYPE)
         sql = '''SELECT * FROM last_archive_message WHERE jid_id = ?'''
         return self._con.execute(sql, (jid_id,)).fetchone()
 
     @timeit
     def set_archive_infos(self, jid: str, **kwargs: Any) -> None:
-        """
+        '''
         Set archive infos
 
         :param jid:                     The jid that belongs to the avatar
@@ -1284,7 +1294,7 @@ class MessageArchiveStorage(SqliteStorage):
         :param sync_threshold:          The max days that we request from a
                                         MUC archive
 
-        """
+        '''
         jid_id = self.get_jid_id(jid)
         exists = self.get_archive_infos(jid)
         if not exists:
@@ -1312,12 +1322,12 @@ class MessageArchiveStorage(SqliteStorage):
 
     @timeit
     def reset_archive_infos(self, jid: str) -> None:
-        """
+        '''
         Set archive infos
 
         :param jid:                     The jid of the archive
 
-        """
+        '''
         jid_id = self.get_jid_id(jid)
         sql = '''UPDATE last_archive_message
                  SET last_mam_id = NULL, oldest_mam_timestamp = NULL,
@@ -1362,10 +1372,10 @@ class MessageArchiveStorage(SqliteStorage):
                 yield result
 
     def remove_history(self, account: str, jid: JID) -> None:
-        """
+        '''
         Remove history for a specific chat.
         If it's a group chat, remove last MAM ID as well.
-        """
+        '''
         account_id = self.get_account_id(account)
         try:
             jid_id = self.get_jid_id(jid)
@@ -1379,9 +1389,9 @@ class MessageArchiveStorage(SqliteStorage):
         log.info('Removed history for: %s', jid)
 
     def remove_all_history(self) -> None:
-        """
+        '''
         Remove all messages for all accounts
-        """
+        '''
         statements = [
             'DELETE FROM logs',
             'DELETE FROM jids',
@@ -1391,9 +1401,9 @@ class MessageArchiveStorage(SqliteStorage):
         log.info('Removed all chat history')
 
     def cleanup_chat_history(self) -> None:
-        """
+        '''
         Remove messages from account where messages are older than max_age
-        """
+        '''
         for account in app.settings.get_accounts():
             max_age = app.settings.get_account_setting(
                 account, 'chat_history_max_age')
