@@ -15,7 +15,6 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Dict
 from typing import Type
 from typing import TypeVar
 from typing import NamedTuple
@@ -28,16 +27,21 @@ from dataclasses import fields
 
 from gi.repository import GLib
 
-from nbxmpp.protocol import JID
-from nbxmpp.const import Role
 from nbxmpp.const import Affiliation
+from nbxmpp.const import Chatstate
 from nbxmpp.const import PresenceShow
+from nbxmpp.const import Role
+from nbxmpp.modules.dataforms import SimpleDataForm
+from nbxmpp.modules.security_labels import SecurityLabel
+from nbxmpp.protocol import JID
+from nbxmpp.structs import MucSubject
+from nbxmpp.structs import PresenceProperties
 
+from gajim.common import types
 from gajim.common.const import MUCJoinedState
 from gajim.common.const import KindConstant
 from gajim.common.const import PresenceShowExt
 from gajim.common.const import URIType
-from gajim.common.const import URIAction
 
 
 _T = TypeVar('_T')
@@ -45,16 +49,19 @@ _T = TypeVar('_T')
 
 class URI(NamedTuple):
     type: URIType
-    action: Optional[URIAction] = None
-    data: Optional[Union[Dict[str, str], str]] = None
+    source: str
+    query_type: str = ''
+    query_params: dict[str, str] = {}
+    data: Optional[Union[dict[str, str], str]] = None
 
 
 class MUCData:
     def __init__(self,
                  room_jid: str,
                  nick: str,
-                 password: str,
-                 config=None) -> None:
+                 password: Optional[str],
+                 config: Optional[dict[str, Any]] = None
+                 ) -> None:
 
         self._room_jid = JID.from_string(room_jid)
         self._config = config
@@ -62,8 +69,12 @@ class MUCData:
         self.password = password
         self.state = MUCJoinedState.NOT_JOINED
         # Message id of the captcha challenge
-        self.captcha_id = None
-        self.subject = None
+        self.captcha_id: Optional[str] = None
+        self.captcha_form: Optional[SimpleDataForm] = None
+        self.error: Optional[str] = None
+        self.error_text: Optional[str] = None
+        self.subject: Optional[MucSubject] = None
+        self.last_subject_timestamp: Optional[float] = None
 
     @property
     def jid(self) -> JID:
@@ -74,29 +85,30 @@ class MUCData:
         return self._room_jid.new_with(resource=self.nick)
 
     @property
-    def config(self):
+    def config(self) -> Optional[dict[str, Any]]:
         return self._config
 
 
 class OutgoingMessage:
     def __init__(self,
                  account: str,
-                 contact,
+                 contact: types.ChatContactT,
                  message: Optional[str],
                  type_: str,
                  subject: Optional[str] = None,
-                 chatstate=None,
+                 chatstate: Optional[Chatstate] = None,
                  marker: Optional[tuple[str, str]] = None,
                  resource: Optional[str] = None,
                  user_nick: Optional[str] = None,
-                 label: Optional[str] = None,
-                 control=None,
+                 label: Optional[SecurityLabel] = None,
+                 control: Optional[Any] = None,
                  attention: Optional[bool] = None,
                  correct_id: Optional[str] = None,
                  oob_url: Optional[str] = None,
-                 xhtml=None,
-                 nodes=None,
-                 play_sound: bool = True):
+                 xhtml: Optional[str] = None,
+                 nodes: Optional[Any] = None,
+                 play_sound: bool = True
+                 ) -> None:
 
         if type_ not in ('chat', 'groupchat', 'normal', 'headline'):
             raise ValueError('Unknown message type: %s' % type_)
@@ -147,8 +159,7 @@ class OutgoingMessage:
         self.timestamp = None
         self.message_id = None
         self.stanza = None
-        self.session = None
-        self.delayed = None # TODO never set
+        self.delayed = None  # TODO never set
 
         self.is_loggable = True
 
@@ -207,7 +218,7 @@ class PresenceData:
     available: bool
 
     @classmethod
-    def from_presence(cls, properties):
+    def from_presence(cls, properties: PresenceProperties) -> PresenceData:
         return cls(show=properties.show,
                    status=properties.status,
                    priority=properties.priority,
@@ -233,7 +244,7 @@ class MUCPresenceData:
     real_jid: Optional[JID]
 
     @classmethod
-    def from_presence(cls, properties):
+    def from_presence(cls, properties: PresenceProperties) -> MUCPresenceData:
         return cls(show=properties.show,
                    status=properties.status,
                    idle_time=properties.idle_timestamp,
