@@ -21,20 +21,22 @@ from typing import Optional
 import os
 import sys
 import signal
+import sqlite3
 import platform
 from ctypes import CDLL, byref, create_string_buffer
 from ctypes.util import find_library
 from packaging.version import Version as V
 
 import gajim.gui
-from gajim.common import i18n
 
 
-_MIN_NBXMPP_VER = '3.0.0'
-_MIN_GTK_VER = '3.22.27'
+_MIN_NBXMPP_VER = '3.2.5'
+_MIN_GTK_VER = '3.24.30'
 _MIN_CAIRO_VER = '1.16.0'
-_MIN_PYGOBJECT_VER = '3.32.0'
+_MIN_PYGOBJECT_VER = '3.42.0'
 _MIN_GLIB_VER = '2.60.0'
+_MIN_PANGO_VER = '1.50.0'
+_MIN_SQLITE_VER = '3.33.0'
 
 
 def check_version(dep_name: str, current_ver: str, min_ver: str) -> None:
@@ -63,6 +65,7 @@ def _check_required_deps() -> None:
                              'GtkSource': '4',
                              'GObject': '2.0',
                              'Pango': '1.0',
+                             'PangoCairo': '1.0',
                              'Soup': '2.4'})
     except ValueError as error:
         sys.exit('Missing dependency: %s' % error)
@@ -70,7 +73,7 @@ def _check_required_deps() -> None:
     try:
         import cairo
     except ImportError as error:
-        sys.exit(error_message % ('python-cairo', error))
+        sys.exit(error_message % ('pycairo', error))
 
     from gi.repository import Gtk
     gtk_ver = '%s.%s.%s' % (Gtk.get_major_version(),
@@ -82,12 +85,16 @@ def _check_required_deps() -> None:
                                   GLib.MINOR_VERSION,
                                   GLib.MICRO_VERSION]))
 
+    from gi.repository import Pango
+
     check_version('python-nbxmpp', nbxmpp.__version__, _MIN_NBXMPP_VER)
     check_version('pygobject', gi.__version__, _MIN_PYGOBJECT_VER)
     check_version('libcairo', cairo.cairo_version_string(), _MIN_CAIRO_VER)
-    check_version('python-cairo', cairo.version, _MIN_CAIRO_VER)
+    check_version('pycairo', cairo.version, _MIN_CAIRO_VER)
     check_version('gtk3', gtk_ver, _MIN_GTK_VER)
     check_version('glib', glib_ver, _MIN_GLIB_VER)
+    check_version('pango', Pango.version_string(), _MIN_PANGO_VER)
+    check_version('sqlite', sqlite3.sqlite_version, _MIN_SQLITE_VER)
 
 
 def _init_gui(gui: str) -> None:
@@ -112,8 +119,6 @@ def _init_gtk() -> None:
     from gajim.gui import exception
     exception.init()
 
-    i18n.initialize_direction_mark()
-
 
 def _run_app() -> None:
     from gajim.gui.application import GajimApplication
