@@ -28,9 +28,8 @@ from gi.repository import Pango
 from gi.repository import GObject
 
 from gajim.common import app
-from gajim.common import ged
 from gajim.common import passwords
-from gajim.common.events import AccountDisonnected
+from gajim.common.const import ClientState
 from gajim.common.i18n import _
 from gajim.common.i18n import Q_
 from gajim.common.settings import AllSettingsT
@@ -41,6 +40,7 @@ from .dialogs import ConfirmationDialog
 from .const import Setting
 from .const import SettingKind
 from .const import SettingType
+from .menus import build_accounts_menu
 from .settings import SettingsDialog
 from .settings import SettingsBox
 from .settings import PopoverSetting
@@ -167,9 +167,9 @@ class AccountsWindow(Gtk.ApplicationWindow):
         if not initial:
             self._accounts[account].show()
 
-    def select_account(self, account: str) -> None:
+    def select_account(self, account: str, page: Optional[str] = None) -> None:
         try:
-            self._accounts[account].select()
+            self._accounts[account].select(page)
         except KeyError:
             log.warning('select_account() failed, account %s not found',
                         account)
@@ -186,8 +186,8 @@ class Settings(Gtk.ScrolledWindow):
         self.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
         self._stack = Gtk.Stack(vhomogeneous=False)
         self._stack.set_transition_type(Gtk.StackTransitionType.CROSSFADE)
+        self._stack.get_style_context().add_class('settings-stack')
         self._stack.add_named(AddNewAccountPage(), 'add-account')
-        self.get_style_context().add_class('accounts-settings')
 
         self.add(self._stack)
         self._page_signal_ids: dict[GenericSettingPage, int] = {}
@@ -236,7 +236,8 @@ class AccountMenu(Gtk.Box):
 
         self._accounts_listbox = Gtk.ListBox()
         self._accounts_listbox.set_sort_func(self._sort_func)
-        self._accounts_listbox.get_style_context().add_class('settings-box')
+        self._accounts_listbox.get_style_context().add_class(
+            'accounts-menu-listbox')
         self._accounts_listbox.connect('row-activated',
                                        self._on_account_row_activated)
 
@@ -293,6 +294,12 @@ class AccountMenu(Gtk.Box):
                       listbox.account,
                       f'{listbox.account}-{row.name}')
 
+    def set_page(self, account: str, page_name: str) -> None:
+        sub_menu = cast(
+            AccountSubMenu, self._stack.get_child_by_name(f'{account}-menu'))
+        sub_menu.select_row_by_name(page_name)
+        self.emit('menu-activated', account, f'{account}-{page_name}')
+
     def update_account_label(self, account: str) -> None:
         self._accounts_listbox.invalidate_sort()
         sub_menu = cast(
@@ -310,7 +317,7 @@ class AccountSubMenu(Gtk.ListBox):
         Gtk.ListBox.__init__(self)
         self.set_vexpand(True)
         self.set_hexpand(True)
-        self.get_style_context().add_class('settings-box')
+        self.get_style_context().add_class('accounts-menu-listbox')
 
         self._account = account
 
@@ -326,6 +333,14 @@ class AccountSubMenu(Gtk.ListBox):
     def account(self) -> str:
         return self._account
 
+    def select_row_by_name(self, row_name: str) -> None:
+        for row in self.get_children():
+            if not isinstance(row, PageMenuItem):
+                continue
+            if row.name == row_name:
+                self.select_row(row)
+                return
+
     def update(self) -> None:
         self.emit('update', self._account)
 
@@ -444,8 +459,10 @@ class Account:
         self._account_row = AccountRow(account)
         self._menu.add_account(self._account_row)
 
-    def select(self) -> None:
+    def select(self, page_name: Optional[str] = None) -> None:
         self._account_row.emit('activate')
+        if page_name is not None:
+            self._menu.set_page(self._account, page_name)
 
     def show(self) -> None:
         self._menu.show_all()
@@ -503,8 +520,10 @@ class AccountRow(Gtk.ListBoxRow):
         self._switch.set_vexpand(False)
 
         self._switch_state_label = Gtk.Label()
-        self._switch_state_label.set_xalign(1)
+        self._switch_state_label.set_xalign(0)
         self._switch_state_label.set_valign(Gtk.Align.CENTER)
+        label_width = max(len(Q_('?switch:On')), len(Q_('?switch:Off')))
+        self._switch_state_label.set_width_chars(label_width)
         self._set_label(account_enabled)
 
         self._switch.connect(
@@ -541,34 +560,26 @@ class AccountRow(Gtk.ListBoxRow):
                           state: bool,
                           account: str
                           ) -> int:
-        def _on_disconnect(event: AccountDisonnected) -> None:
-            if event.account != account:
-                return
-            app.ged.remove_event_handler('account-disconnected',
-                                         ged.CORE,
-                                         _on_disconnect)
-            app.interface.disable_account(account)
 
         def _disable() -> None:
-            app.ged.register_event_handler('account-disconnected',
-                                           ged.CORE,
-                                           _on_disconnect)
             client = app.get_client(account)
+            client.connect_signal('state-changed', self._on_state_changed)
             client.change_status('offline', 'offline')
             switch.set_state(state)
             self._set_label(state)
 
-        old_state = app.settings.get_account_setting(account, 'active')
-        if old_state == state:
+        account_is_active = app.settings.get_account_setting(account, 'active')
+        if account_is_active == state:
             return Gdk.EVENT_PROPAGATE
 
-        if (account in app.connections and
-                not app.connections[account].state.is_disconnected):
+        if (account_is_active and
+                not app.get_client(account).state.is_disconnected):
             # Connecting or connected
             window = cast(AccountsWindow, get_app_window('AccountsWindow'))
+            account_label = app.get_account_label(account)
             ConfirmationDialog(
                 _('Disable Account'),
-                _('Account %s is still connected') % account,
+                _('Account %s is still connected') % account_label,
                 _('All chat and group chat windows will be closed.'),
                 [DialogButton.make('Cancel',
                                    callback=lambda: switch.set_active(True)),
@@ -579,12 +590,21 @@ class AccountRow(Gtk.ListBoxRow):
             return Gdk.EVENT_STOP
 
         if state:
-            app.interface.enable_account(account)
+            app.app.enable_account(account)
         else:
-            app.interface.disable_account(account)
+            app.app.disable_account(account)
 
         return Gdk.EVENT_PROPAGATE
 
+    def _on_state_changed(self,
+                          client: types.Client,
+                          _signal_name: str,
+                          client_state: ClientState
+                          ) -> None:
+
+        if client_state.is_disconnected:
+            app.app.disable_account(client.account)
+
 
 class AddNewAccountPage(Gtk.Box):
     def __init__(self) -> None:
@@ -620,7 +640,7 @@ class GenericSettingPage(Gtk.Box):
         self.account = account
 
         self.listbox = SettingsBox(account)
-        self.listbox.get_style_context().add_class('accounts-settings-border')
+        self.listbox.get_style_context().add_class('border')
         self.listbox.set_selection_mode(Gtk.SelectionMode.NONE)
         self.listbox.set_vexpand(False)
         self.listbox.set_valign(Gtk.Align.END)
@@ -646,6 +666,8 @@ class GeneralPage(GenericSettingPage):
 
     def __init__(self, account: str) -> None:
 
+        workspaces = self._get_workspaces()
+
         settings = [
             Setting(SettingKind.ENTRY,
                     _('Label'),
@@ -653,6 +675,14 @@ class GeneralPage(GenericSettingPage):
                     'account_label',
                     callback=self._on_account_name_change),
 
+            Setting(SettingKind.POPOVER,
+                    _('Default Workspace'),
+                    SettingType.ACCOUNT_CONFIG,
+                    'default_workspace',
+                    props={'entries': workspaces},
+                    desc=_('Chats from this account will use this workspace by '
+                           'default')),
+
             Setting(SettingKind.COLOR,
                     _('Color'),
                     SettingType.ACCOUNT_CONFIG,
@@ -705,9 +735,18 @@ class GeneralPage(GenericSettingPage):
         ]
         GenericSettingPage.__init__(self, account, settings)
 
+    @staticmethod
+    def _get_workspaces() -> dict[str, str]:
+        workspaces: dict[str, str] = {'': _('Disabled')}
+        for workspace_id in app.settings.get_workspaces():
+            name = app.settings.get_workspace_setting(workspace_id, 'name')
+            workspaces[workspace_id] = name
+        return workspaces
+
     def _on_account_name_change(self, *args: Any) -> None:
         window = cast(AccountsWindow, get_app_window('AccountsWindow'))
         window.update_account_label(self.account)
+        build_accounts_menu()
 
 
 class PrivacyPage(GenericSettingPage):
@@ -872,8 +911,6 @@ class PrivacyPage(GenericSettingPage):
         app.settings.set_contact_settings('send_marker', None)
         app.settings.set_group_chat_settings(
             'send_marker', None, context='private')
-        for ctrl in app.window.get_controls(account=self._account):
-            ctrl.update_actions()
 
 
 class ConnectionPage(GenericSettingPage):
@@ -1005,7 +1042,7 @@ class PriorityDialog(SettingsDialog):
 
     def on_destroy(self, *args: Any) -> None:
         # Update priority
-        if self.account not in app.connections:
+        if self.account not in app.settings.get_active_accounts():
             return
         client = app.get_client(self.account)
         show = client.status
@@ -1084,7 +1121,7 @@ class LoginDialog(SettingsDialog):
                     SettingType.ACCOUNT_CONFIG,
                     'savepass',
                     enabled_func=(lambda: not app.settings.get('use_keyring') or
-                                  passwords.KEYRING_AVAILABLE)),
+                                  passwords.is_keyring_available())),
 
             Setting(SettingKind.CHANGEPASSWORD,
                     _('Change Password'),
