@@ -47,6 +47,8 @@ from urllib.parse import unquote
 from nbxmpp.namespaces import Namespace
 from nbxmpp import JID
 from nbxmpp.protocol import InvalidJid
+from nbxmpp.const import ConnectionType
+from nbxmpp.const import ConnectionProtocol
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import Gtk
@@ -70,6 +72,7 @@ from gajim.common.i18n import _
 from gajim.gui import menus
 from gajim.gui import structs
 from gajim.gui.about import AboutDialog
+from gajim.gui.accounts import AccountsWindow
 from gajim.gui.avatar import AvatarStorage
 from gajim.gui.builder import get_builder
 from gajim.gui.const import ACCOUNT_ACTIONS
@@ -83,6 +86,7 @@ from gajim.gui.dialogs import ShortcutsWindow
 from gajim.gui.discovery import ServiceDiscoveryWindow
 from gajim.gui.start_chat import StartChatDialog
 from gajim.gui.util import get_app_window
+from gajim.gui.util import get_app_windows
 from gajim.gui.util import load_user_iconsets
 from gajim.gui.util import open_window
 
@@ -110,7 +114,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
             ord('V'),
             GLib.OptionFlags.NONE,
             GLib.OptionArg.NONE,
-            _('Show the application\'s version'))
+            _("Show the application's version"))
 
         self.add_main_option(
             'quiet',
@@ -165,13 +169,6 @@ class GajimApplication(Gtk.Application, CoreApplication):
             GLib.OptionArg.NONE,
             _('Show all warnings'))
 
-        self.add_main_option(
-            'ipython',
-            ord('i'),
-            GLib.OptionFlags.NONE,
-            GLib.OptionArg.NONE,
-            _('Open IPython shell'))
-
         self.add_main_option(
             'gdebug',
             0,
@@ -247,12 +244,6 @@ class GajimApplication(Gtk.Application, CoreApplication):
         idle.Monitor.set_interval(app.settings.get('autoawaytime') * 60,
                                   app.settings.get('autoxatime') * 60)
 
-        if sys.platform == 'darwin':
-            # TODO: Remove if colored emoji rendering works well on
-            # Windows and MacOS
-            from gajim.gui.emoji_chooser import emoji_chooser
-            emoji_chooser.load()
-
         from gajim.gui_interface import Interface
 
         self.interface = Interface()
@@ -270,15 +261,17 @@ class GajimApplication(Gtk.Application, CoreApplication):
         menus.build_accounts_menu()
         self.update_app_actions_state()
 
-        app.ged.register_event_handler('feature-discovered',
-                                       ged.CORE,
-                                       self._on_feature_discovered)
+        self.register_event('feature-discovered',
+                            ged.CORE,
+                            self._on_feature_discovered)
 
         from gajim.gui.main import MainWindow
         MainWindow()
 
+        GLib.timeout_add(100, self._auto_connect)
+
     def _open_uris(self, uris: list[str]) -> None:
-        accounts = list(app.connections.keys())
+        accounts = app.settings.get_active_accounts()
         if not accounts:
             return
 
@@ -312,7 +305,8 @@ class GajimApplication(Gtk.Application, CoreApplication):
                         'groupchat-join',
                         GLib.Variant('as', [accounts[0], jid]))
                 else:
-                    self.activate_action('start-chat', GLib.Variant('s', jid))
+                    self.activate_action(
+                        'start-chat', GLib.Variant('as', [jid, '']))
 
             elif cmd == 'roster':
                 self.activate_action('add-contact', GLib.Variant('s', jid))
@@ -328,7 +322,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
                     except Exception:
                         app.log('uri_handler').error('Invalid URI: %s', cmd)
 
-                app.interface.start_chat_from_jid(accounts[0], jid, message)
+                app.window.start_chat_from_jid(accounts[0], jid, message)
 
     def _shutdown(self, _application: GajimApplication) -> None:
         self._shutdown_core()
@@ -345,8 +339,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
         options = command_line.get_options_dict()
 
         remote_commands = [
-            ('ipython', None),
-            ('start-chat', GLib.Variant('s', '')),
+            ('start-chat', GLib.Variant('as', ['', ''])),
             ('show', None)
         ]
 
@@ -383,6 +376,8 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
         self.register()
         if self.get_is_remote():
+            print('Gajim is already running. '
+                  'The primary instance will handle remote commands')
             return -1
 
         self._core_command_line(options)
@@ -415,7 +410,6 @@ class GajimApplication(Gtk.Application, CoreApplication):
             ('join-support-chat', self._on_join_support_chat),
             ('about', self._on_about_action),
             ('faq', self._on_faq_action),
-            ('ipython', self._on_ipython_action),
             ('start-chat', self._on_new_chat_action),
             ('accounts', self._on_accounts_action),
             ('add-contact', self._on_add_contact_action),
@@ -524,6 +518,68 @@ class GajimApplication(Gtk.Application, CoreApplication):
         elif event.feature == Namespace.BLOCKING:
             action = '%s-blocking' % event.account
             self.set_action_state(action, True)
+            action = '%s-block-contact' % event.account
+            self.set_action_state(action, True)
+
+    def create_account(self,
+                       account: str,
+                       username: str,
+                       domain: str,
+                       password: str,
+                       proxy_name: str,
+                       custom_host: tuple[str,
+                                          ConnectionProtocol,
+                                          ConnectionType],
+                       anonymous: bool = False
+                       ) -> None:
+
+        CoreApplication.create_account(self,
+                                       account,
+                                       username,
+                                       domain,
+                                       password,
+                                       proxy_name,
+                                       custom_host,
+                                       anonymous)
+
+        app.css_config.refresh()
+
+        # Action must be added before account window is updated
+        self.add_account_actions(account)
+
+        window = cast(AccountsWindow, get_app_window('AccountsWindow'))
+        if window is not None:
+            window.add_account(account)
+
+    def enable_account(self, account: str) -> None:
+        CoreApplication.enable_account(self, account)
+        menus.build_accounts_menu()
+        self.update_app_actions_state()
+        window = cast(AccountsWindow, get_app_window('AccountsWindow'))
+        if window is not None:
+            window.enable_account(account, True)
+
+    def disable_account(self, account: str) -> None:
+        for win in get_app_windows(account):
+            # Close all account specific windows, except the RemoveAccount
+            # dialog. It shows if the removal was successful.
+            if type(win).__name__ == 'RemoveAccount':
+                continue
+            win.destroy()
+
+        CoreApplication.disable_account(self, account)
+
+        menus.build_accounts_menu()
+        self.update_app_actions_state()
+
+    def remove_account(self, account: str) -> None:
+        CoreApplication.remove_account(self, account)
+
+        self.remove_account_actions(account)
+
+        window = cast(AccountsWindow, get_app_window('AccountsWindow'))
+        if window is not None:
+            window.remove_account(account)
 
     # Action Callbacks
 
@@ -567,10 +623,11 @@ class GajimApplication(Gtk.Application, CoreApplication):
     @staticmethod
     def _on_new_chat_action(_action: Gio.SimpleAction,
                             param: GLib.Variant) -> None:
-        window = open_window('StartChatDialog')
-        search_text = param.get_string()
-        if search_text:
-            window.set_search_text(search_text)
+
+        jid, initial_message = param.get_strv()
+        open_window('StartChatDialog',
+                    initial_jid=jid or None,
+                    initial_message=initial_message or None)
 
     @staticmethod
     def _on_profile_action(_action: Gio.SimpleAction,
@@ -583,6 +640,9 @@ class GajimApplication(Gtk.Application, CoreApplication):
                             param: GLib.Variant) -> None:
         account = param.get_string()
         server_jid = app.settings.get_account_setting(account, 'hostname')
+        if account not in app.interface.instances:
+            app.interface.instances[account] = {'disco': {}}
+
         disco = app.interface.instances[account]['disco']
         if server_jid in disco:
             disco[server_jid].window.present()
@@ -671,12 +731,12 @@ class GajimApplication(Gtk.Application, CoreApplication):
     @staticmethod
     def _on_join_support_chat(_action: Gio.SimpleAction,
                               _param: Optional[GLib.Variant]) -> None:
-        accounts = list(app.connections.keys())
+        accounts = app.settings.get_active_accounts()
         if len(accounts) == 1:
-            app.interface.show_add_join_groupchat(
+            app.window.show_add_join_groupchat(
                 accounts[0], GAJIM_SUPPORT_JID)
             return
-        open_window('StartChatDialog', jid=GAJIM_SUPPORT_JID)
+        open_window('StartChatDialog', initial_jid=GAJIM_SUPPORT_JID)
 
     @staticmethod
     def _on_faq_action(_action: Gio.SimpleAction,
@@ -738,18 +798,6 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
         app.window.mark_as_read(params.account, params.jid)
 
-    @staticmethod
-    def _on_ipython_action(_action: Gio.SimpleAction,
-                           _param: Optional[GLib.Variant]) -> None:
-        """
-        Show/hide the ipython window
-        """
-        win = cast(Gtk.Window, app.ipython_window)
-        if win and win.is_visible():
-            win.present()
-        else:
-            app.interface.create_ipython_window()
-
     @staticmethod
     def _on_open_mail_action(_action: Gio.SimpleAction,
                              param: GLib.Variant) -> None:
@@ -775,7 +823,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
     def _on_open_chat_action(_action: Gio.SimpleAction,
                              param: GLib.Variant) -> None:
         account, jid = param.get_strv()
-        app.interface.start_chat_from_jid(account, jid)
+        app.window.start_chat_from_jid(account, jid)
 
     @staticmethod
     @structs.actionfunction
@@ -783,14 +831,14 @@ class GajimApplication(Gtk.Application, CoreApplication):
                                   params: structs.RemoveHistoryActionParams
                                   ) -> None:
         def _remove() -> None:
+            app.storage.archive.remove_history(params.account, params.jid)
+            control = app.window.get_control()
             if params.jid is not None:
-                app.storage.archive.remove_history(params.account, params.jid)
-                control = app.window.get_control(params.account, params.jid)
-                if control is not None:
-                    control.reset_view()
-            else:
-                for control in app.window.get_controls(params.account):
-                    control.reset_view()
+                if not control.is_loaded(params.account, params.jid):
+                    return
+
+            control.reset_view()
+
         ConfirmationDialog(
             _('Remove Chat History'),
             _('Remove Chat History?'),
