@@ -14,9 +14,9 @@
 
 from __future__ import annotations
 
+from typing import Any
 from typing import cast
 from typing import Optional
-from typing import TYPE_CHECKING
 
 from gi.repository import Gdk
 from gi.repository import Gio
@@ -24,17 +24,21 @@ from gi.repository import GLib
 from gi.repository import Gtk
 
 from gajim.common import app
+from gajim.common import types
 from gajim.common.const import Direction
+from gajim.common.modules.contacts import GroupchatContact
 
 from .emoji_data_gtk import get_emoji_data
-if TYPE_CHECKING:
-    from .message_input import MessageInputTextView
+from .groupchat_nick_completion import GroupChatNickCompletion
+from .menus import escape_mnemonic
 
-MAX_ENTRIES = 5
+
+EMOJI_NUM_GENDERS = len(['f', 'm', 'n'])
+MENUS_MAX_ENTRIES = 2 * EMOJI_NUM_GENDERS
 
 
 class ChatActionProcessor(Gtk.Popover):
-    def __init__(self, message_input: MessageInputTextView) -> None:
+    def __init__(self, message_input: Gtk.TextView) -> None:
         Gtk.Popover.__init__(self)
         self._menu = Gio.Menu()
         self.bind_model(self._menu)
@@ -45,24 +49,41 @@ class ChatActionProcessor(Gtk.Popover):
         self.connect('closed', self._on_popover_closed)
         self.connect('destroy', self._on_destroy)
 
+        self._account: Optional[str] = None
+        self._contact: Optional[types.ChatContactT] = None
+
         self._message_input = message_input
         self._message_input.connect('key-press-event', self._on_key_press)
+        self._message_input.connect('focus-out-event', self._on_focus_out)
 
         self._buf = message_input.get_buffer()
         self._buf.connect('changed', self._on_changed)
 
+        self._nick_completion = GroupChatNickCompletion()
+
         self._start_mark: Optional[Gtk.TextMark] = None
         self._current_iter: Optional[Gtk.TextIter] = None
 
         self._active = False
 
+    def switch_contact(self, contact: types.ChatContactT) -> None:
+        self._account = contact.account
+        self._contact = contact
+        if isinstance(contact, GroupchatContact):
+            self._nick_completion.switch_contact(contact)
+
     def _on_destroy(self, _popover: Gtk.Popover) -> None:
         app.check_finalize(self)
 
     def _on_key_press(self,
-                      _textview: Gtk.TextView,
+                      textview: Gtk.TextView,
                       event: Gdk.EventKey
                       ) -> bool:
+        if isinstance(self._contact, GroupchatContact):
+            res = self._nick_completion.process_key_press(textview, event)
+            if res:
+                return True
+
         if not self._active:
             return False
 
@@ -86,6 +107,10 @@ class ChatActionProcessor(Gtk.Popover):
 
         return False
 
+    def _on_focus_out(self, *args: Any) -> bool:
+        self.popdown()
+        return False
+
     def _on_popover_closed(self, _popover: Gtk.Popover) -> None:
         self._active = False
         self._message_input.grab_focus()
@@ -101,17 +126,11 @@ class ChatActionProcessor(Gtk.Popover):
         self._buf.delete(start_iter, self._current_iter)
         self._buf.insert(start_iter, selected_action)
 
-    def _get_commands(self) -> list[str]:
-        commands: list[str] = []
-        control = app.window.get_control(
-            self._message_input.account, self._message_input.contact.jid)
-        assert control is not None
-        for command in control.list_commands():
-            for name in command.names:
-                commands.append(name)
-        return commands
+    def _get_commands(self) -> list[tuple[str, str]]:
+        assert self._contact is not None
+        return app.commands.get_commands(self._contact.type_string)
 
-    def _on_changed(self, _textview: MessageInputTextView) -> None:
+    def _on_changed(self, _text_buffer: Gtk.TextBuffer) -> None:
         insert = self._buf.get_insert()
         self._current_iter = self._buf.get_iter_at_mark(insert)
         current_offset = self._current_iter.get_offset()
@@ -159,15 +178,15 @@ class ChatActionProcessor(Gtk.Popover):
         self._menu.remove_all()
         command_list = self._get_commands()
         num_entries = 0
-        for command in command_list:
+        for command, usage in command_list:
             if not command.startswith(action_text[1:]):
                 continue
-            if num_entries >= MAX_ENTRIES:
+            if num_entries >= MENUS_MAX_ENTRIES:
                 continue
 
             action_data = GLib.Variant('s', f'/{command}')
             menu_item = Gio.MenuItem()
-            menu_item.set_label(f'/{command}')
+            menu_item.set_label(f'/{command} {usage}')
             menu_item.set_attribute_value('action-data', action_data)
             self._menu.append_item(menu_item)
             num_entries += 1
@@ -191,7 +210,7 @@ class ChatActionProcessor(Gtk.Popover):
             start, _end = search
             colon_offset = start.get_offset()
             before_colon = self._buf.get_iter_at_offset(colon_offset - 1)
-            if before_colon.get_char() != ' ':
+            if before_colon.get_char() not in (' ', '\n'):
                 # We want to show the menu only if text begins with a colon,
                 # or if a colon follows on a space. This avoids showing the
                 # menu within normal sentences containing colons.
@@ -199,11 +218,16 @@ class ChatActionProcessor(Gtk.Popover):
                 if not text.startswith(':'):
                     return False
 
-            action_text = self._buf.get_text(start, self._current_iter, False)
+            action_text = self._buf.get_text(
+                start, self._current_iter, False)[1:]
             if self._start_mark is None:
                 self._start_mark = Gtk.TextMark.new('chat-action-start', True)
                 self._buf.add_mark(self._start_mark, start)
-            self._update_emoji_menu(action_text, start)
+            if self._active or len(action_text) > 1:
+                # Don't activate until a sufficient # of chars has been typed,
+                # which is chosen to be > 1 to not interfere with ASCII smilies
+                # consisting of a colon and 1 single other char.
+                self._update_emoji_menu(action_text.casefold(), start)
             return True
 
         return False
@@ -214,18 +238,24 @@ class ChatActionProcessor(Gtk.Popover):
                            ) -> None:
         self._menu.remove_all()
         emoji_data = get_emoji_data()
-        menu_entry_count = 0
-        for shortcode, codepoint in emoji_data.items():
-            if not shortcode.startswith(action_text[1:]):
-                continue
-            if menu_entry_count >= MAX_ENTRIES:
-                continue
-            action_data = GLib.Variant('s', codepoint)
-            menu_item = Gio.MenuItem()
-            menu_item.set_label(f'{codepoint} {shortcode}')
-            menu_item.set_attribute_value('action-data', action_data)
-            self._menu.append_item(menu_item)
-            menu_entry_count += 1
+
+        for keyword, entries in emoji_data.items():
+            for short_name in entries.keys():
+                if not keyword.startswith(action_text):
+                    continue
+                emoji = entries[short_name]
+                label = f'{emoji} {short_name}'
+                if keyword != short_name:
+                    label += f'  [{keyword}]'
+                action_data = GLib.Variant('s', emoji)
+                menu_item = Gio.MenuItem()
+                menu_item.set_label(escape_mnemonic(label))
+                menu_item.set_attribute_value('action-data', action_data)
+                self._menu.append_item(menu_item)
+                if self._menu.get_n_items() >= MENUS_MAX_ENTRIES:
+                    break
+            if self._menu.get_n_items() >= MENUS_MAX_ENTRIES:
+                break
 
         if self._menu.get_n_items() > 0:
             self._show_menu(start)
