@@ -14,7 +14,6 @@
 
 from __future__ import annotations
 
-from typing import Dict
 from typing import Optional
 from typing import cast
 
@@ -27,27 +26,19 @@ from nbxmpp import JID
 
 from gajim.common import app
 from gajim.common import events
+from gajim.common import ged
+from gajim.common.ged import EventHelper
+from gajim.common.i18n import _
 
 from . import structs
 from .chat_filter import ChatFilter
 from .chat_list import ChatList
-from .chat_list import ChatRow
+from .chat_list import ChatListRow
+from .dialogs import DialogButton
+from .dialogs import ConfirmationCheckDialog
 
 
-HANDLED_EVENTS = (
-    events.MessageReceived,
-    events.MamMessageReceived,
-    events.GcMessageReceived,
-    events.MessageUpdated,
-    events.MessageModerated,
-    events.PresenceReceived,
-    events.MessageSent,
-    events.JingleRequestReceived,
-    events.FileRequestReceivedEvent
-)
-
-
-class ChatListStack(Gtk.Stack):
+class ChatListStack(Gtk.Stack, EventHelper):
 
     __gsignals__ = {
         'unread-count-changed': (GObject.SignalFlags.RUN_LAST,
@@ -69,11 +60,12 @@ class ChatListStack(Gtk.Stack):
                  search_entry: Gtk.SearchEntry
                  ) -> None:
         Gtk.Stack.__init__(self)
+        EventHelper.__init__(self)
         self.set_hexpand(True)
         self.set_vexpand(True)
         self.set_vhomogeneous(False)
 
-        self._chat_lists: Dict[str, ChatList] = {}
+        self._chat_lists: dict[str, ChatList] = {}
 
         self._last_visible_child_name: str = 'default'
 
@@ -86,11 +78,22 @@ class ChatListStack(Gtk.Stack):
         self._add_actions()
         self.show_all()
 
+        self.register_events([
+            ('message-received', ged.GUI2, self._on_event),
+            ('mam-message-received', ged.GUI2, self._on_event),
+            ('gc-message-received', ged.GUI2, self._on_event),
+            ('message-updated', ged.GUI2, self._on_event),
+            ('message-moderated', ged.GUI2, self._on_event),
+            ('presence-received', ged.GUI2, self._on_event),
+            ('message-sent', ged.GUI2, self._on_event),
+            ('file-request-received', ged.GUI2, self._on_event),
+            ('jingle-request-received', ged.GUI2, self._on_event),
+        ])
+
     def _add_actions(self) -> None:
         actions = [
             ('toggle-chat-pinned', 'a{sv}', self._toggle_chat_pinned),
             ('move-chat-to-workspace', 'a{sv}', self._move_chat_to_workspace),
-            ('move-chat-to-new-workspace', 'a{sv}', self._move_chat_to_new_workspace),
             ('mark-as-read', 'a{sv}', self._mark_as_read),
         ]
 
@@ -117,7 +120,7 @@ class ChatListStack(Gtk.Stack):
     def get_chatlist(self, workspace_id: str) -> ChatList:
         return self._chat_lists[workspace_id]
 
-    def get_selected_chat(self) -> Optional[ChatRow]:
+    def get_selected_chat(self) -> Optional[ChatListRow]:
         chat_list = self.get_current_chat_list()
         if chat_list is None:
             return None
@@ -130,13 +133,11 @@ class ChatListStack(Gtk.Stack):
 
         return self._chat_lists[workspace_id]
 
-    def is_chat_active(self, account: str, jid: JID) -> bool:
+    def is_chat_selected(self, account: str, jid: JID) -> bool:
         chat = self.get_selected_chat()
         if chat is None:
             return False
-        if chat.account != account or chat.jid != jid:
-            return False
-        return chat.is_active
+        return chat.account == account and chat.jid == jid
 
     def _on_filter_changed(self, _filter: ChatFilter, name: str) -> None:
         chat_list = cast(ChatList, self.get_visible_child())
@@ -149,6 +150,7 @@ class ChatListStack(Gtk.Stack):
     def add_chat_list(self, workspace_id: str) -> ChatList:
         chat_list = ChatList(workspace_id)
         chat_list.connect('row-selected', self._on_row_selected)
+        chat_list.connect('chat-order-changed', self._on_chat_order_changed)
 
         self._chat_lists[workspace_id] = chat_list
         self.add_named(chat_list, workspace_id)
@@ -157,15 +159,17 @@ class ChatListStack(Gtk.Stack):
     def remove_chat_list(self, workspace_id: str) -> None:
         chat_list = self._chat_lists[workspace_id]
         self.remove(chat_list)
-        for account, jid, _, _ in chat_list.get_open_chats():
-            self.remove_chat(workspace_id, account, jid)
+        for open_chat in chat_list.get_open_chats():
+            self.remove_chat(workspace_id,
+                             open_chat['account'],
+                             open_chat['jid'])
 
         self._chat_lists.pop(workspace_id)
         chat_list.destroy()
 
     def _on_row_selected(self,
                          _chat_list: ChatList,
-                         row: Optional[ChatRow]
+                         row: Optional[ChatListRow]
                          ) -> None:
         if row is None:
             self.emit('chat-unselected')
@@ -173,22 +177,32 @@ class ChatListStack(Gtk.Stack):
 
         self.emit('chat-selected', row.workspace_id, row.account, row.jid)
 
+    def _on_chat_order_changed(self,
+                               chat_list: ChatList
+                               ) -> None:
+
+        self.store_open_chats(chat_list.workspace_id)
+
     def show_chat_list(self, workspace_id: str) -> None:
         cur_workspace_id = self.get_visible_child_name()
-        if cur_workspace_id == workspace_id:
-            return
-
         if cur_workspace_id != 'default' and cur_workspace_id is not None:
             self._chat_lists[cur_workspace_id].unselect_all()
 
         self.set_visible_child_name(workspace_id)
 
-    def add_chat(self, workspace_id: str, account: str, jid: JID, type_: str,
-                 pinned: bool = False) -> None:
+    def add_chat(self,
+                 workspace_id: str,
+                 account: str,
+                 jid: JID,
+                 type_: str,
+                 pinned: bool,
+                 position: int
+                 ) -> None:
+
         chat_list = self._chat_lists.get(workspace_id)
         if chat_list is None:
             chat_list = self.add_chat_list(workspace_id)
-        chat_list.add_chat(account, jid, type_, pinned)
+        chat_list.add_chat(account, jid, type_, pinned, position)
 
     def select_chat(self, account: str, jid: JID) -> None:
         chat_list = self.find_chat(account, jid)
@@ -202,7 +216,7 @@ class ChatListStack(Gtk.Stack):
         chat_list = self._chat_lists[workspace_id]
         open_chats = chat_list.get_open_chats()
         app.settings.set_workspace_setting(
-            workspace_id, 'open_chats', open_chats)
+            workspace_id, 'chats', open_chats)
 
     @structs.actionmethod
     def _toggle_chat_pinned(self,
@@ -220,23 +234,22 @@ class ChatListStack(Gtk.Stack):
                                 params: structs.ChatListEntryParam
                                 ) -> None:
 
-        current_chatlist = cast(ChatList, self.get_visible_child())
-        type_ = current_chatlist.get_chat_type(params.account, params.jid)
+        workspace_id = params.workspace_id
+        if not workspace_id:
+            workspace_id = app.window.add_workspace(switch=False)
+
+        source_chatlist = self.get_chatlist(params.source_workspace_id)
+        type_ = source_chatlist.get_chat_type(params.account, params.jid)
         if type_ is None:
             return
-        current_chatlist.remove_chat(params.account, params.jid)
 
-        new_chatlist = self.get_chatlist(params.workspace_id)
-        new_chatlist.add_chat(params.account, params.jid, type_)
-        self.store_open_chats(current_chatlist.workspace_id)
-        self.store_open_chats(params.workspace_id)
+        source_chatlist.remove_chat(params.account, params.jid)
 
-    @structs.actionmethod
-    def _move_chat_to_new_workspace(self,
-                                    _action: Gio.SimpleAction,
-                                    params: structs.AccountJidParam
-                                    ) -> None:
-        app.window.move_chat_to_new_workspace(params.account, params.jid)
+        new_chatlist = self.get_chatlist(workspace_id)
+        new_chatlist.add_chat(params.account, params.jid, type_, False, -1)
+
+        self.store_open_chats(source_chatlist.workspace_id)
+        self.store_open_chats(workspace_id)
 
     @structs.actionmethod
     def _mark_as_read(self,
@@ -249,9 +262,39 @@ class ChatListStack(Gtk.Stack):
     def remove_chat(self, workspace_id: str, account: str, jid: JID) -> None:
         chat_list = self._chat_lists[workspace_id]
         type_ = chat_list.get_chat_type(account, jid)
-        chat_list.remove_chat(account, jid, emit_unread=False)
-        self.store_open_chats(workspace_id)
-        self.emit('chat-removed', account, jid, type_)
+
+        def _leave(not_ask_again: bool) -> None:
+            if not_ask_again:
+                app.settings.set('confirm_close_muc', False)
+            _remove()
+
+        def _remove() -> None:
+            chat_list.remove_chat(account, jid, emit_unread=False)
+            self.store_open_chats(workspace_id)
+            self.emit('chat-removed', account, jid, type_)
+
+        if type_ != 'groupchat' or not app.settings.get('confirm_close_muc'):
+            _remove()
+            return
+
+        client = app.get_client(account)
+        contact = client.get_module('Contacts').get_contact(jid, groupchat=True)
+
+        if contact.is_not_joined:
+            _remove()
+            return
+
+        ConfirmationCheckDialog(
+            _('Leave Group Chat'),
+            _('Are you sure you want to leave this group chat?'),
+            _('If you close this chat, you will leave '
+              '"%s".') % contact.name,
+            _('_Do not ask me again'),
+            [DialogButton.make('Cancel'),
+             DialogButton.make('Accept',
+                               text=_('_Leave'),
+                               callback=_leave)],
+            transient_for=app.window).show()
 
     def remove_chats_for_account(self, account: str) -> None:
         for workspace_id, chat_list in self._chat_lists.items():
@@ -305,10 +348,7 @@ class ChatListStack(Gtk.Stack):
         for chat_list in self._chat_lists.values():
             chat_list.mark_as_read(account, jid)
 
-    def process_event(self, event: events.ApplicationEvent) -> None:
-        if not isinstance(event, HANDLED_EVENTS):
-            return
-
+    def _on_event(self, event: events.ChatListEventT) -> None:
         jid = JID.from_string(event.jid)
 
         chat_list = self.find_chat(event.account, jid)
