@@ -12,10 +12,12 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import annotations
+
 from typing import Any
 from typing import Literal
 from typing import Optional
-from typing import Generator
+from typing import TYPE_CHECKING
 
 import logging
 
@@ -28,17 +30,17 @@ from gi.repository import GObject
 from nbxmpp import JID
 
 from gajim.common import app
-from gajim.common.events import ApplicationEvent
 
 from .builder import get_builder
 from .chat_filter import ChatFilter
 from .chat_list import ChatList
 from .chat_list_stack import ChatListStack
 from .chat_stack import ChatStack
-from .control_stack import ControlStack
+from .menus import get_start_chat_button_menu
 from .search_view import SearchView
-from .types import ControlT
 
+if TYPE_CHECKING:
+    from .control import ChatControl
 
 log = logging.getLogger('gajim.gui.chat_page')
 
@@ -61,7 +63,7 @@ class ChatPage(Gtk.Box):
         self._chat_stack = ChatStack()
         self._ui.right_grid_overlay.add(self._chat_stack)
 
-        self._control_stack = self._chat_stack.get_control_stack()
+        self._chat_control = self._chat_stack.get_chat_control()
 
         self._search_view = SearchView()
         self._search_view.connect('hide-search', self._on_search_hide)
@@ -89,13 +91,14 @@ class ChatPage(Gtk.Box):
                                       self._on_chat_list_changed)
         self._ui.chat_list_scrolled.add(self._chat_list_stack)
 
-        self._ui.start_chat_button.connect('clicked',
-                                           self._on_start_chat_clicked)
+        self._ui.start_chat_menu_button.set_menu_model(
+            get_start_chat_button_menu())
 
         self._ui.paned.set_position(app.settings.get('chat_handle_position'))
         self._ui.paned.connect('button-release-event', self._on_button_release)
 
         self._startup_finished: bool = False
+        self._closed_chat_memory: list[tuple[str, JID, str]] = []
 
         self._add_actions()
 
@@ -121,12 +124,8 @@ class ChatPage(Gtk.Box):
     def get_chat_list_stack(self) -> ChatListStack:
         return self._chat_list_stack
 
-    def get_control_stack(self) -> ControlStack:
-        return self._control_stack
-
-    @staticmethod
-    def _on_start_chat_clicked(_button: Gtk.Button) -> None:
-        app.app.activate_action('start-chat', GLib.Variant('s', ''))
+    def get_chat_stack(self) -> ChatStack:
+        return self._chat_stack
 
     @staticmethod
     def _on_button_release(paned: Gtk.Paned, event: Gdk.EventButton) -> None:
@@ -152,7 +151,10 @@ class ChatPage(Gtk.Box):
                           jid: JID) -> None:
 
         self._chat_stack.show_chat(account, jid)
-        self._search_view.set_context(account, jid)
+
+        if self._search_revealer.get_reveal_child():
+            self._search_view.set_context(account, jid)
+
         self.emit('chat-selected', workspace_id, account, jid)
 
     def _on_chat_unselected(self, _chat_list_stack: ChatListStack) -> None:
@@ -163,10 +165,10 @@ class ChatPage(Gtk.Box):
                            _action: Gio.SimpleAction,
                            _param: Literal[None]) -> None:
 
-        control = self.get_active_control()
-        if control is not None:
-            self._search_view.set_context(control.account, control.contact.jid)
-        self._search_view.clear()
+        if self._chat_control.has_active_chat():
+            self._search_view.set_context(self._chat_control.contact.account,
+                                          self._chat_control.contact.jid)
+
         self._search_revealer.set_reveal_child(True)
         self._search_view.set_focus()
 
@@ -183,10 +185,7 @@ class ChatPage(Gtk.Box):
                                                   'name')
         self._ui.workspace_label.set_text(name)
         self._ui.search_entry.set_text('')
-
-    def process_event(self, event: ApplicationEvent):
-        self._chat_stack.process_event(event)
-        self._chat_list_stack.process_event(event)
+        self._ui.chat_list_scrolled.get_vadjustment().set_value(0)
 
     def add_chat_list(self, workspace_id: str) -> None:
         self._chat_list_stack.add_chat_list(workspace_id)
@@ -223,47 +222,71 @@ class ChatPage(Gtk.Box):
                                jid: JID,
                                type_: str,
                                pinned: bool = False,
-                               select: bool = False) -> None:
+                               position: int = -1,
+                               select: bool = False,
+                               message: Optional[str] = None) -> None:
+
+        client = app.get_client(account)
+
+        if type_ == 'chat':
+            client.get_module('Contacts').add_chat_contact(jid)
+
+        elif type_ == 'groupchat':
+            client.get_module('Contacts').add_group_chat_contact(jid)
+
+        elif type_ == 'pm':
+            client.get_module('Contacts').add_private_contact(jid)
+
         if self.chat_exists(account, jid):
             if select:
                 self._chat_list_stack.select_chat(account, jid)
             return
 
-        if type_ == 'groupchat':
-            self._control_stack.add_group_chat(account, jid)
-        elif type_ == 'pm':
-            if not self._startup_finished:
-                # TODO: Currently we can’t load private chats at start
-                # because the Contacts dont exist yet
-                return
-            self._control_stack.add_private_chat(account, jid)
-        else:
-            self._control_stack.add_chat(account, jid)
         self._chat_list_stack.add_chat(workspace_id, account, jid, type_,
-                                       pinned)
+                                       pinned, position)
 
         if self._startup_finished:
             if select:
                 self._chat_list_stack.select_chat(account, jid)
             self._chat_list_stack.store_open_chats(workspace_id)
+            if message is not None:
+                message_input = self._chat_stack.get_message_input()
+                message_input.insert_text(message)
 
     def load_workspace_chats(self, workspace_id: str) -> None:
         open_chats = app.settings.get_workspace_setting(workspace_id,
-                                                        'open_chats')
+                                                        'chats')
 
         active_accounts = app.settings.get_active_accounts()
-        for account, jid, type_, pinned in open_chats:
+        for open_chat in open_chats:
+            account = open_chat['account']
             if account not in active_accounts:
                 continue
 
             self.add_chat_for_workspace(workspace_id,
                                         account,
-                                        jid,
-                                        type_,
-                                        pinned=pinned)
+                                        open_chat['jid'],
+                                        open_chat['type'],
+                                        pinned=open_chat['pinned'],
+                                        position=open_chat['position'])
 
-    def is_chat_active(self, account: str, jid: JID) -> bool:
-        return self._chat_list_stack.is_chat_active(account, jid)
+    def is_chat_selected(self, account: str, jid: JID) -> bool:
+        return self._chat_list_stack.is_chat_selected(account, jid)
+
+    def restore_chat(self) -> None:
+        if not self._closed_chat_memory:
+            return
+
+        account, jid, workspace_id = self._closed_chat_memory.pop()
+
+        client = app.get_client(account)
+        contact = client.get_module('Contacts').get_contact(jid)
+
+        self.add_chat_for_workspace(workspace_id,
+                                    account,
+                                    jid,
+                                    contact.type_string,
+                                    select=True)
 
     def _remove_chat(self,
                      _action: Gio.SimpleAction,
@@ -277,13 +300,21 @@ class ChatPage(Gtk.Box):
     def remove_chat(self, account: str, jid: JID) -> None:
         for workspace_id in app.settings.get_workspaces():
             if self.chat_exists_for_workspace(workspace_id, account, jid):
+                self._closed_chat_memory.append((account, jid, workspace_id))
                 self._chat_list_stack.remove_chat(workspace_id, account, jid)
                 return
 
-    def _on_chat_removed(self, _chat_list: ChatList, account: str, jid: JID,
-                         type_: str) -> None:
-        self._control_stack.remove_chat(account, jid)
-        if type_ == 'groupchat':
+    def _on_chat_removed(self,
+                         _chat_list: ChatList,
+                         account: str,
+                         jid: JID,
+                         type_: str
+                         ) -> None:
+
+        if self._chat_control.is_loaded(account, jid):
+            self._chat_control.clear()
+
+        if type_ == 'groupchat' and app.account_is_connected(account):
             client = app.get_client(account)
             client.get_module('MUC').leave(jid)
 
@@ -293,23 +324,12 @@ class ChatPage(Gtk.Box):
             chat_list.unselect_all()
 
         self._chat_list_stack.remove_chats_for_account(account)
-        self._control_stack.remove_chats_for_account(account)
+        if self._chat_control.has_active_chat():
+            if self._chat_control.contact.account == account:
+                self._chat_control.clear()
 
-    def get_control(self, account: str, jid: JID) -> Optional[ControlT]:
-        return self._control_stack.get_control(account, jid)
-
-    def get_active_control(self) -> Optional[ControlT]:
-        chat = self._chat_list_stack.get_selected_chat()
-        if chat is None:
-            return None
-        return self.get_control(chat.account, chat.jid)
-
-    def is_chat_loaded(self, account: str, jid: JID) -> bool:
-        return self._control_stack.is_chat_loaded(account, jid)
-
-    def get_controls(self, account: Optional[str]
-                     ) -> Generator[ControlT, None, None]:
-        return self._control_stack.get_controls(account)
+    def get_control(self) -> ChatControl:
+        return self._chat_control
 
     def hide_search(self) -> bool:
         if self._search_revealer.get_reveal_child():
