@@ -29,6 +29,7 @@ from gi.repository import Gtk
 from gi.repository import Gdk
 
 from gajim.common import app
+from gajim.common import passwords
 from gajim.common.i18n import _
 
 
@@ -86,6 +87,11 @@ class Features(Gtk.ApplicationWindow):
         self.feature_listbox.add(item)
 
     def _get_features(self) -> list[Feature]:
+        tray_icon_available = bool(
+            sys.platform == 'win32' or
+            app.is_installed('APPINDICATOR') or
+            app.is_installed('AYATANA_APPINDICATOR'))
+        av_available = app.is_installed('AV') and sys.platform != 'win32'
         notification_sounds_available: bool = (
             app.is_installed('GSOUND') or sys.platform in ('win32', 'darwin'))
         notification_sounds_enabled: bool = app.settings.get('sounds_on')
@@ -95,25 +101,29 @@ class Features(Gtk.ApplicationWindow):
         auto_status_enabled = bool(any(auto_status))
 
         return [
-            Feature(_('App Indicator Icon'),
-                    app.is_installed('APPINDICATOR') or
-                    app.is_installed('AYATANA_APPINDICATOR'),
-                    _('Enables Gajim to provide a system notification area '
-                      'icon'),
+            Feature(_('Notification Area Icon'),
+                    tray_icon_available,
+                    _('Enables Gajim to provide a notification area icon'),
                     _('Requires: libappindicator3'),
-                    _('Feature not available under Windows'),
+                    _('No additional requirements'),
                     None),
-            Feature(_('Audio / Video'),
-                    app.is_installed('AV'),
+            Feature(_('Audio Preview'),
+                    app.is_installed('GST'),
+                    _('Enables Gajim to provide a Audio preview'),
+                    _('Requires: gstreamer-1.0, gst-plugins-base-1.0'),
+                    _('No additional requirements'),
+                    None),
+            Feature(_('Audio / Video Calls'),
+                    av_available,
                     _('Enables Gajim to provide Audio and Video chats'),
-                    _('Requires: gir1.2-farstream-0.2, gir1.2-gstreamer-1.0, '
-                      'gstreamer1.0-plugins-base, gstreamer1.0-plugins-ugly, '
-                      'gstreamer1.0-libav, and gstreamer1.0-gtk3'),
-                    _('Feature not available under Windows'),
+                    _('Requires: farstream-0.2, gstreamer-1.0, '
+                      'gst-plugins-base-1.0, gst-plugins-ugly-1.0, '
+                      'gst-libav and gstreamer-gtk3-plugin'),
+                    _('Feature not available on Windows'),
                     None),
             Feature(_('Automatic Status'),
                     self._idle_available(),
-                    _('Enables Gajim to measure your computer\'s idle time in '
+                    _('Enables Gajim to measure your computer’s idle time in '
                       'order to set your Status automatically'),
                     _('Requires: libxss'),
                     _('No additional requirements'),
@@ -125,34 +135,28 @@ class Features(Gtk.ApplicationWindow):
                     _('No additional requirements'),
                     notification_sounds_enabled),
             Feature(_('Secure Password Storage'),
-                    self._some_keyring_available(),
+                    passwords.is_keyring_available(),
                     _('Enables Gajim to store Passwords securely instead of '
                       'storing them in plaintext'),
                     _('Requires: gnome-keyring or kwallet'),
-                    _('Windows Credential Vault is used for secure password '
-                      'storage'),
+                    _('No additional requirements'),
                     app.settings.get('use_keyring')),
             Feature(_('Spell Checker'),
                     app.is_installed('GSPELL'),
                     _('Enables Gajim to spell check your messages while '
                       'composing'),
                     _('Requires: Gspell'),
-                    _('Requires: Gspell'),
+                    _('No additional requirements'),
                     spell_check_enabled),
             Feature(_('UPnP-IGD Port Forwarding'),
                     app.is_installed('UPNP'),
                     _('Enables Gajim to request your router to forward ports '
                       'for file transfers'),
-                    _('Requires: gir1.2-gupnpigd-1.0'),
-                    _('Feature not available under Windows'),
+                    _('Requires: gupnpigd-1.0'),
+                    _('Feature not available on Windows'),
                     None)
         ]
 
-    @staticmethod
-    def _some_keyring_available() -> bool:
-        from gajim.common import passwords
-        return passwords.KEYRING_AVAILABLE
-
     @staticmethod
     def _idle_available() -> bool:
         from gajim.common import idle
