@@ -21,6 +21,8 @@ from typing import cast
 from typing import Optional
 from typing import Union
 
+from datetime import datetime
+import textwrap
 from urllib.parse import quote
 
 from gi.repository import Gtk
@@ -31,39 +33,42 @@ from nbxmpp import JID
 
 from gajim.common import app
 from gajim.common import types
+from gajim.common.helpers import from_one_line
 from gajim.common.helpers import is_affiliation_change_allowed
+from gajim.common.helpers import is_retraction_allowed
 from gajim.common.helpers import is_role_change_allowed
 from gajim.common.helpers import jid_is_blocked
-from gajim.common.helpers import reduce_chars_newlines
 from gajim.common.i18n import _
+from gajim.common.i18n import Q_
 from gajim.common.i18n import get_short_lang_code
 from gajim.common.const import URIType
-from gajim.common.const import URIAction
+from gajim.common.const import XmppUriQuery
+from gajim.common.preview import Preview
 from gajim.common.structs import URI
 from gajim.common.structs import VariantMixin
+from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import can_add_to_roster
 
 from gajim.gui.structs import AddChatActionParams
 from gajim.gui.structs import AccountJidParam
 from gajim.gui.structs import ChatListEntryParam
 from gajim.gui.structs import RemoveHistoryActionParams
+from gajim.gui.structs import RetractMessageParam
 from gajim.gui.util import GajimMenu
-from gajim.gui.const import ControlType
 
 
 MenuValueT = Union[None, str, GLib.Variant, VariantMixin]
 MenuItemListT = list[tuple[str, str, MenuValueT]]
 
 
-def get_self_contact_menu(control_id: str,
-                          contact: types.BareContact) -> GajimMenu:
+def get_self_contact_menu(contact: types.BareContact) -> GajimMenu:
     account = contact.account
     jid = contact.jid
 
     menu = GajimMenu()
 
     menu.add_item(_('Profile'), f'app.{account}-profile', f'"{account}"')
-    submenu = get_send_file_submenu(control_id)
+    submenu = get_send_file_submenu()
     menu.append_submenu(_('Send File'), submenu)
 
     params = RemoveHistoryActionParams(account=account, jid=jid)
@@ -72,60 +77,57 @@ def get_self_contact_menu(control_id: str,
     return menu
 
 
-def get_singlechat_menu(control_id: str,
-                        contact: Union[types.BareContact,
-                                       types.GroupchatParticipant]
-                        ) -> GajimMenu:
+def get_singlechat_menu(contact: types.BareContact) -> GajimMenu:
+    account = contact.account
+
+    value = f'"{contact.jid}"'
 
     menu = GajimMenu()
-    menu.add_item(_('Details'), f'win.information-{control_id}')
-    menu.add_item(_('Block Contact…'), f'win.block-contact-{control_id}')
+    menu.add_item(_('Details'), 'win.show-contact-info')
+    menu.add_item(_('Block Contact…'), f'app.{account}-block-contact', value)
 
-    submenu = get_send_file_submenu(control_id)
+    submenu = get_send_file_submenu()
     menu.append_submenu(_('Send File'), submenu)
 
-    menu.add_item(_('Start Voice Call…'), f'win.start-voice-call-{control_id}')
-    menu.add_item(_('Start Video Call…'), f'win.start-video-call-{control_id}')
+    menu.add_item(_('Start Voice Call…'), 'win.start-voice-call')
+    menu.add_item(_('Start Video Call…'), 'win.start-video-call')
+
+    menu.add_item(_('Search…'), 'win.search-history', None)
+
+    if can_add_to_roster(contact):
+        params = AccountJidParam(account=account, jid=contact.jid)
+        menu.add_item(_('Add to Contact List…'), 'win.add-to-roster', params)
+
+    return menu
+
+
+def get_private_chat_menu(contact: types.GroupchatParticipant) -> GajimMenu:
+    menu = GajimMenu()
+    menu.add_item(_('Details'), 'win.show-contact-info')
+    menu.add_item(_('Upload File…'), 'win.send-file-httpupload([""])')
     menu.add_item(_('Search…'), 'win.search-history')
 
     if can_add_to_roster(contact):
-        menu.append(_('Add to Contact List…'),
-                    f'win.add-to-roster-{control_id}')
+        params = AccountJidParam(account=contact.account, jid=contact.jid)
+        menu.add_item(_('Add to Contact List…'), 'win.add-to-roster', params)
 
     return menu
 
 
-def get_private_chat_menu(control_id: str,
-                          contact: types.GroupchatParticipant
-                          ) -> GajimMenu:
-
-    menu = GajimMenu()
-    menu.add_item(_('Details'), f'win.information-{control_id}')
-    menu.add_item(_('Upload File…'), f'win.send-file-httpupload-{control_id}')
-    menu.add_item(_('Search…'), 'win.search-history')
-
-    if can_add_to_roster(contact):
-        menu.append(_('Add to Contact List…'),
-                    f'win.add-to-roster-{control_id}')
-
-    return menu
-
-
-def get_send_file_submenu(control_id: str) -> GajimMenu:
+def get_send_file_submenu() -> GajimMenu:
     menu = GajimMenu()
 
-    menu.add_item(_('Upload File…'), f'win.send-file-httpupload-{control_id}')
-    menu.add_item(_('Send File Directly…'),
-                  f'win.send-file-jingle-{control_id}')
+    menu.add_item(_('Upload File…'), 'win.send-file-httpupload([""])')
+    menu.add_item(_('Send File Directly…'), 'win.send-file-jingle([""])')
     return menu
 
 
-def get_groupchat_menu(control_id: str, account: str, jid: JID) -> GajimMenu:
+def get_groupchat_menu(contact: GroupchatContact) -> GajimMenu:
     menuitems: MenuItemListT = [
-        (_('Details'), f'win.groupchat-details-{control_id}', None),
-        (_('Change Nickname…'), f'win.change-nickname-{control_id}', None),
-        (_('Request Voice'), f'win.request-voice-{control_id}', None),
-        (_('Execute Command…'), f'win.execute-command-{control_id}', '""'),
+        (_('Details'), 'win.show-contact-info', None),
+        (_('Change Nickname…'), 'win.muc-change-nickname', None),
+        (_('Request Voice'), 'win.muc-request-voice', None),
+        (_('Execute Command…'), 'win.muc-execute-command', '""'),
         (_('Search…'), 'win.search-history', None)
     ]
 
@@ -199,33 +201,27 @@ def build_accounts_menu() -> None:
         menubar.insert_submenu(menu_position, _('Accounts'), acc_menu)
 
 
-def get_encryption_menu(control_id: str,
-                        control_type: ControlType,
-                        ) -> Optional[GajimMenu]:
-    menu = GajimMenu()
-    action = f'win.set-encryption-{control_id}'
-    menu.add_item(_('Disabled'), action, '"disabled"')
-    for name, plugin in app.plugin_manager.encryption_plugins.items():
-        if control_type.is_groupchat:
-            if not hasattr(plugin, 'allow_groupchat'):
-                continue
-        if control_type.is_privatechat:
-            if not hasattr(plugin, 'allow_privatechat'):
-                continue
-        menu.add_item(name, action, f'"{name}"')
+def get_encryption_menu() -> GajimMenu:
 
-    if menu.get_n_items() == 1:
-        return None
-    return menu
+    menuitems: MenuItemListT = [
+        (_('Disabled'), 'win.set-encryption', '""'),
+        ('OMEMO', 'win.set-encryption', '"OMEMO"'),
+        ('OpenPGP', 'win.set-encryption', '"OpenPGP"'),
+        ('PGP', 'win.set-encryption', '"PGP"'),
+    ]
+
+    return GajimMenu.from_list(menuitems)
 
 
 def get_conv_action_context_menu(account: str,
                                  selected_text: str
                                  ) -> Gtk.MenuItem:
-    selected_text_short = reduce_chars_newlines(selected_text, 10, 1)
+    selected_text_short = textwrap.shorten(selected_text,
+                                           width=10,
+                                           placeholder='…')
 
     action_menu_item = Gtk.MenuItem.new_with_mnemonic(
-        _('_Actions for "%s"') % selected_text_short)
+        _('_Actions for "%s"') % escape_mnemonic(selected_text_short))
     submenu = Gtk.Menu()
     action_menu_item.set_submenu(submenu)
 
@@ -279,7 +275,7 @@ def get_conv_action_context_menu(account: str,
     submenu.append(item)
 
     item = Gtk.MenuItem.new_with_mnemonic(_('Open as _Link'))
-    value = GLib.Variant.new_strv([account, uri_text])
+    value = GLib.Variant.new_strv([account, f'https://{uri_text}'])
     item.set_action_target_value(value)
     submenu.append(item)
 
@@ -291,7 +287,7 @@ def get_conv_action_context_menu(account: str,
 
 def get_conv_uri_context_menu(account: str, uri: URI) -> Optional[Gtk.Menu]:
     if uri.type == URIType.XMPP:
-        if uri.action == URIAction.JOIN:
+        if XmppUriQuery.from_str(uri.query_type) == XmppUriQuery.JOIN:
             context_menu = [
                 ('copy-text', _('Copy XMPP Address')),
                 ('groupchat-join', _('Join Groupchat')),
@@ -371,15 +367,24 @@ def get_roster_menu(account: str, jid: str, gateway: bool = False) -> GajimMenu:
     value = f'"{jid}"'
 
     menuitems: MenuItemListT = [
-        (_('Details'), f'win.contact-info-{account}', value),
-        (_('Execute Command…'), f'win.execute-command-{account}', value),
-        (block_label, f'win.block-contact-{account}', value),
-        (_('Remove…'), f'win.remove-contact-{account}', value),
+        (_('Details'), f'app.{account}-contact-info', value),
+        (_('Execute Command…'), f'app.{account}-execute-command', value),
+        (block_label, f'app.{account}-block-contact', value),
+        (_('Remove…'), f'app.{account}-remove-contact', value),
     ]
 
     if gateway:
         menuitems.insert(
-            1, (_('Modify Gateway…'), f'win.modify-gateway-{account}', value))
+            1, (_('Modify Gateway…'), f'app.{account}-modify-gateway', value))
+
+    return GajimMenu.from_list(menuitems)
+
+
+def get_roster_view_menu() -> GajimMenu:
+    menuitems: MenuItemListT = [
+        (_('Show Offline Contacts'), 'win.show-offline', None),
+        (_('Sort by Status'), 'win.sort-by-show', None),
+    ]
 
     return GajimMenu.from_list(menuitems)
 
@@ -401,6 +406,16 @@ def get_subscription_menu(account: str, jid: JID) -> GajimMenu:
     return GajimMenu.from_list(menuitems)
 
 
+def get_start_chat_button_menu() -> GajimMenu:
+    menuitems: MenuItemListT = [
+        (_('Start Chat…'), 'app.start-chat(["", ""])', None),
+        (_('Create Group Chat…'), 'app.create-groupchat::', None),
+        (_('Add Contact…'), 'app.add-contact::', None),
+    ]
+
+    return GajimMenu.from_list(menuitems)
+
+
 def get_start_chat_row_menu(account: str, jid: JID) -> GajimMenu:
     params = AccountJidParam(account=account, jid=jid)
     menuitems: MenuItemListT = [
@@ -422,6 +437,7 @@ def get_chat_list_row_menu(workspace_id: str,
     menu = GajimMenu()
 
     params = ChatListEntryParam(workspace_id=workspace_id,
+                                source_workspace_id=workspace_id,
                                 account=account,
                                 jid=jid)
 
@@ -429,14 +445,17 @@ def get_chat_list_row_menu(workspace_id: str,
     menu.add_item(toggle_label, 'win.toggle-chat-pinned', params)
 
     submenu = menu.add_submenu(_('Move Chat'))
-    workspaces = app.settings.get_workspaces()
-    if len(workspaces) > 1:
+    if app.settings.get_workspace_count() > 1:
         for name, params in get_workspace_params(workspace_id, account, jid):
             submenu.add_item(name, 'win.move-chat-to-workspace', params)
 
-    params = AccountJidParam(account=account, jid=jid)
+    params = ChatListEntryParam(workspace_id='',
+                                source_workspace_id=workspace_id,
+                                account=account,
+                                jid=jid)
+
     submenu.add_item(
-        _('New Workspace'), 'win.move-chat-to-new-workspace', params)
+        _('New Workspace'), 'win.move-chat-to-workspace', params)
 
     if can_add_to_roster(contact):
         params = AccountJidParam(account=account, jid=jid)
@@ -459,18 +478,22 @@ def get_workspace_params(current_workspace_id: str,
             continue
         name = app.settings.get_workspace_setting(workspace_id, 'name')
         params = ChatListEntryParam(workspace_id=workspace_id,
+                                    source_workspace_id=current_workspace_id,
                                     account=account,
                                     jid=jid)
         yield name, params
 
 
-def get_groupchat_admin_menu(control_id: str,
-                             self_contact: types.GroupchatParticipant,
+def get_groupchat_admin_menu(self_contact: types.GroupchatParticipant,
                              contact: types.GroupchatParticipant) -> GajimMenu:
 
     menu = GajimMenu()
 
-    action = f'win.change-affiliation-{control_id}'
+    if contact.real_jid is None:
+        menu.add_item(_('Not Available'), 'dummy', None)
+        return menu
+
+    action = 'win.muc-change-affiliation'
 
     if is_affiliation_change_allowed(self_contact, contact, 'owner'):
         value = f'["{contact.real_jid}", "owner"]'
@@ -490,7 +513,7 @@ def get_groupchat_admin_menu(control_id: str,
 
     if is_affiliation_change_allowed(self_contact, contact, 'outcast'):
         value = f'"{contact.real_jid}"'
-        menu.add_item(_('Ban…'), f'win.ban-{control_id}', value)
+        menu.add_item(_('Ban…'), 'win.muc-ban', value)
 
     if not menu.get_n_items():
         menu.add_item(_('Not Available'), 'dummy', None)
@@ -498,18 +521,21 @@ def get_groupchat_admin_menu(control_id: str,
     return menu
 
 
-def get_groupchat_mod_menu(control_id: str,
-                           self_contact: types.GroupchatParticipant,
+def get_groupchat_mod_menu(self_contact: types.GroupchatParticipant,
                            contact: types.GroupchatParticipant
                            ) -> GajimMenu:
 
     menu = GajimMenu()
 
+    if not contact.is_available:
+        menu.add_item(_('Not Available'), 'dummy', None)
+        return menu
+
     if is_role_change_allowed(self_contact, contact):
         value = f'"{contact.name}"'
-        menu.add_item(_('Kick…'), f'win.kick-{control_id}', value)
+        menu.add_item(_('Kick…'), 'win.muc-kick', value)
 
-    action = f'win.change-role-{control_id}'
+    action = 'win.muc-change-role'
 
     if is_role_change_allowed(self_contact, contact):
         if contact.role.is_visitor:
@@ -525,17 +551,16 @@ def get_groupchat_mod_menu(control_id: str,
     return menu
 
 
-def get_groupchat_roster_menu(account: str,
-                              control_id: str,
-                              self_contact: types.GroupchatParticipant,
-                              contact: types.GroupchatParticipant
-                              ) -> GajimMenu:
+def get_groupchat_participant_menu(account: str,
+                                   self_contact: types.GroupchatParticipant,
+                                   contact: types.GroupchatParticipant
+                                   ) -> GajimMenu:
 
     value = f'"{contact.name}"'
 
     general_items: MenuItemListT = [
-        (_('Details'), f'win.contact-information-{control_id}', value),
-        (_('Execute Command…'), f'win.execute-command-{control_id}', value),
+        (_('Details'), 'win.muc-contact-info', value),
+        (_('Execute Command…'), 'win.muc-execute-command', value),
     ]
 
     real_contact = contact.get_real_contact()
@@ -544,8 +569,8 @@ def get_groupchat_roster_menu(account: str,
         action = f'app.{account}-add-contact'
         general_items.insert(1, (_('Add to Contact List…'), action, value))
 
-    mod_menu = get_groupchat_mod_menu(control_id, self_contact, contact)
-    admin_menu = get_groupchat_admin_menu(control_id, self_contact, contact)
+    mod_menu = get_groupchat_mod_menu(self_contact, contact)
+    admin_menu = get_groupchat_admin_menu(self_contact, contact)
 
     menu = GajimMenu.from_list(general_items)
     menu.append_section(_('Moderation'), mod_menu)
@@ -575,6 +600,141 @@ def get_component_search_menu(jid: Optional[str], copy_text: str) -> Gio.Menu:
     return menu
 
 
+def get_chat_row_menu(contact: types.ChatContactT,
+                      name: str,
+                      text: str,
+                      timestamp: datetime,
+                      message_id: Optional[str],
+                      stanza_id: Optional[str],
+                      log_line_id: Optional[int]
+                      ) -> GajimMenu:
+
+    menu_items: MenuItemListT = []
+
+    text = text.replace('"', '\\"')
+
+    time_format = from_one_line(app.settings.get('time_format'))
+    timestamp_formatted = timestamp.strftime(time_format)
+    copy_text = f'{timestamp_formatted} - {name}: {text}'
+    menu_items.append(
+        (Q_('?Message row action:Copy'), 'win.copy-message', f'"{copy_text}"'))
+
+    menu_items.append(
+        (Q_('?Message row action:Select Messages…'),
+         'win.activate-message-selection',
+         GLib.Variant('u', log_line_id or 0)))
+
+    show_quote = True
+    if isinstance(contact, GroupchatContact):
+        if contact.is_joined:
+            self_contact = contact.get_self()
+            assert self_contact is not None
+            show_quote = not self_contact.role.is_visitor
+        else:
+            show_quote = False
+    if show_quote:
+        menu_items.append((
+            Q_('?Message row action:Quote…'),
+            'win.quote',
+            f'"{text}"'))
+
+    show_correction = False
+    if message_id is not None:
+        show_correction = app.window.is_message_correctable(
+            contact, message_id)
+    if show_correction:
+        menu_items.append((
+            Q_('?Message row action:Correct…'), 'win.correct-message', None))
+
+    show_retract = False
+    if isinstance(contact, GroupchatContact) and contact.is_joined:
+        resource_contact = contact.get_resource(name)
+        self_contact = contact.get_self()
+        assert self_contact is not None
+        is_allowed = is_retraction_allowed(self_contact, resource_contact)
+
+        disco_info = app.storage.cache.get_last_disco_info(contact.jid)
+        assert disco_info is not None
+
+        if disco_info.has_message_moderation and is_allowed:
+            show_retract = True
+    if show_retract:
+        assert stanza_id is not None
+        param = RetractMessageParam(
+            account=contact.account,
+            jid=contact.jid,
+            stanza_id=stanza_id)
+        menu_items.append((
+            Q_('?Message row action:Retract…'), 'win.retract-message', param))
+
+    menu = GajimMenu.from_list(menu_items)
+    return menu
+
+
+def get_preview_menu(preview: Preview) -> GajimMenu:
+    menu_items: MenuItemListT = []
+
+    variant = GLib.Variant('s', preview.id)
+
+    download = (_('_Download'), 'win.preview-download', variant)
+    open_file = (_('_Open'), 'win.preview-open', variant)
+    save_as = (_('_Save as…'), 'win.preview-save-as', variant)
+    open_folder = (_('Open _Folder'), 'win.preview-open-folder', variant)
+    copy_link = (_('_Copy Link'), 'win.preview-copy-link', variant)
+    open_link = (_('Open Link in _Browser'), 'win.preview-open-link', variant)
+
+    if preview.is_geo_uri:
+        menu_items.append(open_file)
+        menu_items.append(copy_link)
+        return GajimMenu.from_list(menu_items)
+
+    if preview.orig_exists:
+        menu_items.append(open_file)
+        menu_items.append(save_as)
+        menu_items.append(open_folder)
+    else:
+        if not preview.download_in_progress:
+            menu_items.append(download)
+
+    menu_items.append(copy_link)
+
+    if not preview.is_aes_encrypted:
+        menu_items.append(open_link)
+
+    return GajimMenu.from_list(menu_items)
+
+
+def get_format_menu() -> GajimMenu:
+
+    menuitems: MenuItemListT = [
+        (_('bold'), 'win.input-bold', None),
+        (_('italic'), 'win.input-italic', None),
+        (_('strike'), 'win.input-strike', None),
+    ]
+
+    return GajimMenu.from_list(menuitems)
+
+
+def get_workspace_menu(workspace_id: str, has_unread: bool) -> GajimMenu:
+    remove_action = 'win.dummy'
+    if app.settings.get_workspace_count() > 1:
+        remove_action = 'win.remove-workspace'
+
+    menuitems: MenuItemListT = [
+        (_('Edit…'), 'win.edit-workspace', f'"{workspace_id}"'),
+        (_('Remove'), remove_action, f'"{workspace_id}"'),
+    ]
+
+    if has_unread:
+        menuitems.insert(
+            0,
+            (_('Mark as read'),
+             'win.mark-workspace-as-read',
+             f'"{workspace_id}"'))
+
+    return GajimMenu.from_list(menuitems)
+
+
 def escape_mnemonic(label: Optional[str]) -> Optional[str]:
     if label is None:
         return None
