@@ -17,17 +17,26 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-from typing import Any
+from __future__ import annotations
 
-import sys
+from typing import Any
+from typing import Optional
+
+from collections import defaultdict
+
+import logging
 
 from gi.repository import Gtk
 from gi.repository import Gdk
-from gi.repository import GdkPixbuf
 from gi.repository import GLib
 from gi.repository import Pango
 
 from gajim.common import app
+from gajim.common import ged
+from gajim.common.events import MessageSent
+from gajim.common.ged import EventHelper
+from gajim.common.i18n import LANG
+from gajim.common.i18n import _
 from gajim.common.styling import process
 from gajim.common.styling import PlainBlock
 from gajim.common.types import ChatContactT
@@ -47,15 +56,16 @@ FORMAT_CHARS: dict[str, str] = {
     'pre': '`',
 }
 
+log = logging.getLogger('gajim.gui.message_input')
 
-class MessageInputTextView(Gtk.TextView):
-    """
+
+class MessageInputTextView(Gtk.TextView, EventHelper):
+    '''
     A Gtk.Textview for chat message input
-    """
-    def __init__(self, account: str, contact: ChatContactT) -> None:
+    '''
+    def __init__(self) -> None:
         Gtk.TextView.__init__(self)
-        self.account = account
-        self.contact = contact
+        EventHelper.__init__(self)
 
         self.set_border_width(3)
         self.set_accepts_tab(True)
@@ -73,6 +83,12 @@ class MessageInputTextView(Gtk.TextView):
         self._undo_list: list[str] = []
         self.undo_pressed: bool = False
 
+        self._contact: Optional[ChatContactT] = None
+
+        # XEP-0308 Message Correction
+        self._correcting: dict[ChatContactT, bool] = defaultdict(lambda: False)
+        self._last_message_id: dict[ChatContactT, Optional[str]] = {}
+
         self._chat_action_processor = ChatActionProcessor(self)
 
         self.get_buffer().create_tag('strong', weight=Pango.Weight.BOLD)
@@ -85,6 +101,160 @@ class MessageInputTextView(Gtk.TextView):
         self.connect('focus-in-event', self._on_focus_in)
         self.connect('focus-out-event', self._on_focus_out)
         self.connect('destroy', self._on_destroy)
+        self.connect('populate-popup', self._on_populate_popup)
+
+        self.register_events([
+            ('message-sent', ged.GUI2, self._on_message_sent)
+        ])
+
+        self._language_handler_id = self._init_spell_checker()
+        app.settings.connect_signal('use_speller', self._on_toggle_spell_check)
+
+        app.plugin_manager.gui_extension_point('message_input', self)
+
+    @property
+    def is_correcting(self) -> bool:
+        if self._contact is None:
+            return False
+
+        return self._correcting[self._contact]
+
+    def get_last_message_id(self, contact: ChatContactT) -> Optional[str]:
+        return self._last_message_id.get(contact)
+
+    def toggle_message_correction(self) -> None:
+        self.clear()
+        self.grab_focus()
+
+        if self.is_correcting:
+            self._set_correcting(False)
+            self.get_style_context().remove_class('gajim-msg-correcting')
+            return
+
+        assert self._contact is not None
+        last_message_id = self._last_message_id.get(self._contact)
+        if last_message_id is None:
+            return
+
+        message_row = app.storage.archive.get_last_correctable_message(
+            self._contact.account, self._contact.jid, last_message_id)
+        if message_row is None:
+            return
+
+        self._set_correcting(True)
+        self.get_style_context().add_class('gajim-msg-correcting')
+        self.insert_text(message_row.message)
+
+    def try_message_correction(self, message: str) -> Optional[str]:
+        assert self._contact is not None
+        if not self.is_correcting:
+            return None
+
+        self.toggle_message_correction()
+
+        correct_id = self._last_message_id.get(self._contact)
+        message_row = app.storage.archive.get_last_correctable_message(
+            self._contact.account, self._contact.jid, correct_id)
+        if message_row is None:
+            log.info('Trying to correct message older than threshold')
+            return None
+
+        if message_row.message == message:
+            log.info('Trying to correct message with original text')
+            return None
+
+        return correct_id
+
+    def _set_correcting(self, state: bool) -> None:
+        assert self._contact is not None
+        self._correcting[self._contact] = state
+
+    def _on_message_sent(self, event: MessageSent) -> None:
+        if not event.message:
+            return
+
+        if event.correct_id is None:
+            # This wasn't a corrected message
+            assert self._contact is not None
+            oob_url = event.additional_data.get_value('gajim', 'oob_url')
+            if oob_url == event.message:
+                # Don't allow to correct HTTP Upload file transfer URLs
+                self._last_message_id[self._contact] = None
+            else:
+                self._last_message_id[self._contact] = event.message_id
+
+    def _init_spell_checker(self) -> int:
+        if not app.is_installed('GSPELL'):
+            return 0
+
+        checker = Gspell.Checker.new(Gspell.language_get_default())
+
+        buffer = Gspell.TextBuffer.get_from_gtk_text_buffer(self.get_buffer())
+        buffer.set_spell_checker(checker)
+
+        view = Gspell.TextView.get_from_gtk_text_view(self)
+        view.set_enable_language_menu(True)
+
+        self._on_toggle_spell_check()
+
+        return checker.connect('notify::language', self._on_language_changed)
+
+    def _set_spell_checker_language(self, contact: ChatContactT) -> None:
+        if not app.is_installed('GSPELL'):
+            return
+
+        buffer = Gspell.TextBuffer.get_from_gtk_text_buffer(self.get_buffer())
+        checker = buffer.get_spell_checker()
+        assert checker is not None
+        lang = self._get_spell_checker_language(contact)
+
+        with checker.handler_block(self._language_handler_id):
+            checker.set_language(lang)
+
+    def _get_spell_checker_language(self,
+                                    contact: ChatContactT
+                                    ) -> Optional[Gspell.Language]:
+
+        lang = contact.settings.get('speller_language')
+        if not lang:
+            # use the default one
+            lang = app.settings.get('speller_language')
+            if not lang:
+                lang = LANG
+
+        assert isinstance(lang, str)
+        lang = Gspell.language_lookup(lang)
+        if lang is None:
+            lang = Gspell.language_get_default()
+        return lang
+
+    def _on_language_changed(self,
+                             checker: Gspell.Checker,
+                             _param: Any) -> None:
+
+        gspell_lang = checker.get_language()
+        if gspell_lang is not None:
+            assert self._contact is not None
+            self._contact.settings.set('speller_language',
+                                       gspell_lang.get_code())
+
+    def switch_contact(self, contact: ChatContactT) -> None:
+        if self._contact is not None:
+            app.storage.drafts.set(self._contact, self.get_text())
+
+        self.clear()
+        draft = app.storage.drafts.get(contact)
+        if draft is not None:
+            self.insert_text(draft)
+
+        self._contact = contact
+        if self.is_correcting:
+            self.get_style_context().add_class('gajim-msg-correcting')
+        else:
+            self.get_style_context().remove_class('gajim-msg-correcting')
+
+        self._chat_action_processor.switch_contact(contact)
+        self._set_spell_checker_language(contact)
 
     def _on_destroy(self, _widget: Gtk.Widget) -> None:
         # We restore the TextView’s drag destination to avoid a GTK warning
@@ -103,7 +273,7 @@ class MessageInputTextView(Gtk.TextView):
                      _widget: Gtk.Widget,
                      _event: Gdk.EventFocus
                      ) -> bool:
-        self.toggle_speller(True)
+
         scrolled = self.get_parent()
         assert scrolled
         scrolled.get_style_context().add_class('message-input-focus')
@@ -113,11 +283,10 @@ class MessageInputTextView(Gtk.TextView):
                       _widget: Gtk.Widget,
                       _event: Gdk.EventFocus
                       ) -> bool:
+
         scrolled = self.get_parent()
         assert scrolled
         scrolled.get_style_context().remove_class('message-input-focus')
-        if not self.has_text():
-            self.toggle_speller(False)
         return False
 
     def _clear_tags(self) -> None:
@@ -151,8 +320,10 @@ class MessageInputTextView(Gtk.TextView):
         for block in result.blocks:
             if isinstance(block, PlainBlock):
                 for span in block.spans:
-                    start_iter = buf.get_iter_at_offset(span.start)
-                    end_iter = buf.get_iter_at_offset(span.end)
+                    start_iter = buf.get_iter_at_offset(
+                        span.start + block.start)
+                    end_iter = buf.get_iter_at_offset(
+                        span.end + block.start)
                     buf.apply_tag_by_name(span.name, start_iter, end_iter)
 
     def insert_text(self, text: str) -> None:
@@ -166,6 +337,7 @@ class MessageInputTextView(Gtk.TextView):
         if buf.get_end_iter().equal(iter_):
             GLib.idle_add(scroll_to_end, self.get_parent())
 
+    @property
     def has_text(self) -> bool:
         buf = self.get_buffer()
         start, end = buf.get_bounds()
@@ -178,10 +350,13 @@ class MessageInputTextView(Gtk.TextView):
         text = self.get_buffer().get_text(start, end, True)
         return text
 
-    def toggle_speller(self, activate: bool) -> None:
-        if app.is_installed('GSPELL') and app.settings.get('use_speller'):
-            spell_view = Gspell.TextView.get_from_gtk_text_view(self)
-            spell_view.set_inline_spell_checking(activate)
+    def _on_toggle_spell_check(self, *args: Any) -> None:
+        if not app.is_installed('GSPELL'):
+            return
+
+        use_spell_check = app.settings.get('use_speller')
+        spell_view = Gspell.TextView.get_from_gtk_text_view(self)
+        spell_view.set_inline_spell_checking(use_spell_check)
 
     @staticmethod
     def _after_paste_clipboard(textview: Gtk.TextView) -> None:
@@ -243,64 +418,11 @@ class MessageInputTextView(Gtk.TextView):
             buf.get_iter_at_offset(start_offset),
             buf.get_iter_at_offset(end_offset + 2))
 
-    def replace_emojis(self) -> None:
-        if sys.platform != 'darwin':
-            # TODO: Remove if colored emoji rendering works well on MacOS
-            return
-
-        theme = app.settings.get('emoticons_theme')
-        if not theme or theme == 'font':
-            return
-
-        def _replace(anchor: Gtk.TextChildAnchor) -> None:
-            if anchor is None:
-                return
-            image = anchor.get_widgets()[0]
-            if hasattr(image, 'codepoint'):
-                # found emoji
-                codepoint = getattr(image, 'codepoint')
-                self._replace_char_at_iter(iter_, codepoint)
-                image.destroy()
-
-        iter_ = self.get_buffer().get_start_iter()
-        _replace(iter_.get_child_anchor())
-
-        while iter_.forward_char():
-            _replace(iter_.get_child_anchor())
-
-    def _replace_char_at_iter(self,
-                              iter_: Gtk.TextIter,
-                              new_char: str
-                              ) -> None:
-        buf = self.get_buffer()
-        iter_2 = iter_.copy()
-        iter_2.forward_char()
-        buf.delete(iter_, iter_2)
-        buf.insert(iter_, new_char)
-
-    def insert_emoji(self, codepoint: str, pixbuf: GdkPixbuf.Pixbuf) -> None:
-        buf = self.get_buffer()
-        if buf.get_char_count():
-            # buffer contains text
-            buf.insert_at_cursor(' ')
-
-        insert_mark = buf.get_insert()
-        insert_iter = buf.get_iter_at_mark(insert_mark)
-
-        if pixbuf is None:
-            buf.insert(insert_iter, codepoint)
-        else:
-            anchor = buf.create_child_anchor(insert_iter)
-            image = Gtk.Image.new_from_pixbuf(pixbuf)
-            setattr(image, 'codepoint', codepoint)
-            image.show()
-            self.add_child_at_anchor(image, anchor)
-        buf.insert_at_cursor(' ')
-
     def clear(self, *args: Any) -> None:
         buf = self.get_buffer()
         start, end = buf.get_bounds()
         buf.delete(start, end)
+        self._undo_list.clear()
 
     def save_undo(self, text: str) -> None:
         self._undo_list.append(text)
@@ -313,3 +435,60 @@ class MessageInputTextView(Gtk.TextView):
         if self._undo_list:
             buf.set_text(self._undo_list.pop())
         self.undo_pressed = True
+
+    def _on_populate_popup(self,
+                           _textview: MessageInputTextView,
+                           menu: Gtk.Widget
+                           ) -> None:
+        assert isinstance(menu, Gtk.Menu)
+        item = Gtk.MenuItem.new_with_mnemonic(_('_Undo'))
+        menu.prepend(item)
+        item.connect('activate', self.undo)
+
+        item = Gtk.SeparatorMenuItem()
+        menu.prepend(item)
+
+        item = Gtk.MenuItem.new_with_mnemonic(_('_Clear'))
+        menu.prepend(item)
+        item.connect('activate', self.clear)
+
+        paste_code_block_item = Gtk.MenuItem.new_with_label(
+            _('Paste as Code Block'))
+        paste_code_block_item.connect(
+            'activate', self._paste_clipboard_as_code_block)
+        menu.append(paste_code_block_item)
+
+        paste_item = Gtk.MenuItem.new_with_label(_('Paste as Quote'))
+        paste_item.connect('activate', self._paste_clipboard_as_quote)
+        menu.append(paste_item)
+
+        menu.show_all()
+
+    def mention_participant(self, name: str) -> None:
+        gc_refer_to_nick_char = app.settings.get('gc_refer_to_nick_char')
+        text = f'{name}{gc_refer_to_nick_char} '
+        self.insert_text(text)
+        self.grab_focus()
+
+    def insert_as_quote(self, text: str) -> None:
+        text = '> ' + text.replace('\n', '\n> ') + '\n'
+        self.insert_text(text)
+        self.grab_focus()
+
+    def insert_as_code_block(self, text: str) -> None:
+        self.insert_text(f'```\n{text}\n```')
+        self.grab_focus()
+
+    def _paste_clipboard_as_quote(self, _item: Gtk.MenuItem) -> None:
+        clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+        text = clipboard.wait_for_text()
+        if text is None:
+            return
+        self.insert_as_quote(text)
+
+    def _paste_clipboard_as_code_block(self, _item: Gtk.MenuItem) -> None:
+        clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+        text = clipboard.wait_for_text()
+        if text is None:
+            return
+        self.insert_as_code_block(text)
