@@ -17,9 +17,6 @@ from typing import Any
 from typing import Optional
 
 import logging
-import shutil
-import os
-from pathlib import Path
 
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
@@ -28,24 +25,33 @@ from gi.repository import Gtk
 from gi.repository import Pango
 
 from gajim.common import app
-from gajim.common.helpers import open_file
-from gajim.common.helpers import open_uri
 from gajim.common.i18n import _
 from gajim.common.preview import Preview
 from gajim.common.preview_helpers import split_geo_uri
+from gajim.common.preview_helpers import format_geo_coords
 from gajim.common.preview_helpers import contains_audio_streams
 from gajim.common.preview_helpers import get_icon_for_mime_type
 from gajim.common.types import GdkPixbufType
 
-from .dialogs import ErrorDialog
-from .filechoosers import FileSaveDialog
 from .preview_audio import AudioWidget
 from .builder import get_builder
+from .menus import get_preview_menu
+from .util import GajimPopover
+from .util import ensure_not_destroyed
 from .util import get_cursor
 from .util import load_icon_pixbuf
 
 log = logging.getLogger('gajim.gui.preview')
 
+PREVIEW_ACTIONS: dict[str, tuple[str, str]] = {
+    'open': (_('Open'), 'preview-open'),
+    'save_as': (_('Save as…'), 'preview-save-as'),
+    'open_folder': (_('Open Folder'), 'preview-open-folder'),
+    'copy_link_location': (_('Copy Link'), 'preview-copy-link'),
+    'open_link_in_browser': (_('Open Link in Browser'), 'preview-open-link'),
+    'download': (_('Download File'), 'preview-download'),
+}
+
 
 class PreviewWidget(Gtk.Box):
     def __init__(self, account: str) -> None:
@@ -53,7 +59,7 @@ class PreviewWidget(Gtk.Box):
         self.account = account
         self._preview: Optional[Preview] = None
 
-        self._in_progress = False
+        self._destroyed = False
 
         if app.settings.get('use_kib_mib'):
             self._units = GLib.FormatSizeFlags.IEC_UNITS
@@ -63,16 +69,31 @@ class PreviewWidget(Gtk.Box):
         self._ui = get_builder('preview.ui')
         self._ui.connect_signals(self)
         self.add(self._ui.preview_box)
+
+        leftclick_action = app.settings.get('preview_leftclick_action')
+        self._ui.icon_button.set_tooltip_text(
+            PREVIEW_ACTIONS[leftclick_action][0])
+        app.settings.connect_signal(
+            'preview_leftclick_action', self._update_icon_button_tooltip)
+
+        self.connect('destroy', self._on_destroy)
+
         self.show_all()
 
+    def _on_destroy(self, _widget: Gtk.Widget) -> None:
+        self._destroyed = True
+
+    def _update_icon_button_tooltip(self, setting: str, *args: Any) -> None:
+        self._ui.icon_button.set_tooltip_text(
+            PREVIEW_ACTIONS[setting][0])
+
     def get_text(self) -> str:
         if self._preview is None:
             return ''
         return self._preview.uri
 
+    @ensure_not_destroyed
     def update_progress(self, _preview: Preview, progress: float) -> None:
-        self._in_progress = True
-
         self._ui.download_button.hide()
 
         self._ui.progress_box.show()
@@ -81,10 +102,10 @@ class PreviewWidget(Gtk.Box):
         self._ui.info_message.set_text(_('Downloading…'))
         self._ui.info_message.set_tooltip_text('')
 
+    @ensure_not_destroyed
     def update(self, preview: Preview, data: Optional[GdkPixbufType]) -> None:
         self._preview = preview
 
-        self._in_progress = False
         self._ui.progress_box.hide()
         self._ui.info_message.hide()
 
@@ -112,15 +133,14 @@ class PreviewWidget(Gtk.Box):
             self._ui.download_button.hide()
 
             location = split_geo_uri(preview.uri)
-            self._ui.file_size.set_text(_('Lat: %s Lon: %s') % (
-                location.lat, location.lon))
+            text = format_geo_coords(float(location.lat), float(location.lon))
+            self._ui.file_size.set_text(text)
             self._ui.image_button.set_tooltip_text(
-                _('Location at Lat: %s Lon: %s') % (
-                    location.lat, location.lon))
+                _('Location at %s') % text)
             self._ui.preview_box.set_size_request(160, -1)
             return
 
-        if preview.is_previewable and preview.orig_exists():
+        if preview.is_previewable and preview.orig_exists:
             self._ui.icon_event_box.hide()
             self._ui.image_button.show()
             self._ui.save_as_button.show()
@@ -147,7 +167,7 @@ class PreviewWidget(Gtk.Box):
             self._ui.info_message.set_tooltip_text(preview.info_message)
             self._ui.info_message.show()
 
-        if preview.orig_exists():
+        if preview.orig_exists:
             self._ui.download_button.hide()
             self._ui.open_folder_button.show()
             self._ui.save_as_button.show()
@@ -180,132 +200,45 @@ class PreviewWidget(Gtk.Box):
         self._ui.file_name.set_text(preview.filename)
         self._ui.file_name.set_tooltip_text(preview.filename)
 
-    def _get_context_menu(self) -> Gtk.Menu:
-        def _destroy(menu: Gtk.Menu, _pspec: Any) -> None:
-            visible = menu.get_property('visible')
-            if not visible:
-                GLib.idle_add(menu.destroy)
-
-        menu = get_builder('preview_context_menu.ui')
-        menu.connect_signals(self)
-        menu.context_menu.connect('notify::visible', _destroy)
-
-        assert self._preview
-        if self._preview.is_aes_encrypted:
-            menu.open_link_in_browser.hide()
-
-        if self._preview.is_geo_uri:
-            menu.download.hide()
-            menu.open_link_in_browser.hide()
-            menu.save_as.hide()
-            menu.open_folder.hide()
-            return menu.context_menu
-
-        if self._preview.orig_exists():
-            menu.download.hide()
-        else:
-            if self._in_progress:
-                menu.download.hide()
-            menu.open.hide()
-            menu.save_as.hide()
-            menu.open_folder.hide()
-
-        return menu.context_menu
-
-    def _on_download(self, _menu: Gtk.Menu) -> None:
-        if self._preview is None:
-            return
-        if not self._preview.orig_exists():
-            app.preview_manager.download_content(self._preview, force=True)
-
-    def _on_open(self, _menu: Gtk.Menu) -> None:
+    def _on_download(self, _button: Gtk.Button) -> None:
         if self._preview is None:
             return
 
-        if self._preview.is_geo_uri:
-            open_uri(self._preview.uri)
-            return
+        variant = GLib.Variant('s', self._preview.id)
+        app.window.activate_action('preview-download', variant)
 
-        if not self._preview.orig_exists():
-            app.preview_manager.download_content(self._preview, force=True)
-            return
-
-        assert self._preview.orig_path
-        open_file(self._preview.orig_path)
-
-    def _on_save_as(self, _menu: Gtk.Menu) -> None:
+    def _on_save_as(self, _button: Gtk.Button) -> None:
         assert self._preview is not None
+        variant = GLib.Variant('s', self._preview.id)
+        app.window.activate_action('preview-save-as', variant)
 
-        def _on_ok(target: str) -> None:
-            assert self._preview is not None
-            assert self._preview.orig_path is not None
-
-            target_path = Path(target)
-            orig_ext = self._preview.orig_path.suffix
-            new_ext = target_path.suffix
-            if orig_ext != new_ext:
-                # Windows file chooser selects the full file name including
-                # extension. Starting to type will overwrite the extension
-                # as well. Restore the extension if it's lost.
-                target_path = target_path.with_suffix(orig_ext)
-            dirname = target_path.parent
-            if not os.access(dirname, os.W_OK):
-                ErrorDialog(
-                    _('Directory "%s" is not writable') % dirname,
-                    _('You do not have the proper permissions to '
-                      'create files in this directory.'),
-                    transient_for=app.app.get_active_window())
-                return
-            shutil.copyfile(str(self._preview.orig_path), target_path)
-
-        if not self._preview.orig_exists():
-            app.preview_manager.download_content(self._preview, force=True)
-            return
-
-        FileSaveDialog(_on_ok,
-                       path=app.settings.get('last_save_dir'),
-                       file_name=self._preview.filename,
-                       transient_for=app.app.get_active_window())
-
-    def _on_open_folder(self, _menu: Gtk.Menu) -> None:
-        assert self._preview
-        if not self._preview.orig_exists():
-            app.preview_manager.download_content(self._preview, force=True)
-            return
-        assert self._preview.orig_path
-        open_file(self._preview.orig_path.parent)
-
-    def _on_copy_link_location(self, _menu: Gtk.Menu) -> None:
-        display = Gdk.Display.get_default()
-        assert display is not None
-        clipboard = Gtk.Clipboard.get_default(display)
-        assert self._preview
-        clipboard.set_text(self._preview.uri, -1)
-
-    def _on_open_link_in_browser(self, _menu: Gtk.Menu) -> None:
-        assert self._preview
-        if self._preview.is_aes_encrypted:
-            if self._preview.is_geo_uri:
-                open_uri(self._preview.uri)
-                return
-            assert self._preview.orig_path
-            open_file(self._preview.orig_path)
-        else:
-            open_uri(self._preview.uri)
+    def _on_open_folder(self, _button: Gtk.Button) -> None:
+        assert self._preview is not None
+        variant = GLib.Variant('s', self._preview.id)
+        app.window.activate_action('preview-open-folder', variant)
 
     def _on_content_button_clicked(self, _button: Gtk.Button) -> None:
-        action = app.settings.get('preview_leftclick_action')
-        method = getattr(self, f'_on_{action}')
-        method(None)
+        if self._preview is None:
+            return
+
+        leftclick_action = app.settings.get('preview_leftclick_action')
+        variant = GLib.Variant('s', self._preview.id)
+        action = PREVIEW_ACTIONS[leftclick_action][1]
+        app.window.activate_action(action, variant)
 
     def _on_button_press_event(self,
                                _button: Gtk.Button,
                                event: Gdk.EventButton
                                ) -> None:
-        if event.type == Gdk.EventType.BUTTON_PRESS and event.button == 3:
-            # Right click
-            menu = self._get_context_menu()
-            menu.popup_at_pointer(event)
+
+        if self._preview is None:
+            return
+
+        if (event.type == Gdk.EventType.BUTTON_PRESS and
+                event.button == Gdk.BUTTON_SECONDARY):
+            menu = get_preview_menu(self._preview)
+            popover = GajimPopover(menu, relative_to=self, event=event)
+            popover.popup()
 
     def _on_cancel_download_clicked(self, _button: Gtk.Button) -> None:
         assert self._preview is not None
