@@ -55,7 +55,7 @@ class AudioWidget(Gtk.Box):
     def _build_audio_widget(self) -> None:
         play_button = Gtk.Button()
         play_button.get_style_context().add_class('flat')
-        play_button.get_style_context().add_class('preview-button')
+        play_button.get_style_context().add_class('border')
         play_button.set_tooltip_text(_('Start/stop playback'))
         self._play_icon = Gtk.Image.new_from_icon_name(
             'media-playback-start-symbolic',
@@ -116,7 +116,7 @@ class AudioWidget(Gtk.Box):
             is_paused = self._get_paused()
             if (duration > 0 and not is_paused and
                     not self._has_timeout):
-                GLib.timeout_add(500, self._update_seek_bar)
+                GLib.timeout_add(50, self._update_seek_bar)
                 self._has_timeout = True
 
     def _on_seek(self,
