@@ -12,30 +12,44 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
+from typing import Any
+from typing import cast
+
 import logging
 
 from gi.repository import Gtk
 
 from nbxmpp.errors import StanzaError
+from nbxmpp.task import Task
 
 from gajim.common import app
 from gajim.common import ged
+from gajim.common.events import AccountConnected
+from gajim.common.events import AccountDisconnected
 from gajim.common.i18n import _
 from gajim.common.helpers import to_user_string
 from gajim.common.helpers import event_filter
 
 from .assistant import Assistant
 from .assistant import Page
+from .assistant import ProgressPage
+from .assistant import ErrorPage
+from .assistant import SuccessPage
 
 log = logging.getLogger('gajim.gui.remove_account')
 
 
 class RemoveAccount(Assistant):
-    def __init__(self, account):
+    def __init__(self, account: str) -> None:
         Assistant.__init__(self)
 
         self.account = account
-        self._con = app.connections.get(account)
+
+        try:
+            self._client = app.get_client(account)
+        except KeyError:
+            self._client = None
+
         self._destroyed = False
         self._account_removed = False
 
@@ -43,31 +57,21 @@ class RemoveAccount(Assistant):
         self.add_button('close', _('Close'))
         self.add_button('back', _('Back'))
 
-        self.add_pages({'remove_choice': RemoveChoice(account)})
+        self.add_pages({'remove_choice': RemoveChoice(account),
+                        'error': Error(),
+                        'success': Success()})
 
-        progress = self.add_default_page('progress')
+        progress = cast(ProgressPage, self.add_default_page('progress'))
         progress.set_title(_('Removing Account...'))
         progress.set_text(_('Trying to remove account...'))
 
-        success = self.add_default_page('success')
-        success.set_title(_('Account Removed'))
-        success.set_heading(_('Account Removed'))
-        success.set_text(
-            _('Your account has has been removed successfully.'))
-
-        error = self.add_default_page('error')
-        error.set_title(_('Account Removal Failed'))
-        error.set_heading(_('Account Removal Failed'))
-
-        self.set_button_visible_func(self._visible_func)
-
         self.connect('button-clicked', self._on_button_clicked)
-        self.connect('page-changed', self._on_page_changed)
         self.connect('destroy', self._on_destroy)
 
         self.register_events([
-            ('account-connected', ged.GUI1, self._on_account_connected),
-            ('account-disconnected', ged.GUI1, self._on_account_disconnected),
+            ('account-connected', ged.POSTGUI, self._on_account_connected),
+            ('account-disconnected', ged.POSTGUI,
+             self._on_account_disconnected),
         ])
 
         self._set_remove_from_server_checkbox()
@@ -75,38 +79,27 @@ class RemoveAccount(Assistant):
         self.show_all()
 
     @event_filter(['account'])
-    def _on_account_connected(self, event):
-        self._con = app.connections.get(self.account)
+    def _on_account_connected(self, _event: AccountConnected) -> None:
+        self._client = app.get_client(self.account)
         self._set_remove_from_server_checkbox()
 
     @event_filter(['account'])
-    def _on_account_disconnected(self, event):
+    def _on_account_disconnected(self, _event: AccountDisconnected) -> None:
         self._set_remove_from_server_checkbox()
 
         if self._account_removed:
             self.show_page('success')
-            app.interface.remove_account(self.account)
+            app.app.remove_account(self.account)
 
-    def _set_remove_from_server_checkbox(self):
-        enabled = self._con is not None and self._con.state.is_available
-        self.get_page('remove_choice').set_remove_from_server(enabled)
+    def _set_remove_from_server_checkbox(self) -> None:
+        enabled = self._client is not None and self._client.state.is_available
+        remove_choice_page = cast(RemoveChoice, self.get_page('remove_choice'))
+        remove_choice_page.set_remove_from_server(enabled)
 
-    @staticmethod
-    def _visible_func(_assistant, page_name):
-        if page_name == 'remove_choice':
-            return ['remove']
-
-        if page_name == 'progress':
-            return None
-
-        if page_name == 'error':
-            return ['back']
-
-        if page_name == 'success':
-            return ['close']
-        raise ValueError('page %s unknown' % page_name)
-
-    def _on_button_clicked(self, _assistant, button_name):
+    def _on_button_clicked(self,
+                           _assistant: Assistant,
+                           button_name: str
+                           ) -> None:
         page = self.get_current_page()
         if button_name == 'remove':
             if page == 'remove_choice':
@@ -123,50 +116,44 @@ class RemoveAccount(Assistant):
         if button_name == 'close':
             self.destroy()
 
-    def _on_page_changed(self, _assistant, page_name):
-        if page_name == 'remove_choice':
-            self.set_default_button('remove')
-
-        elif page_name == 'success':
-            self.set_default_button('close')
-
-        elif page_name == 'error':
-            self.set_default_button('back')
-
-    def _on_remove(self, *args):
-        if self.get_page('remove_choice').remove_from_server:
-            self._con.set_remove_account(True)
-            self._con.get_module('Register').unregister(
+    def _on_remove(self, *args: Any) -> None:
+        remove_choice_page = cast(RemoveChoice, self.get_page('remove_choice'))
+        if remove_choice_page.remove_from_server:
+            assert self._client is not None
+            self._client.set_remove_account(True)
+            self._client.get_module('Register').unregister(
                 callback=self._on_remove_response)
             return
 
-        if self._con is None or self._con.state.is_disconnected:
-            app.interface.remove_account(self.account)
+        if self._client is None or self._client.state.is_disconnected:
+            app.app.remove_account(self.account)
             self.show_page('success')
             return
 
-        self._con.disconnect(gracefully=True, reconnect=False)
+        self._client.disconnect(gracefully=True, reconnect=False)
         self._account_removed = True
 
-    def _on_remove_response(self, task):
+    def _on_remove_response(self, task: Task) -> None:
         try:
             task.finish()
         except StanzaError as error:
-            self._con.set_remove_account(False)
+            assert self._client is not None
+            self._client.set_remove_account(False)
 
             error_text = to_user_string(error)
-            self.get_page('error').set_text(error_text)
+            error_page = cast(Error, self.get_page('error'))
+            error_page.set_text(error_text)
             self.show_page('error')
             return
 
         self._account_removed = True
 
-    def _on_destroy(self, *args):
+    def _on_destroy(self, *args: Any) -> None:
         self._destroyed = True
 
 
 class RemoveChoice(Page):
-    def __init__(self, account):
+    def __init__(self, account: str) -> None:
         Page.__init__(self)
         self.title = _('Remove Account')
 
@@ -205,13 +192,38 @@ class RemoveChoice(Page):
         self.show_all()
 
     @property
-    def remove_from_server(self):
+    def remove_from_server(self) -> bool:
         return self._server.get_active()
 
-    def set_remove_from_server(self, enabled):
+    def set_remove_from_server(self, enabled: bool) -> None:
         self._server.set_sensitive(enabled)
         if enabled:
             self._server.set_tooltip_text('')
         else:
             self._server.set_active(False)
             self._server.set_tooltip_text(_('Account has to be connected'))
+
+    def get_visible_buttons(self) -> list[str]:
+        return ['remove']
+
+
+class Error(ErrorPage):
+    def __init__(self) -> None:
+        ErrorPage.__init__(self)
+        self.set_title(_('Account Removal Failed'))
+        self.set_heading(_('Account Removal Failed'))
+
+    def get_visible_buttons(self) -> list[str]:
+        return ['back']
+
+
+class Success(SuccessPage):
+    def __init__(self) -> None:
+        SuccessPage.__init__(self)
+        self.set_title(_('Account Removed'))
+        self.set_heading(_('Account Removed'))
+        self.set_text(
+            _('Your account has has been removed successfully.'))
+
+    def get_visible_buttons(self) -> list[str]:
+        return ['close']
