@@ -78,11 +78,10 @@ class SearchView(Gtk.Box):
         app.ged.register_event_handler('account-disabled',
                                        ged.GUI1,
                                        self._on_account_state)
-        self._update_calendar()
         self.show_all()
 
     def _on_account_state(self, _event: Any) -> None:
-        self.clear()
+        self._clear()
 
     @staticmethod
     def _header_func(row: ResultRow, before: ResultRow) -> None:
@@ -100,17 +99,23 @@ class SearchView(Gtk.Box):
 
     def _on_hide_clicked(self, _button: Gtk.Button) -> None:
         self.emit('hide-search')
-        self.clear()
+        self._clear()
 
-    def clear(self) -> None:
+    def _clear(self) -> None:
         self._ui.search_entry.set_text('')
         self._clear_results()
 
     def _clear_results(self) -> None:
+        # Unset the header_func to reduce load when clearing
+        self._ui.results_listbox.set_header_func(None)
+
         for row in self._ui.results_listbox.get_children():
             self._ui.results_listbox.remove(row)
             row.destroy()
 
+        self._ui.results_listbox.set_header_func(self._header_func)
+        self._ui.results_scrolled.get_vadjustment().set_value(0)
+
     def _on_search(self, entry: Gtk.Entry) -> None:
         self._clear_results()
         self._ui.date_hint.hide()
@@ -196,7 +201,7 @@ class SearchView(Gtk.Box):
                 result_row = ResultRow(
                     msg,
                     accounts[msg.account_id],
-                    archive_jid.jid)
+                    JID.from_string(archive_jid.jid))
             else:
                 assert self._account is not None
                 assert self._jid is not None
@@ -307,8 +312,8 @@ class SearchView(Gtk.Box):
         self._ui.calendar.select_day(date.day)
 
     def _scroll_to_date(self, date: datetime) -> None:
-        control = app.window.get_active_control()
-        if control is None:
+        control = app.window.get_control()
+        if not control.has_active_chat():
             return
         if control.contact.jid == self._jid:
             meta_row = app.storage.archive.get_first_message_meta_for_date(
@@ -336,22 +341,23 @@ class SearchView(Gtk.Box):
 
     @staticmethod
     def _on_row_activated(_listbox: SearchView, row: ResultRow) -> None:
-        control = app.window.get_active_control()
-        if control is not None:
+        control = app.window.get_control()
+        if control.has_active_chat():
             if control.contact.jid == row.jid:
                 control.scroll_to_message(row.log_line_id, row.timestamp)
                 return
 
-        # Wrong chat or no control opened
-        # TODO: type 'pm' is KindConstant.CHAT_MSG_RECV, too
+        # Other chat or no control opened
         jid = JID.from_string(row.jid)
         app.window.add_chat(row.account, jid, row.type, select=True)
-        control = app.window.get_active_control()
-        if control is not None:
+        control = app.window.get_control()
+        if control.has_active_chat():
             control.scroll_to_message(row.log_line_id, row.timestamp)
 
     def set_focus(self) -> None:
         self._ui.search_entry.grab_focus()
+        self._clear()
+        self._update_calendar()
 
     def set_context(self, account: Optional[str], jid: Optional[JID]) -> None:
         self._account = account
@@ -400,11 +406,13 @@ class ResultRow(Gtk.ListBoxRow):
         self.type = 'contact'
         if msg.kind == KindConstant.GC_MSG:
             self.type = 'groupchat'
+        if jid.is_full:
+            self.type = 'pm'
 
         self.contact = self._client.get_module('Contacts').get_contact(
             jid, groupchat=self.type == 'groupchat')
 
-        self.get_style_context().add_class('conversation-row')
+        self.get_style_context().add_class('search-view-row')
         self._ui = get_builder('search_view.ui')
         self.add(self._ui.result_row_grid)
 
