@@ -75,16 +75,16 @@ class GCTooltip:
         return False, self._ui.tooltip_grid
 
     def _hide_grid_children(self) -> None:
-        """
+        '''
         Hide all Elements of the Tooltip Grid
-        """
+        '''
         for child in self._ui.tooltip_grid.get_children():
             child.hide()
 
     def _populate_grid(self, contact: types.GroupchatParticipant) -> None:
-        """
+        '''
         Populate the Tooltip Grid with data of from the contact
-        """
+        '''
         self._hide_grid_children()
 
         self._ui.nick.set_text(contact.name)
@@ -118,7 +118,7 @@ class GCTooltip:
         self._ui.avatar.show()
         self._ui.fillelement.show()
 
-        app.plugin_manager.gui_extension_point(
+        app.plugin_manager.extension_point(
             'gc_tooltip_populate', self, contact, self._ui.tooltip_grid)
 
     def destroy(self) -> None:
@@ -177,7 +177,7 @@ class RosterTooltip:
 
         self._append_pep_info(contact)
 
-        app.plugin_manager.gui_extension_point(
+        app.plugin_manager.extension_point(
             'roster_tooltip_populate', self, contact)
 
         # This sets the bottom-most widget to expand, in case the avatar
@@ -206,6 +206,8 @@ class RosterTooltip:
             show_image.set_halign(Gtk.Align.START)
             show_image.set_valign(Gtk.Align.CENTER)
 
+            show_string = helpers.get_uf_show(resource.show.value)
+
             assert resource.jid.resource is not None
             resource_string = GLib.markup_escape_text(resource.jid.resource)
             resource_label = Gtk.Label()
@@ -213,7 +215,7 @@ class RosterTooltip:
             resource_label.set_xalign(0)
             resource_label.set_ellipsize(Pango.EllipsizeMode.END)
             resource_label.set_max_width_chars(30)
-            resource_label.set_text(resource_string)
+            resource_label.set_text(f'{show_string} ({resource_string})')
 
             base_box = Gtk.Box(spacing=6)
             base_box.add(show_image)
@@ -243,7 +245,7 @@ class RosterTooltip:
                 idle_label.set_xalign(0)
                 resource_box.add(idle_label)
 
-            app.plugin_manager.gui_extension_point(
+            app.plugin_manager.extension_point(
                 'roster_tooltip_resource_populate',
                 resource_box,
                 resource)
@@ -327,11 +329,11 @@ class FileTransfersTooltip:
         properties.append((Q_('?transfer type:Type: '), type_))
         properties.append((actor, GLib.markup_escape_text(name)))
 
-        transfered_len = file_prop.received_len
-        if not transfered_len:
-            transfered_len = 0
+        transferred_len = file_prop.received_len
+        if not transferred_len:
+            transferred_len = 0
         properties.append((Q_('?transfer status:Transferred: '),
-                           GLib.format_size_full(transfered_len, self.units)))
+                           GLib.format_size_full(transferred_len, self.units)))
         status = self._get_current_status(file_prop)
         properties.append((Q_('?transfer status:Status: '), status))
         file_desc = file_prop.desc or ''
