@@ -23,6 +23,7 @@ from typing import Union
 
 import logging
 import math
+import sys
 import textwrap
 from io import BytesIO
 from importlib import import_module
@@ -31,14 +32,11 @@ from functools import lru_cache
 from pathlib import Path
 from re import Match
 
-try:
-    from PIL import Image
-except Exception:
-    pass
-
+from PIL import Image
 from gi.repository import Gdk
 from gi.repository import Gio
 from gi.repository import Gtk
+from gi.repository import GtkSource
 from gi.repository import GLib
 from gi.repository import Pango
 from gi.repository import GdkPixbuf
@@ -50,12 +48,14 @@ from nbxmpp.structs import TuneData
 
 from gajim.common import app
 from gajim.common import configpaths
+from gajim.common import types
 from gajim.common.i18n import _
 from gajim.common.helpers import URL_REGEX
 from gajim.common.const import LOCATION_DATA
 from gajim.common.const import Display
 from gajim.common.const import StyleAttr
 from gajim.common.ged import EventHelper as CommonEventHelper
+from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.styling import PlainBlock
 from gajim.common.structs import VariantMixin
 
@@ -69,89 +69,6 @@ MenuValueT = Union[None, str, GLib.Variant, VariantMixin]
 MenuItemListT = list[tuple[str, str, MenuValueT]]
 
 
-class NickCompletionGenerator:
-    def __init__(self, self_nick: str) -> None:
-        self.nick = self_nick
-        self.sender_list: list[str] = []
-        self.attention_list: list[str] = []
-
-    def change_nick(self, new_nick: str) -> None:
-        self.nick = new_nick
-
-    def record_message(self, contact: str, highlight: bool) -> None:
-        if contact == self.nick:
-            return
-
-        log.debug('Recorded a message from %s, highlight; %s', contact,
-                  highlight)
-        if highlight:
-            try:
-                self.attention_list.remove(contact)
-            except ValueError:
-                pass
-            if len(self.attention_list) > 6:
-                self.attention_list.pop(0)  # remove older
-            self.attention_list.append(contact)
-
-        # TODO implement it in a more efficient way
-        # Currently it's O(n*m + n*s), where n is the number of participants and
-        # m is the number of messages processed, s - the number of times the
-        # suggestions are requested
-        #
-        # A better way to do it would be to keep a dict: contact -> timestamp
-        # with expected O(1) insert, and sort it by timestamps in O(n log n)
-        # for each suggestion (currently generating the suggestions is O(n))
-        # this would give the expected complexity of O(m + s * n log n)
-        try:
-            self.sender_list.remove(contact)
-        except ValueError:
-            pass
-        self.sender_list.append(contact)
-
-    def contact_renamed(self, contact_old: str, contact_new: str) -> None:
-        log.debug('Contact %s renamed to %s', contact_old, contact_new)
-        for lst in (self.attention_list, self.sender_list):
-            for idx, contact in enumerate(lst):
-                if contact == contact_old:
-                    lst[idx] = contact_new
-
-    def generate_suggestions(self, nicks: list[str],
-                             beginning: str) -> list[str]:
-        """
-        Generate the order of suggested MUC autocompletions
-
-        `nicks` is the list of contacts currently participating in a MUC
-        `beginning` is the text already typed by the user
-        """
-        def nick_matching(nick: str) -> bool:
-            return nick != self.nick \
-                and nick.lower().startswith(beginning.lower())
-
-        if beginning == '':
-            # empty message, so just suggest recent mentions
-            potential_matches = self.attention_list
-        else:
-            # nick partially typed, try completing it
-            potential_matches = self.sender_list
-
-        potential_matches_set = set(potential_matches)
-        log.debug('Priority matches: %s', potential_matches_set)
-
-        matches = [n for n in potential_matches if nick_matching(n)]
-        # the most recent nick is the last one on the list
-        matches.reverse()
-
-        # handle people who have not posted/mentioned us
-        other_nicks = [
-            n for n in nicks
-            if nick_matching(n) and n not in potential_matches_set
-        ]
-        other_nicks.sort(key=str.lower)
-        log.debug('Other matches: %s', other_nicks)
-
-        return matches + other_nicks
-
-
 def set_urgency_hint(window: Gtk.Window, setting: bool) -> None:
     if app.settings.get('use_urgency_hint'):
         window.set_urgency_hint(setting)
@@ -191,8 +108,8 @@ def load_icon_surface(
         icon_name: str,
         size: int = 16,
         scale: Optional[int] = None,
-        flags: Gtk.IconLookupFlags = Gtk.IconLookupFlags.FORCE_SIZE,
-        ) -> Optional[cairo.Surface]:
+        flags: Gtk.IconLookupFlags = Gtk.IconLookupFlags.FORCE_SIZE
+) -> Optional[cairo.ImageSurface]:
 
     icon_info = load_icon_info(icon_name, size, scale, flags)
     if icon_info is None:
@@ -203,7 +120,7 @@ def load_icon_surface(
 def load_icon_pixbuf(icon_name: str,
                      size: int = 16,
                      scale: Optional[int] = None,
-                     flags: Gtk.IconLookupFlags = Gtk.IconLookupFlags.FORCE_SIZE,
+                     flags: Gtk.IconLookupFlags = Gtk.IconLookupFlags.FORCE_SIZE
                      ) -> Optional[GdkPixbuf.Pixbuf]:
 
     icon_info = load_icon_info(icon_name, size, scale, flags)
@@ -224,9 +141,6 @@ def get_app_icon_list(scale_widget: Gtk.Widget) -> list[GdkPixbuf.Pixbuf]:
 
 def get_icon_name(name: str,
                   transport: Optional[str] = None) -> str:
-    if name == 'not in roster':
-        name = 'notinroster'
-
     if transport is not None:
         return f'{transport}-{name}'
 
@@ -264,9 +178,9 @@ def get_total_screen_geometry() -> tuple[int, int]:
 
 
 def resize_window(window: Gtk.Window, width: int, height: int) -> None:
-    """
+    '''
     Resize window, but also checks if huge window or negative values
-    """
+    '''
     screen_w, screen_h = get_total_screen_geometry()
     if not width or not height:
         return
@@ -277,9 +191,9 @@ def resize_window(window: Gtk.Window, width: int, height: int) -> None:
 
 
 def move_window(window: Gtk.Window, pos_x: int, pos_y: int) -> None:
-    """
+    '''
     Move the window, but also check if out of screen
-    """
+    '''
     screen_w, screen_h = get_total_screen_geometry()
     pos_x = max(pos_x, 0)
     pos_y = max(pos_y, 0)
@@ -313,11 +227,18 @@ def restore_main_window_position() -> None:
                 app.settings.get('mainwin_y_position'))
 
 
+def get_source_view_style_scheme() -> Optional[GtkSource.StyleScheme]:
+    style_scheme_manager = GtkSource.StyleSchemeManager.get_default()
+    if app.css_config.prefer_dark:
+        return style_scheme_manager.get_scheme('solarized-dark')
+    return style_scheme_manager.get_scheme('solarized-light')
+
+
 def get_completion_liststore(entry: Gtk.Entry) -> Gtk.ListStore:
-    """
+    '''
     Create a completion model for entry widget completion list consists of
     (Pixbuf, Text) rows
-    """
+    '''
     completion = Gtk.EntryCompletion()
     liststore = Gtk.ListStore(str, str)
 
@@ -344,14 +265,14 @@ def get_cursor(name: str) -> Gdk.Cursor:
 
 
 def scroll_to_end(widget: Gtk.ScrolledWindow) -> bool:
-    """Scrolls to the end of a GtkScrolledWindow.
+    '''Scrolls to the end of a GtkScrolledWindow.
 
     Args:
         widget (GtkScrolledWindow)
 
     Returns:
         bool: The return value is False so it can be used with GLib.idle_add.
-    """
+    '''
     adj_v = widget.get_vadjustment()
     if adj_v is None:
         # This can happen when the Widget is already destroyed when called
@@ -366,14 +287,14 @@ def scroll_to_end(widget: Gtk.ScrolledWindow) -> bool:
 
 
 def at_the_end(widget: Gtk.ScrolledWindow) -> bool:
-    """Determines if a Scrollbar in a GtkScrolledWindow is at the end.
+    '''Determines if a Scrollbar in a GtkScrolledWindow is at the end.
 
     Args:
         widget (GtkScrolledWindow)
 
     Returns:
         bool: The return value is True if at the end, False if not.
-    """
+    '''
     adj_v = widget.get_vadjustment()
     max_scroll_pos = adj_v.get_upper() - adj_v.get_page_size()
     return adj_v.get_value() == max_scroll_pos
@@ -441,11 +362,23 @@ def get_monitor_scale_factor() -> int:
 
 
 def get_primary_accel_mod() -> Optional[Gdk.ModifierType]:
-    """
+    '''
     Returns the primary Gdk.ModifierType modifier.
     cmd on osx, ctrl everywhere else.
-    """
-    return Gtk.accelerator_parse("<Primary>")[1]
+    '''
+    return Gtk.accelerator_parse('<Primary>')[1]
+
+
+def get_copy_modifier() -> Gdk.ModifierType:
+    if sys.platform == 'darwin':
+        return Gdk.ModifierType.META_MASK
+    return Gdk.ModifierType.CONTROL_MASK
+
+
+def get_copy_modifier_keys() -> tuple[int, int]:
+    if sys.platform == 'darwin':
+        return Gdk.KEY_Meta_L, Gdk.KEY_Meta_R
+    return Gdk.KEY_Control_L, Gdk.KEY_Control_R
 
 
 def get_hardware_key_codes(keyval: int) -> list[int]:
@@ -545,6 +478,7 @@ class MultiLineLabel(Gtk.Label):
         self.set_line_wrap(True)
         self.set_line_wrap_mode(Pango.WrapMode.WORD_CHAR)
         self.set_single_line_mode(False)
+        self.set_selectable(True)
 
 
 class MaxWidthComboBoxText(Gtk.ComboBoxText):
@@ -576,6 +510,19 @@ def text_to_color(text: str) -> tuple[float, float, float]:
     return nbxmpp.util.text_to_color(text, background)  # type: ignore
 
 
+def get_contact_color(contact: types.ChatContactT
+                      ) -> tuple[float, float, float]:
+
+    if isinstance(contact, GroupchatParticipant):
+        if contact.room.muc_context in (None, 'public'):
+            return text_to_color(contact.name)
+
+        if contact.real_jid is not None:
+            return text_to_color(str(contact.real_jid))
+
+    return text_to_color(str(contact.jid))
+
+
 def get_color_for_account(account: str) -> str:
     col_r, col_g, col_b = text_to_color(account)
     rgba = Gdk.RGBA(red=col_r, green=col_g, blue=col_b)
@@ -605,9 +552,9 @@ def add_css_to_widget(widget: Any, css: str) -> None:
 
 def get_pixbuf_from_data(file_data: bytes) -> Optional[GdkPixbuf.Pixbuf]:
     # TODO: This already exists in preview_helpery pixbuf_from_data
-    """
+    '''
     Get image data and returns GdkPixbuf.Pixbuf
-    """
+    '''
     pixbufloader = GdkPixbuf.PixbufLoader()
     try:
         pixbufloader.write(file_data)
@@ -619,7 +566,7 @@ def get_pixbuf_from_data(file_data: bytes) -> Optional[GdkPixbuf.Pixbuf]:
         log.warning('loading avatar using pixbufloader failed, trying to '
                     'convert avatar image using pillow')
         try:
-            avatar = Image.open(BytesIO(file_data)).convert("RGBA")
+            avatar = Image.open(BytesIO(file_data)).convert('RGBA')
             array = GLib.Bytes.new(avatar.tobytes())  # type: ignore
             width, height = avatar.size
             pixbuf = GdkPixbuf.Pixbuf.new_from_bytes(
@@ -674,7 +621,7 @@ def load_pixbuf(path: Union[str, Path],
         try:
             with open(path, 'rb') as im_handle:
                 img = Image.open(im_handle)
-                avatar = img.convert("RGBA")
+                avatar = img.convert('RGBA')
         except (NameError, OSError):
             log.warning('Pillow convert failed: %s', path)
             log.debug('Error', exc_info=True)
@@ -801,21 +748,21 @@ def _connect_destroy(sender: Any,
                      handler: Any,
                      *args: Any,
                      **kwargs: Any) -> int:
-    """Connect a bound method to a foreign object signal and disconnect
+    '''Connect a bound method to a foreign object signal and disconnect
     if the object the method is bound to emits destroy (Gtk.Widget subclass).
     Also works if the handler is a nested function in a method and
     references the method's bound object.
     This solves the problem that the sender holds a strong reference
     to the bound method and the bound to object doesn't get GCed.
-    """
+    '''
 
-    if hasattr(handler, "__self__"):
+    if hasattr(handler, '__self__'):
         obj = handler.__self__
     else:
         # XXX: get the "self" var of the enclosing scope.
         # Used for nested functions which ref the object but aren't methods.
         # In case they don't ref "self" normal connect() should be used anyway.
-        index = handler.__code__.co_freevars.index("self")
+        index = handler.__code__.co_freevars.index('self')
         obj = handler.__closure__[index].cell_contents
 
     assert obj is not sender
@@ -851,23 +798,46 @@ def wrap_with_event_box(klass: Any) -> Any:
 
 
 class AccountBadge(Gtk.Label):
-    def __init__(self, account: str) -> None:
+    def __init__(self, account: Optional[str] = None) -> None:
         Gtk.Label.__init__(self)
         self.set_ellipsize(Pango.EllipsizeMode.END)
         self.set_max_width_chars(12)
         self.set_size_request(50, -1)
         self.get_style_context().add_class('badge')
-        self._account = account
+        self.set_no_show_all(True)
 
-        self.refresh()
-        self.show()
+        if account is not None:
+            self.set_account(account)
+            self.show()
 
-    def refresh(self) -> None:
-        label = app.get_account_label(self._account)
+    def set_account(self, account: str) -> None:
+        label = app.get_account_label(account)
         self.set_text(label)
-        account_class = app.css_config.get_dynamic_class(self._account)
-        self.get_style_context().add_class(account_class)
+
+        style_context = self.get_style_context()
+        for style_class in style_context.list_classes():
+            if style_class != 'badge':
+                style_context.remove_class(style_class)
+
+        account_class = app.css_config.get_dynamic_class(account)
+        style_context.add_class(account_class)
+
         self.set_tooltip_text(_('Account: %s') % label)
+        app.settings.disconnect_signals(self)
+        app.settings.connect_signal(
+            'account_label',
+            self._on_account_label_changed,
+            account)
+
+    def _on_account_label_changed(self,
+                                  _value: str,
+                                  _setting: str,
+                                  account: Optional[str],
+                                  *args: Any
+                                  ) -> None:
+
+        assert account is not None
+        self.set_account(account)
 
 
 def make_pango_attributes(block: PlainBlock) -> Pango.AttrList:
@@ -880,6 +850,24 @@ def make_pango_attributes(block: PlainBlock) -> Pango.AttrList:
     return attrlist
 
 
+_grapheme_buffer = Gtk.TextBuffer()
+
+
+def get_first_graphemes(text: str, n: int) -> str:
+    # This should be possible with lower-level APIs like Pango.break_* or
+    # Pango.get_log_attrs, but their Python bindings seem totally broken.
+    # The re-use of one global buffer is to mitigate very probable memory leaks.
+    _grapheme_buffer.set_text(text)
+    cursor = _grapheme_buffer.get_start_iter()
+    cursor.forward_cursor_positions(n)
+    return _grapheme_buffer.get_slice(
+        _grapheme_buffer.get_start_iter(), cursor, False)
+
+
+def get_first_grapheme(text: str) -> str:
+    return get_first_graphemes(text, 1)
+
+
 def get_style_attribute_with_name(name: str) -> Pango.Attribute:
     if name == 'strong':
         return Pango.attr_weight_new(Pango.Weight.BOLD)
@@ -943,3 +931,47 @@ class GajimMenu(Gio.Menu):
         menu = GajimMenu()
         self.append_submenu(label, menu)
         return menu
+
+
+class GdkRectangle(Gdk.Rectangle):
+    def __init__(self,
+                 x: int,
+                 y: int,
+                 height: int = 1,
+                 width: int = 1
+                 ) -> None:
+
+        Gdk.Rectangle.__init__(self)
+        self.x = x
+        self.y = y
+        self.height = height
+        self.width = width
+
+
+class GajimPopover(Gtk.Popover):
+    def __init__(self,
+                 menu: Gio.MenuModel,
+                 relative_to: Optional[Gtk.Widget] = None,
+                 position: Gtk.PositionType = Gtk.PositionType.RIGHT,
+                 event: Optional[Gdk.EventButton] = None) -> None:
+
+        Gtk.Popover.__init__(self)
+
+        self.bind_model(menu)
+        self.set_relative_to(relative_to)
+        self.set_position(position)
+        if event is not None:
+            self.set_pointing_from_event(event)
+
+        self.connect('closed', self._destroy)
+
+    def set_pointing_from_event(self, event: Gdk.EventButton) -> None:
+        self.set_pointing_to_coord(event.x, event.y)
+
+    def set_pointing_to_coord(self, x: float, y: float) -> None:
+        rectangle = GdkRectangle(x=int(x), y=int(y))
+        self.set_pointing_to(rectangle)
+
+    def _destroy(self, popover: Gtk.Popover) -> None:
+        app.check_finalize(popover)
+        GLib.idle_add(popover.set_relative_to, None)
