@@ -14,9 +14,9 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-import os
 import locale
 import gettext
+from pathlib import Path
 
 from gajim.common import configpaths
 
@@ -26,9 +26,10 @@ try:
 except KeyError:
     # This allows to import the module for tests
     print('No plugin translation path available')
-    plugin_user_dir = ''
+    plugin_user_dir = Path.cwd()
 
-plugins_locale_dir = os.path.join(plugin_user_dir, 'locale')
+
+plugins_locale_dir = plugin_user_dir / 'locale'
 
 try:
     t = gettext.translation(DOMAIN, plugins_locale_dir)
@@ -37,4 +38,4 @@ except OSError:
     _ = gettext.gettext
 
 if hasattr(locale, 'bindtextdomain'):
-    locale.bindtextdomain(DOMAIN, plugins_locale_dir)  # type: ignore
+    locale.bindtextdomain(DOMAIN, plugins_locale_dir)
